=== modified file 'breezy/builtins.py'
--- breezy/builtins.py	2020-07-18 23:14:00 +0000
+++ breezy/builtins.py	2020-08-10 15:00:17 +0000
@@ -61,7 +61,6 @@
     views,
     )
 from breezy.branch import Branch
-from breezy.conflicts import ConflictList
 from breezy.transport import memory
 from breezy.smtp_connection import SMTPConnection
 from breezy.workingtree import WorkingTree
@@ -4820,7 +4819,7 @@
             restore_files = [c.path for c in conflicts
                              if c.typestring in allowed_conflicts]
         _mod_merge.transform_tree(tree, tree.basis_tree(), interesting_files)
-        tree.set_conflicts(ConflictList(new_conflicts))
+        tree.set_conflicts(new_conflicts)
         if file_list is not None:
             restore_files = file_list
         for filename in restore_files:

=== modified file 'breezy/bzr/_dirstate_helpers_pyx.pyx'
--- breezy/bzr/_dirstate_helpers_pyx.pyx	2020-06-08 19:42:53 +0000
+++ breezy/bzr/_dirstate_helpers_pyx.pyx	2020-08-10 15:00:17 +0000
@@ -31,7 +31,7 @@
 from .. import cache_utf8, errors, osutils
 from .dirstate import DirState, DirstateCorrupt
 from ..osutils import parent_directories, pathjoin, splitpath, is_inside_any, is_inside
-from ..tree import TreeChange
+from .inventorytree import InventoryTreeChange
 
 
 # This is the Windows equivalent of ENOTDIR
@@ -1303,7 +1303,7 @@
                     else:
                         path_u = self.utf8_decode(path)[0]
                 source_kind = _minikind_to_kind(source_minikind)
-                return TreeChange(entry[0][2],
+                return InventoryTreeChange(entry[0][2],
                        (old_path_u, path_u),
                        content_change,
                        (True, True),
@@ -1336,7 +1336,7 @@
                         and S_IXUSR & path_info[3].st_mode)
                 else:
                     target_exec = target_details[3]
-                return TreeChange(entry[0][2],
+                return InventoryTreeChange(entry[0][2],
                        (None, self.utf8_decode(path)[0]),
                        True,
                        (False, True),
@@ -1346,7 +1346,7 @@
                        (None, target_exec)), True
             else:
                 # Its a missing file, report it as such.
-                return TreeChange(entry[0][2],
+                return InventoryTreeChange(entry[0][2],
                        (None, self.utf8_decode(path)[0]),
                        False,
                        (False, True),
@@ -1364,7 +1364,7 @@
             parent_id = self.state._get_entry(self.source_index, path_utf8=entry[0][0])[0][2]
             if parent_id == entry[0][2]:
                 parent_id = None
-            return TreeChange(
+            return InventoryTreeChange(
                    entry[0][2],
                    (self.utf8_decode(old_path)[0], None),
                    True,
@@ -1562,7 +1562,7 @@
                 new_executable = bool(
                     stat.S_ISREG(self.root_dir_info[3].st_mode)
                     and stat.S_IEXEC & self.root_dir_info[3].st_mode)
-                return TreeChange(
+                return InventoryTreeChange(
                        None,
                        (None, self.current_root_unicode),
                        True,
@@ -1674,7 +1674,7 @@
                             new_executable = bool(
                                 stat.S_ISREG(current_path_info[3].st_mode)
                                 and stat.S_IEXEC & current_path_info[3].st_mode)
-                            return TreeChange(
+                            return InventoryTreeChange(
                                 None,
                                 (None, self.utf8_decode(current_path_info[0])[0]),
                                 True,
@@ -1844,7 +1844,7 @@
                             if changed is not None:
                                 raise AssertionError(
                                     "result is not None: %r" % result)
-                            result = TreeChange(
+                            result = InventoryTreeChange(
                                 None,
                                 (None, relpath_unicode),
                                 True,

=== added file 'breezy/bzr/conflicts.py'
--- breezy/bzr/conflicts.py	1970-01-01 00:00:00 +0000
+++ breezy/bzr/conflicts.py	2020-08-09 18:10:01 +0000
@@ -0,0 +1,664 @@
+# Copyright (C) 2005, 2006, 2007, 2009, 2010, 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from __future__ import absolute_import
+
+import errno
+import os
+import re
+
+from ..lazy_import import lazy_import
+lazy_import(globals(), """
+
+from breezy import (
+    cache_utf8,
+    errors,
+    rio,
+    transform,
+    osutils,
+    )
+""")
+
+from ..conflicts import (
+    Conflict as BaseConflict,
+    ConflictList as BaseConflictList,
+    )
+from ..sixish import text_type
+
+
+CONFLICT_SUFFIXES = ('.THIS', '.BASE', '.OTHER')
+
+
+class Conflict(BaseConflict):
+    """Base class for all types of conflict"""
+
+    # FIXME: cleanup should take care of that ? -- vila 091229
+    has_files = False
+
+    def __init__(self, path, file_id=None):
+        super(Conflict, self).__init__(path)
+        # the factory blindly transfers the Stanza values to __init__ and
+        # Stanza is purely a Unicode api.
+        if isinstance(file_id, text_type):
+            file_id = cache_utf8.encode(file_id)
+        self.file_id = file_id
+
+    def as_stanza(self):
+        s = rio.Stanza(type=self.typestring, path=self.path)
+        if self.file_id is not None:
+            # Stanza requires Unicode apis
+            s.add('file_id', self.file_id.decode('utf8'))
+        return s
+
+    def _cmp_list(self):
+        return [type(self), self.path, self.file_id]
+
+    def __cmp__(self, other):
+        if getattr(other, "_cmp_list", None) is None:
+            return -1
+        x = self._cmp_list()
+        y = other._cmp_list()
+        return (x > y) - (x < y)
+
+    def __hash__(self):
+        return hash((type(self), self.path, self.file_id))
+
+    def __eq__(self, other):
+        return self.__cmp__(other) == 0
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __unicode__(self):
+        return self.describe()
+
+    def __str__(self):
+        return self.describe()
+
+    def describe(self):
+        return self.format % self.__dict__
+
+    def __repr__(self):
+        rdict = dict(self.__dict__)
+        rdict['class'] = self.__class__.__name__
+        return self.rformat % rdict
+
+    @staticmethod
+    def factory(type, **kwargs):
+        global ctype
+        return ctype[type](**kwargs)
+
+    @staticmethod
+    def sort_key(conflict):
+        if conflict.path is not None:
+            return conflict.path, conflict.typestring
+        elif getattr(conflict, "conflict_path", None) is not None:
+            return conflict.conflict_path, conflict.typestring
+        else:
+            return None, conflict.typestring
+
+    def do(self, action, tree):
+        """Apply the specified action to the conflict.
+
+        :param action: The method name to call.
+
+        :param tree: The tree passed as a parameter to the method.
+        """
+        meth = getattr(self, 'action_%s' % action, None)
+        if meth is None:
+            raise NotImplementedError(self.__class__.__name__ + '.' + action)
+        meth(tree)
+
+    def action_auto(self, tree):
+        raise NotImplementedError(self.action_auto)
+
+    def action_done(self, tree):
+        """Mark the conflict as solved once it has been handled."""
+        # This method does nothing but simplifies the design of upper levels.
+        pass
+
+    def action_take_this(self, tree):
+        raise NotImplementedError(self.action_take_this)
+
+    def action_take_other(self, tree):
+        raise NotImplementedError(self.action_take_other)
+
+    def _resolve_with_cleanups(self, tree, *args, **kwargs):
+        with tree.transform() as tt:
+            self._resolve(tt, *args, **kwargs)
+
+
+class ConflictList(BaseConflictList):
+
+    @staticmethod
+    def from_stanzas(stanzas):
+        """Produce a new ConflictList from an iterable of stanzas"""
+        conflicts = ConflictList()
+        for stanza in stanzas:
+            conflicts.append(Conflict.factory(**stanza.as_dict()))
+        return conflicts
+
+    def to_stanzas(self):
+        """Generator of stanzas"""
+        for conflict in self:
+            yield conflict.as_stanza()
+
+    def select_conflicts(self, tree, paths, ignore_misses=False,
+                         recurse=False):
+        """Select the conflicts associated with paths in a tree.
+
+        File-ids are also used for this.
+        :return: a pair of ConflictLists: (not_selected, selected)
+        """
+        path_set = set(paths)
+        ids = {}
+        selected_paths = set()
+        new_conflicts = ConflictList()
+        selected_conflicts = ConflictList()
+        for path in paths:
+            file_id = tree.path2id(path)
+            if file_id is not None:
+                ids[file_id] = path
+
+        for conflict in self:
+            selected = False
+            for key in ('path', 'conflict_path'):
+                cpath = getattr(conflict, key, None)
+                if cpath is None:
+                    continue
+                if cpath in path_set:
+                    selected = True
+                    selected_paths.add(cpath)
+                if recurse:
+                    if osutils.is_inside_any(path_set, cpath):
+                        selected = True
+                        selected_paths.add(cpath)
+
+            for key in ('file_id', 'conflict_file_id'):
+                cfile_id = getattr(conflict, key, None)
+                if cfile_id is None:
+                    continue
+                try:
+                    cpath = ids[cfile_id]
+                except KeyError:
+                    continue
+                selected = True
+                selected_paths.add(cpath)
+            if selected:
+                selected_conflicts.append(conflict)
+            else:
+                new_conflicts.append(conflict)
+        if ignore_misses is not True:
+            for path in [p for p in paths if p not in selected_paths]:
+                if not os.path.exists(tree.abspath(path)):
+                    print("%s does not exist" % path)
+                else:
+                    print("%s is not conflicted" % path)
+        return new_conflicts, selected_conflicts
+
+
+
+
+class PathConflict(Conflict):
+    """A conflict was encountered merging file paths"""
+
+    typestring = 'path conflict'
+
+    format = 'Path conflict: %(path)s / %(conflict_path)s'
+
+    rformat = '%(class)s(%(path)r, %(conflict_path)r, %(file_id)r)'
+
+    def __init__(self, path, conflict_path=None, file_id=None):
+        Conflict.__init__(self, path, file_id)
+        self.conflict_path = conflict_path
+
+    def as_stanza(self):
+        s = Conflict.as_stanza(self)
+        if self.conflict_path is not None:
+            s.add('conflict_path', self.conflict_path)
+        return s
+
+    def associated_filenames(self):
+        # No additional files have been generated here
+        return []
+
+    def _resolve(self, tt, file_id, path, winner):
+        """Resolve the conflict.
+
+        :param tt: The TreeTransform where the conflict is resolved.
+        :param file_id: The retained file id.
+        :param path: The retained path.
+        :param winner: 'this' or 'other' indicates which side is the winner.
+        """
+        path_to_create = None
+        if winner == 'this':
+            if self.path == '<deleted>':
+                return  # Nothing to do
+            if self.conflict_path == '<deleted>':
+                path_to_create = self.path
+                revid = tt._tree.get_parent_ids()[0]
+        elif winner == 'other':
+            if self.conflict_path == '<deleted>':
+                return  # Nothing to do
+            if self.path == '<deleted>':
+                path_to_create = self.conflict_path
+                # FIXME: If there are more than two parents we may need to
+                # iterate. Taking the last parent is the safer bet in the mean
+                # time. -- vila 20100309
+                revid = tt._tree.get_parent_ids()[-1]
+        else:
+            # Programmer error
+            raise AssertionError('bad winner: %r' % (winner,))
+        if path_to_create is not None:
+            tid = tt.trans_id_tree_path(path_to_create)
+            tree = self._revision_tree(tt._tree, revid)
+            transform.create_from_tree(
+                tt, tid, tree, tree.id2path(file_id))
+            tt.version_file(tid, file_id=file_id)
+        else:
+            tid = tt.trans_id_file_id(file_id)
+        # Adjust the path for the retained file id
+        parent_tid = tt.get_tree_parent(tid)
+        tt.adjust_path(osutils.basename(path), parent_tid, tid)
+        tt.apply()
+
+    def _revision_tree(self, tree, revid):
+        return tree.branch.repository.revision_tree(revid)
+
+    def _infer_file_id(self, tree):
+        # Prior to bug #531967, file_id wasn't always set, there may still be
+        # conflict files in the wild so we need to cope with them
+        # Establish which path we should use to find back the file-id
+        possible_paths = []
+        for p in (self.path, self.conflict_path):
+            if p == '<deleted>':
+                # special hard-coded path
+                continue
+            if p is not None:
+                possible_paths.append(p)
+        # Search the file-id in the parents with any path available
+        file_id = None
+        for revid in tree.get_parent_ids():
+            revtree = self._revision_tree(tree, revid)
+            for p in possible_paths:
+                file_id = revtree.path2id(p)
+                if file_id is not None:
+                    return revtree, file_id
+        return None, None
+
+    def action_take_this(self, tree):
+        if self.file_id is not None:
+            self._resolve_with_cleanups(tree, self.file_id, self.path,
+                                        winner='this')
+        else:
+            # Prior to bug #531967 we need to find back the file_id and restore
+            # the content from there
+            revtree, file_id = self._infer_file_id(tree)
+            tree.revert([revtree.id2path(file_id)],
+                        old_tree=revtree, backups=False)
+
+    def action_take_other(self, tree):
+        if self.file_id is not None:
+            self._resolve_with_cleanups(tree, self.file_id,
+                                        self.conflict_path,
+                                        winner='other')
+        else:
+            # Prior to bug #531967 we need to find back the file_id and restore
+            # the content from there
+            revtree, file_id = self._infer_file_id(tree)
+            tree.revert([revtree.id2path(file_id)],
+                        old_tree=revtree, backups=False)
+
+
+class ContentsConflict(PathConflict):
+    """The files are of different types (or both binary), or not present"""
+
+    has_files = True
+
+    typestring = 'contents conflict'
+
+    format = 'Contents conflict in %(path)s'
+
+    def associated_filenames(self):
+        return [self.path + suffix for suffix in ('.BASE', '.OTHER')]
+
+    def _resolve(self, tt, suffix_to_remove):
+        """Resolve the conflict.
+
+        :param tt: The TreeTransform where the conflict is resolved.
+        :param suffix_to_remove: Either 'THIS' or 'OTHER'
+
+        The resolution is symmetric: when taking THIS, OTHER is deleted and
+        item.THIS is renamed into item and vice-versa.
+        """
+        try:
+            # Delete 'item.THIS' or 'item.OTHER' depending on
+            # suffix_to_remove
+            tt.delete_contents(
+                tt.trans_id_tree_path(self.path + '.' + suffix_to_remove))
+        except errors.NoSuchFile:
+            # There are valid cases where 'item.suffix_to_remove' either
+            # never existed or was already deleted (including the case
+            # where the user deleted it)
+            pass
+        try:
+            this_path = tt._tree.id2path(self.file_id)
+        except errors.NoSuchId:
+            # The file is not present anymore. This may happen if the user
+            # deleted the file either manually or when resolving a conflict on
+            # the parent.  We may raise some exception to indicate that the
+            # conflict doesn't exist anymore and as such doesn't need to be
+            # resolved ? -- vila 20110615
+            this_tid = None
+        else:
+            this_tid = tt.trans_id_tree_path(this_path)
+        if this_tid is not None:
+            # Rename 'item.suffix_to_remove' (note that if
+            # 'item.suffix_to_remove' has been deleted, this is a no-op)
+            parent_tid = tt.get_tree_parent(this_tid)
+            tt.adjust_path(osutils.basename(self.path), parent_tid, this_tid)
+            tt.apply()
+
+    def action_take_this(self, tree):
+        self._resolve_with_cleanups(tree, 'OTHER')
+
+    def action_take_other(self, tree):
+        self._resolve_with_cleanups(tree, 'THIS')
+
+
+# TODO: There should be a base revid attribute to better inform the user about
+# how the conflicts were generated.
+class TextConflict(Conflict):
+    """The merge algorithm could not resolve all differences encountered."""
+
+    has_files = True
+
+    typestring = 'text conflict'
+
+    format = 'Text conflict in %(path)s'
+
+    rformat = '%(class)s(%(path)r, %(file_id)r)'
+
+    _conflict_re = re.compile(b'^(<{7}|={7}|>{7})')
+
+    def associated_filenames(self):
+        return [self.path + suffix for suffix in CONFLICT_SUFFIXES]
+
+    def _resolve(self, tt, winner_suffix):
+        """Resolve the conflict by copying one of .THIS or .OTHER into file.
+
+        :param tt: The TreeTransform where the conflict is resolved.
+        :param winner_suffix: Either 'THIS' or 'OTHER'
+
+        The resolution is symmetric, when taking THIS, item.THIS is renamed
+        into item and vice-versa. This takes one of the files as a whole
+        ignoring every difference that could have been merged cleanly.
+        """
+        # To avoid useless copies, we switch item and item.winner_suffix, only
+        # item will exist after the conflict has been resolved anyway.
+        item_tid = tt.trans_id_file_id(self.file_id)
+        item_parent_tid = tt.get_tree_parent(item_tid)
+        winner_path = self.path + '.' + winner_suffix
+        winner_tid = tt.trans_id_tree_path(winner_path)
+        winner_parent_tid = tt.get_tree_parent(winner_tid)
+        # Switch the paths to preserve the content
+        tt.adjust_path(osutils.basename(self.path),
+                       winner_parent_tid, winner_tid)
+        tt.adjust_path(osutils.basename(winner_path),
+                       item_parent_tid, item_tid)
+        # Associate the file_id to the right content
+        tt.unversion_file(item_tid)
+        tt.version_file(winner_tid, file_id=self.file_id)
+        tt.apply()
+
+    def action_auto(self, tree):
+        # GZ 2012-07-27: Using NotImplementedError to signal that a conflict
+        #                can't be auto resolved does not seem ideal.
+        try:
+            kind = tree.kind(self.path)
+        except errors.NoSuchFile:
+            return
+        if kind != 'file':
+            raise NotImplementedError("Conflict is not a file")
+        conflict_markers_in_line = self._conflict_re.search
+        # GZ 2012-07-27: What if not tree.has_id(self.file_id) due to removal?
+        with tree.get_file(self.path) as f:
+            for line in f:
+                if conflict_markers_in_line(line):
+                    raise NotImplementedError("Conflict markers present")
+
+    def action_take_this(self, tree):
+        self._resolve_with_cleanups(tree, 'THIS')
+
+    def action_take_other(self, tree):
+        self._resolve_with_cleanups(tree, 'OTHER')
+
+
+class HandledConflict(Conflict):
+    """A path problem that has been provisionally resolved.
+    This is intended to be a base class.
+    """
+
+    rformat = "%(class)s(%(action)r, %(path)r, %(file_id)r)"
+
+    def __init__(self, action, path, file_id=None):
+        Conflict.__init__(self, path, file_id)
+        self.action = action
+
+    def _cmp_list(self):
+        return Conflict._cmp_list(self) + [self.action]
+
+    def as_stanza(self):
+        s = Conflict.as_stanza(self)
+        s.add('action', self.action)
+        return s
+
+    def associated_filenames(self):
+        # Nothing has been generated here
+        return []
+
+
+class HandledPathConflict(HandledConflict):
+    """A provisionally-resolved path problem involving two paths.
+    This is intended to be a base class.
+    """
+
+    rformat = "%(class)s(%(action)r, %(path)r, %(conflict_path)r,"\
+        " %(file_id)r, %(conflict_file_id)r)"
+
+    def __init__(self, action, path, conflict_path, file_id=None,
+                 conflict_file_id=None):
+        HandledConflict.__init__(self, action, path, file_id)
+        self.conflict_path = conflict_path
+        # the factory blindly transfers the Stanza values to __init__,
+        # so they can be unicode.
+        if isinstance(conflict_file_id, text_type):
+            conflict_file_id = cache_utf8.encode(conflict_file_id)
+        self.conflict_file_id = conflict_file_id
+
+    def _cmp_list(self):
+        return HandledConflict._cmp_list(self) + [self.conflict_path,
+                                                  self.conflict_file_id]
+
+    def as_stanza(self):
+        s = HandledConflict.as_stanza(self)
+        s.add('conflict_path', self.conflict_path)
+        if self.conflict_file_id is not None:
+            s.add('conflict_file_id', self.conflict_file_id.decode('utf8'))
+
+        return s
+
+
+class DuplicateID(HandledPathConflict):
+    """Two files want the same file_id."""
+
+    typestring = 'duplicate id'
+
+    format = 'Conflict adding id to %(conflict_path)s.  %(action)s %(path)s.'
+
+
+class DuplicateEntry(HandledPathConflict):
+    """Two directory entries want to have the same name."""
+
+    typestring = 'duplicate'
+
+    format = 'Conflict adding file %(conflict_path)s.  %(action)s %(path)s.'
+
+    def action_take_this(self, tree):
+        tree.remove([self.conflict_path], force=True, keep_files=False)
+        tree.rename_one(self.path, self.conflict_path)
+
+    def action_take_other(self, tree):
+        tree.remove([self.path], force=True, keep_files=False)
+
+
+class ParentLoop(HandledPathConflict):
+    """An attempt to create an infinitely-looping directory structure.
+    This is rare, but can be produced like so:
+
+    tree A:
+      mv foo bar
+    tree B:
+      mv bar foo
+    merge A and B
+    """
+
+    typestring = 'parent loop'
+
+    format = 'Conflict moving %(path)s into %(conflict_path)s. %(action)s.'
+
+    def action_take_this(self, tree):
+        # just acccept brz proposal
+        pass
+
+    def action_take_other(self, tree):
+        with tree.transform() as tt:
+            p_tid = tt.trans_id_file_id(self.file_id)
+            parent_tid = tt.get_tree_parent(p_tid)
+            cp_tid = tt.trans_id_file_id(self.conflict_file_id)
+            cparent_tid = tt.get_tree_parent(cp_tid)
+            tt.adjust_path(osutils.basename(self.path), cparent_tid, cp_tid)
+            tt.adjust_path(osutils.basename(self.conflict_path),
+                           parent_tid, p_tid)
+            tt.apply()
+
+
+class UnversionedParent(HandledConflict):
+    """An attempt to version a file whose parent directory is not versioned.
+    Typically, the result of a merge where one tree unversioned the directory
+    and the other added a versioned file to it.
+    """
+
+    typestring = 'unversioned parent'
+
+    format = 'Conflict because %(path)s is not versioned, but has versioned'\
+             ' children.  %(action)s.'
+
+    # FIXME: We silently do nothing to make tests pass, but most probably the
+    # conflict shouldn't exist (the long story is that the conflict is
+    # generated with another one that can be resolved properly) -- vila 091224
+    def action_take_this(self, tree):
+        pass
+
+    def action_take_other(self, tree):
+        pass
+
+
+class MissingParent(HandledConflict):
+    """An attempt to add files to a directory that is not present.
+    Typically, the result of a merge where THIS deleted the directory and
+    the OTHER added a file to it.
+    See also: DeletingParent (same situation, THIS and OTHER reversed)
+    """
+
+    typestring = 'missing parent'
+
+    format = 'Conflict adding files to %(path)s.  %(action)s.'
+
+    def action_take_this(self, tree):
+        tree.remove([self.path], force=True, keep_files=False)
+
+    def action_take_other(self, tree):
+        # just acccept brz proposal
+        pass
+
+
+class DeletingParent(HandledConflict):
+    """An attempt to add files to a directory that is not present.
+    Typically, the result of a merge where one OTHER deleted the directory and
+    the THIS added a file to it.
+    """
+
+    typestring = 'deleting parent'
+
+    format = "Conflict: can't delete %(path)s because it is not empty.  "\
+             "%(action)s."
+
+    # FIXME: It's a bit strange that the default action is not coherent with
+    # MissingParent from the *user* pov.
+
+    def action_take_this(self, tree):
+        # just acccept brz proposal
+        pass
+
+    def action_take_other(self, tree):
+        tree.remove([self.path], force=True, keep_files=False)
+
+
+class NonDirectoryParent(HandledConflict):
+    """An attempt to add files to a directory that is not a directory or
+    an attempt to change the kind of a directory with files.
+    """
+
+    typestring = 'non-directory parent'
+
+    format = "Conflict: %(path)s is not a directory, but has files in it."\
+             "  %(action)s."
+
+    # FIXME: .OTHER should be used instead of .new when the conflict is created
+
+    def action_take_this(self, tree):
+        # FIXME: we should preserve that path when the conflict is generated !
+        if self.path.endswith('.new'):
+            conflict_path = self.path[:-(len('.new'))]
+            tree.remove([self.path], force=True, keep_files=False)
+            tree.add(conflict_path)
+        else:
+            raise NotImplementedError(self.action_take_this)
+
+    def action_take_other(self, tree):
+        # FIXME: we should preserve that path when the conflict is generated !
+        if self.path.endswith('.new'):
+            conflict_path = self.path[:-(len('.new'))]
+            tree.remove([conflict_path], force=True, keep_files=False)
+            tree.rename_one(self.path, conflict_path)
+        else:
+            raise NotImplementedError(self.action_take_other)
+
+
+ctype = {}
+
+
+def register_types(*conflict_types):
+    """Register a Conflict subclass for serialization purposes"""
+    global ctype
+    for conflict_type in conflict_types:
+        ctype[conflict_type.typestring] = conflict_type
+
+
+register_types(ContentsConflict, TextConflict, PathConflict, DuplicateID,
+               DuplicateEntry, ParentLoop, UnversionedParent, MissingParent,
+               DeletingParent, NonDirectoryParent)

=== modified file 'breezy/bzr/dirstate.py'
--- breezy/bzr/dirstate.py	2020-07-18 23:14:00 +0000
+++ breezy/bzr/dirstate.py	2020-08-10 15:00:17 +0000
@@ -243,7 +243,7 @@
     trace,
     urlutils,
     )
-from ..tree import TreeChange
+from .inventorytree import InventoryTreeChange
 
 
 # This is the Windows equivalent of ENOTDIR
@@ -3759,7 +3759,7 @@
                     else:
                         path_u = self.utf8_decode(path)[0]
                 source_kind = DirState._minikind_to_kind[source_minikind]
-                return TreeChange(
+                return InventoryTreeChange(
                     entry[0][2],
                     (old_path_u, path_u),
                     content_change,
@@ -3788,7 +3788,7 @@
                         and stat.S_IEXEC & path_info[3].st_mode)
                 else:
                     target_exec = target_details[3]
-                return TreeChange(
+                return InventoryTreeChange(
                     entry[0][2],
                     (None, self.utf8_decode(path)[0]),
                     True,
@@ -3799,7 +3799,7 @@
                     (None, target_exec)), True
             else:
                 # Its a missing file, report it as such.
-                return TreeChange(
+                return InventoryTreeChange(
                     entry[0][2],
                     (None, self.utf8_decode(path)[0]),
                     False,
@@ -3819,7 +3819,7 @@
                 self.source_index, path_utf8=entry[0][0])[0][2]
             if parent_id == entry[0][2]:
                 parent_id = None
-            return TreeChange(
+            return InventoryTreeChange(
                 entry[0][2],
                 (self.utf8_decode(old_path)[0], None),
                 True,
@@ -3961,7 +3961,7 @@
                 new_executable = bool(
                     stat.S_ISREG(root_dir_info[3].st_mode)
                     and stat.S_IEXEC & root_dir_info[3].st_mode)
-                yield TreeChange(
+                yield InventoryTreeChange(
                     None,
                     (None, current_root_unicode),
                     True,
@@ -4044,7 +4044,7 @@
                                 new_executable = bool(
                                     stat.S_ISREG(current_path_info[3].st_mode)
                                     and stat.S_IEXEC & current_path_info[3].st_mode)
-                                yield TreeChange(
+                                yield InventoryTreeChange(
                                     None,
                                     (None, utf8_decode(current_path_info[0])[0]),
                                     True,
@@ -4180,7 +4180,7 @@
                                 except UnicodeDecodeError:
                                     raise errors.BadFilenameEncoding(
                                         current_path_info[0], osutils._fs_enc)
-                                yield TreeChange(
+                                yield InventoryTreeChange(
                                     None,
                                     (None, relpath_unicode),
                                     True,

=== modified file 'breezy/bzr/inventorytree.py'
--- breezy/bzr/inventorytree.py	2020-07-18 23:14:00 +0000
+++ breezy/bzr/inventorytree.py	2020-08-10 15:00:17 +0000
@@ -67,6 +67,52 @@
     )
 
 
+class InventoryTreeChange(TreeChange):
+
+    __slots__ = TreeChange.__slots__ + ['file_id', 'parent_id']
+
+    def __init__(self, file_id, path, changed_content, versioned, parent_id,
+                 name, kind, executable, copied=False):
+        self.file_id = file_id
+        self.parent_id = parent_id
+        super(InventoryTreeChange, self).__init__(
+            path=path, changed_content=changed_content, versioned=versioned,
+            name=name, kind=kind, executable=executable, copied=copied)
+
+    def __repr__(self):
+        return "%s%r" % (self.__class__.__name__, self._as_tuple())
+
+    def _as_tuple(self):
+        return (self.file_id, self.path, self.changed_content, self.versioned,
+                self.parent_id, self.name, self.kind, self.executable, self.copied)
+
+    def __eq__(self, other):
+        if isinstance(other, TreeChange):
+            return self._as_tuple() == other._as_tuple()
+        if isinstance(other, tuple):
+            return self._as_tuple() == other
+        return False
+
+    def __lt__(self, other):
+        return self._as_tuple() < other._as_tuple()
+
+    def meta_modified(self):
+        if self.versioned == (True, True):
+            return (self.executable[0] != self.executable[1])
+        return False
+
+    def is_reparented(self):
+        return self.parent_id[0] != self.parent_id[1]
+
+    def discard_new(self):
+        return self.__class__(
+            self.file_id, (self.path[0], None), self.changed_content,
+            (self.versioned[0], None), (self.parent_id[0], None),
+            (self.name[0], None), (self.kind[0], None),
+            (self.executable[0], None),
+            copied=False)
+
+
 class InventoryTree(Tree):
     """A tree that relies on an inventory for its metadata.
 
@@ -418,6 +464,51 @@
             inv.apply_delta(changes)
             self._write_inventory(inv)
 
+    def has_changes(self, _from_tree=None):
+        """Quickly check that the tree contains at least one commitable change.
+
+        :param _from_tree: tree to compare against to find changes (default to
+            the basis tree and is intended to be used by tests).
+
+        :return: True if a change is found. False otherwise
+        """
+        with self.lock_read():
+            # Check pending merges
+            if len(self.get_parent_ids()) > 1:
+                return True
+            if _from_tree is None:
+                _from_tree = self.basis_tree()
+            changes = self.iter_changes(_from_tree)
+            if self.supports_symlinks():
+                # Fast path for has_changes.
+                try:
+                    change = next(changes)
+                    # Exclude root (talk about black magic... --vila 20090629)
+                    if change.parent_id == (None, None):
+                        change = next(changes)
+                    return True
+                except StopIteration:
+                    # No changes
+                    return False
+            else:
+                # Slow path for has_changes.
+                # Handle platforms that do not support symlinks in the
+                # conditional below. This is slower than the try/except
+                # approach below that but we don't have a choice as we
+                # need to be sure that all symlinks are removed from the
+                # entire changeset. This is because in platforms that
+                # do not support symlinks, they show up as None in the
+                # working copy as compared to the repository.
+                # Also, exclude root as mention in the above fast path.
+                changes = filter(
+                    lambda c: c[6][0] != 'symlink' and c[4] != (None, None),
+                    changes)
+                try:
+                    next(iter(changes))
+                except StopIteration:
+                    return False
+                return True
+
     def _fix_case_of_inventory_path(self, path):
         """If our tree isn't case sensitive, return the canonical path"""
         if not self.case_sensitive:
@@ -906,27 +997,26 @@
         if top_id is None:
             pending = []
         else:
-            pending = [(prefix, '', _directory, None, top_id, None)]
+            pending = [(prefix, top_id)]
         while pending:
             dirblock = []
-            currentdir = pending.pop()
-            # 0 - relpath, 1- basename, 2- kind, 3- stat, id, v-kind
-            if currentdir[0]:
-                relroot = currentdir[0] + '/'
+            root, file_id = pending.pop()
+            if root:
+                relroot = root + '/'
             else:
                 relroot = ""
             # FIXME: stash the node in pending
-            entry = inv.get_entry(currentdir[4])
+            entry = inv.get_entry(file_id)
+            subdirs = []
             for name, child in entry.sorted_children():
                 toppath = relroot + name
                 dirblock.append((toppath, name, child.kind, None,
-                                 child.file_id, child.kind
-                                 ))
-            yield (currentdir[0], entry.file_id), dirblock
+                                 child.kind))
+                if child.kind == _directory:
+                    subdirs.append((toppath, child.file_id))
+            yield root, dirblock
             # push the user specified dirs from dirblock
-            for dir in reversed(dirblock):
-                if dir[2] == _directory:
-                    pending.append(dir)
+            pending.extend(reversed(subdirs))
 
     def iter_files_bytes(self, desired_files):
         """See Tree.iter_files_bytes.
@@ -1047,7 +1137,7 @@
             changes = True
         else:
             changes = False
-        return TreeChange(
+        return InventoryTreeChange(
             file_id, (source_path, target_path), changed_content,
             versioned, parent, name, kind, executable), changes
 
@@ -1141,7 +1231,7 @@
                 target_kind, target_executable, target_stat = \
                     self.target._comparison_data(
                         fake_entry, unversioned_path[1])
-                yield TreeChange(
+                yield InventoryTreeChange(
                     None, (None, unversioned_path[1]), True, (False, False),
                     (None, None),
                     (None, unversioned_path[0][-1]),
@@ -1178,7 +1268,7 @@
             unversioned_path = all_unversioned.popleft()
             to_kind, to_executable, to_stat = \
                 self.target._comparison_data(fake_entry, unversioned_path[1])
-            yield TreeChange(
+            yield InventoryTreeChange(
                 None, (None, unversioned_path[1]), True, (False, False),
                 (None, None),
                 (None, unversioned_path[0][-1]),
@@ -1204,7 +1294,7 @@
             changed_content = from_kind is not None
             # the parent's path is necessarily known at this point.
             changed_file_ids.append(file_id)
-            yield TreeChange(
+            yield InventoryTreeChange(
                 file_id, (path, to_path), changed_content, versioned, parent,
                 name, kind, executable)
         changed_file_ids = set(changed_file_ids)
@@ -1390,7 +1480,7 @@
         # FIXME: nested tree support
         for result in self.target.root_inventory.iter_changes(
                 self.source.root_inventory):
-            result = TreeChange(*result)
+            result = InventoryTreeChange(*result)
             if specific_file_ids is not None:
                 if result.file_id not in specific_file_ids:
                     # A change from the whole tree that we don't want to show yet.
@@ -1417,7 +1507,7 @@
                         entry.file_id not in specific_file_ids):
                     continue
                 if entry.file_id not in changed_file_ids:
-                    yield TreeChange(
+                    yield InventoryTreeChange(
                         entry.file_id,
                         (relpath, relpath),  # Not renamed
                         False,  # Not modified

=== modified file 'breezy/bzr/tests/__init__.py'
--- breezy/bzr/tests/__init__.py	2020-06-30 00:19:29 +0000
+++ breezy/bzr/tests/__init__.py	2020-08-09 18:10:01 +0000
@@ -63,6 +63,7 @@
         'test_bzrdir',
         'test_chk_map',
         'test_chk_serializer',
+        'test_conflicts',
         'test_generate_ids',
         'test_groupcompress',
         'test_index',

=== added file 'breezy/bzr/tests/test_conflicts.py'
--- breezy/bzr/tests/test_conflicts.py	1970-01-01 00:00:00 +0000
+++ breezy/bzr/tests/test_conflicts.py	2020-08-09 18:10:01 +0000
@@ -0,0 +1,134 @@
+# Copyright (C) 2005-2011 Canonical Ltd
+# Copyright (C) 2018-2020 Breezy Developers
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import os
+
+from ...conflicts import resolve
+from ... import (
+    tests,
+    )
+
+from ...tests import (
+    scenarios,
+    )
+from ...tests.test_conflicts import vary_by_conflicts
+from ...sixish import text_type
+
+
+from .. import conflicts as bzr_conflicts
+
+
+load_tests = scenarios.load_tests_apply_scenarios
+
+
+class TestPerConflict(tests.TestCase):
+
+    scenarios = scenarios.multiply_scenarios(vary_by_conflicts())
+
+    def test_stringification(self):
+        text = text_type(self.conflict)
+        self.assertContainsString(text, self.conflict.path)
+        self.assertContainsString(text.lower(), "conflict")
+        self.assertContainsString(repr(self.conflict),
+                                  self.conflict.__class__.__name__)
+
+    def test_stanza_roundtrip(self):
+        p = self.conflict
+        o = bzr_conflicts.Conflict.factory(**p.as_stanza().as_dict())
+        self.assertEqual(o, p)
+
+        self.assertIsInstance(o.path, text_type)
+
+        if o.file_id is not None:
+            self.assertIsInstance(o.file_id, bytes)
+
+        conflict_path = getattr(o, 'conflict_path', None)
+        if conflict_path is not None:
+            self.assertIsInstance(conflict_path, text_type)
+
+        conflict_file_id = getattr(o, 'conflict_file_id', None)
+        if conflict_file_id is not None:
+            self.assertIsInstance(conflict_file_id, bytes)
+
+    def test_stanzification(self):
+        stanza = self.conflict.as_stanza()
+        if 'file_id' in stanza:
+            # In Stanza form, the file_id has to be unicode.
+            self.assertStartsWith(stanza['file_id'], u'\xeed')
+        self.assertStartsWith(stanza['path'], u'p\xe5th')
+        if 'conflict_path' in stanza:
+            self.assertStartsWith(stanza['conflict_path'], u'p\xe5th')
+        if 'conflict_file_id' in stanza:
+            self.assertStartsWith(stanza['conflict_file_id'], u'\xeed')
+
+
+class TestConflicts(tests.TestCaseWithTransport):
+
+    def test_resolve_conflict_dir(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('hello', b'hello world4'),
+                                  ('hello.THIS', b'hello world2'),
+                                  ('hello.BASE', b'hello world1'),
+                                  ])
+        os.mkdir('hello.OTHER')
+        tree.add('hello', b'q')
+        l = bzr_conflicts.ConflictList([bzr_conflicts.TextConflict('hello')])
+        l.remove_files(tree)
+
+    def test_select_conflicts(self):
+        tree = self.make_branch_and_tree('.')
+        clist = bzr_conflicts.ConflictList
+
+        def check_select(not_selected, selected, paths, **kwargs):
+            self.assertEqual(
+                (not_selected, selected),
+                tree_conflicts.select_conflicts(tree, paths, **kwargs))
+
+        foo = bzr_conflicts.ContentsConflict('foo')
+        bar = bzr_conflicts.ContentsConflict('bar')
+        tree_conflicts = clist([foo, bar])
+
+        check_select(clist([bar]), clist([foo]), ['foo'])
+        check_select(clist(), tree_conflicts,
+                     [''], ignore_misses=True, recurse=True)
+
+        foobaz = bzr_conflicts.ContentsConflict('foo/baz')
+        tree_conflicts = clist([foobaz, bar])
+
+        check_select(clist([bar]), clist([foobaz]),
+                     ['foo'], ignore_misses=True, recurse=True)
+
+        qux = bzr_conflicts.PathConflict('qux', 'foo/baz')
+        tree_conflicts = clist([qux])
+
+        check_select(clist(), tree_conflicts,
+                     ['foo'], ignore_misses=True, recurse=True)
+        check_select(tree_conflicts, clist(), ['foo'], ignore_misses=True)
+
+    def test_resolve_conflicts_recursive(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['dir/', 'dir/hello'])
+        tree.add(['dir', 'dir/hello'])
+
+        dirhello = [bzr_conflicts.TextConflict('dir/hello')]
+        tree.set_conflicts(dirhello)
+
+        resolve(tree, ['dir'], recursive=False, ignore_misses=True)
+        self.assertEqual(dirhello, tree.conflicts())
+
+        resolve(tree, ['dir'], recursive=True, ignore_misses=True)
+        self.assertEqual(bzr_conflicts.ConflictList([]), tree.conflicts())

=== modified file 'breezy/bzr/transform.py'
--- breezy/bzr/transform.py	2020-07-20 02:02:23 +0000
+++ breezy/bzr/transform.py	2020-08-10 15:00:17 +0000
@@ -24,6 +24,7 @@
 
 from .. import (
     annotate,
+    conflicts,
     errors,
     lock,
     multiparent,
@@ -52,7 +53,8 @@
     MalformedTransform,
     PreviewTree,
     )
-from ..tree import TreeChange
+from .conflicts import Conflict
+
 from . import (
     inventory,
     inventorytree,
@@ -296,7 +298,7 @@
             if value == trans_id:
                 return key
 
-    def find_conflicts(self):
+    def find_raw_conflicts(self):
         """Find any violations of inventory or filesystem invariants"""
         if self._done is True:
             raise ReusingTransform()
@@ -315,7 +317,7 @@
         return conflicts
 
     def _check_malformed(self):
-        conflicts = self.find_conflicts()
+        conflicts = self.find_raw_conflicts()
         if len(conflicts) != 0:
             raise MalformedTransform(conflicts=conflicts)
 
@@ -703,7 +705,7 @@
         """Produce output in the same format as Tree.iter_changes.
 
         Will produce nonsensical results if invoked while inventory/filesystem
-        conflicts (as reported by TreeTransform.find_conflicts()) are present.
+        conflicts (as reported by TreeTransform.find_raw_conflicts()) are present.
 
         This reads the Transform, but only reproduces changes involving a
         file_id.  Files that are not versioned in either of the FROM or TO
@@ -755,7 +757,7 @@
                     and from_executable == to_executable):
                 continue
             results.append(
-                TreeChange(
+                inventorytree.InventoryTreeChange(
                     file_id, (from_path, to_path), modified,
                     (from_versioned, to_versioned),
                     (from_parent, to_parent),
@@ -1010,6 +1012,100 @@
         """
         raise NotImplementedError(self.apply)
 
+    def cook_conflicts(self, raw_conflicts):
+        """Generate a list of cooked conflicts, sorted by file path"""
+        content_conflict_file_ids = set()
+        cooked_conflicts = list(iter_cook_conflicts(raw_conflicts, self))
+        for c in cooked_conflicts:
+            if c.typestring == 'contents conflict':
+                content_conflict_file_ids.add(c.file_id)
+        # We want to get rid of path conflicts when a corresponding contents
+        # conflict exists. This can occur when one branch deletes a file while
+        # the other renames *and* modifies it. In this case, the content
+        # conflict is enough.
+        cooked_conflicts = [
+            c for c in cooked_conflicts
+            if c.typestring != 'path conflict' or
+            c.file_id not in content_conflict_file_ids]
+        return sorted(cooked_conflicts, key=Conflict.sort_key)
+
+
+def cook_path_conflict(
+        tt, fp, conflict_type, trans_id, file_id, this_parent, this_name,
+        other_parent, other_name):
+    if this_parent is None or this_name is None:
+        this_path = '<deleted>'
+    else:
+        parent_path = fp.get_path(tt.trans_id_file_id(this_parent))
+        this_path = osutils.pathjoin(parent_path, this_name)
+    if other_parent is None or other_name is None:
+        other_path = '<deleted>'
+    else:
+        try:
+            parent_path = fp.get_path(tt.trans_id_file_id(other_parent))
+        except NoFinalPath:
+            # The other entry was in a path that doesn't exist in our tree.
+            # Put it in the root.
+            parent_path = ''
+        other_path = osutils.pathjoin(parent_path, other_name)
+    return Conflict.factory(
+        conflict_type, path=this_path,
+        conflict_path=other_path,
+        file_id=file_id)
+
+
+def cook_content_conflict(tt, fp, conflict_type, trans_ids):
+    for trans_id in trans_ids:
+        file_id = tt.final_file_id(trans_id)
+        if file_id is not None:
+            # Ok we found the relevant file-id
+            break
+    path = fp.get_path(trans_id)
+    for suffix in ('.BASE', '.THIS', '.OTHER'):
+        if path.endswith(suffix):
+            # Here is the raw path
+            path = path[:-len(suffix)]
+            break
+    return Conflict.factory(conflict_type, path=path, file_id=file_id)
+
+
+def cook_text_conflict(tt, fp, conflict_type, trans_id):
+    path = fp.get_path(trans_id)
+    file_id = tt.final_file_id(trans_id)
+    return Conflict.factory(conflict_type, path=path, file_id=file_id)
+
+
+CONFLICT_COOKERS = {
+    'path conflict': cook_path_conflict,
+    'text conflict': cook_text_conflict,
+    'contents conflict': cook_content_conflict,
+}
+
+def iter_cook_conflicts(raw_conflicts, tt):
+    fp = FinalPaths(tt)
+    for conflict in raw_conflicts:
+        c_type = conflict[0]
+        try:
+            cooker = CONFLICT_COOKERS[c_type]
+        except KeyError:
+            action = conflict[1]
+            modified_path = fp.get_path(conflict[2])
+            modified_id = tt.final_file_id(conflict[2])
+            if len(conflict) == 3:
+                yield Conflict.factory(
+                    c_type, action=action, path=modified_path, file_id=modified_id)
+
+            else:
+                conflicting_path = fp.get_path(conflict[3])
+                conflicting_id = tt.final_file_id(conflict[3])
+                yield Conflict.factory(
+                    c_type, action=action, path=modified_path,
+                    file_id=modified_id,
+                    conflict_path=conflicting_path,
+                    conflict_file_id=conflicting_id)
+        else:
+            yield cooker(tt, fp, *conflict)
+
 
 class DiskTreeTransform(TreeTransformBase):
     """Tree transform storing its contents on disk."""
@@ -1497,8 +1593,8 @@
                 conflicts.append(('duplicate id', old_trans_id, trans_id))
         return conflicts
 
-    def find_conflicts(self):
-        conflicts = super(InventoryTreeTransform, self).find_conflicts()
+    def find_raw_conflicts(self):
+        conflicts = super(InventoryTreeTransform, self).find_raw_conflicts()
         conflicts.extend(self._duplicate_ids())
         return conflicts
 
@@ -2121,10 +2217,10 @@
                 if versioned_kind == 'directory':
                     subdirs.append(child_id)
                 children.append((path_from_root, basename, kind, None,
-                                 file_id, versioned_kind))
+                                 versioned_kind))
             children.sort()
             if parent_path.startswith(prefix):
-                yield (parent_path, parent_file_id), children
+                yield parent_path, children
             pending.extend(sorted(subdirs, key=self._final_paths.get_path,
                                   reverse=True))
 

=== modified file 'breezy/bzr/vf_repository.py'
--- breezy/bzr/vf_repository.py	2020-07-18 23:14:00 +0000
+++ breezy/bzr/vf_repository.py	2020-08-10 15:00:17 +0000
@@ -80,7 +80,7 @@
 from ..trace import (
     mutter
     )
-from ..tree import TreeChange
+from .inventorytree import InventoryTreeChange
 
 
 class VersionedFileRepositoryFormat(RepositoryFormat):
@@ -399,7 +399,7 @@
                 # by the user. So we discard this change.
                 pass
             else:
-                change = TreeChange(
+                change = InventoryTreeChange(
                     file_id,
                     (basis_inv.id2path(file_id), tree.id2path(file_id)),
                     False, (True, True),

=== modified file 'breezy/bzr/workingtree.py'
--- breezy/bzr/workingtree.py	2020-07-18 23:14:00 +0000
+++ breezy/bzr/workingtree.py	2020-08-10 15:00:17 +0000
@@ -60,6 +60,7 @@
     rio as _mod_rio,
     )
 from breezy.bzr import (
+    conflicts as _mod_bzr_conflicts,
     inventory,
     serializer,
     xml5,
@@ -403,8 +404,8 @@
         def recurse_directory_to_add_files(directory):
             # Recurse directory and add all files
             # so we can check if they have changed.
-            for parent_info, file_infos in self.walkdirs(directory):
-                for relpath, basename, kind, lstat, fileid, kind in file_infos:
+            for parent_path, file_infos in self.walkdirs(directory):
+                for relpath, basename, kind, lstat, kind in file_infos:
                     # Is it versioned or ignored?
                     if self.is_versioned(relpath):
                         # Add nested content for deletion.
@@ -573,23 +574,24 @@
         return xml7.serializer_v7.write_inventory_to_lines(inventory)
 
     def set_conflicts(self, conflicts):
+        conflict_list = _mod_bzr_conflicts.ConflictList(conflicts)
         with self.lock_tree_write():
-            self._put_rio('conflicts', conflicts.to_stanzas(),
+            self._put_rio('conflicts', conflict_list.to_stanzas(),
                           CONFLICT_HEADER_1)
 
     def add_conflicts(self, new_conflicts):
         with self.lock_tree_write():
             conflict_set = set(self.conflicts())
             conflict_set.update(set(list(new_conflicts)))
-            self.set_conflicts(_mod_conflicts.ConflictList(
-                sorted(conflict_set, key=_mod_conflicts.Conflict.sort_key)))
+            self.set_conflicts(
+                sorted(conflict_set, key=_mod_bzr_conflicts.Conflict.sort_key))
 
     def conflicts(self):
         with self.lock_read():
             try:
                 confile = self._transport.get('conflicts')
             except errors.NoSuchFile:
-                return _mod_conflicts.ConflictList()
+                return _mod_bzr_conflicts.ConflictList()
             try:
                 try:
                     if next(confile) != CONFLICT_HEADER_1 + b'\n':
@@ -597,7 +599,7 @@
                 except StopIteration:
                     raise errors.ConflictFormatError()
                 reader = _mod_rio.RioReader(confile)
-                return _mod_conflicts.ConflictList.from_stanzas(reader)
+                return _mod_bzr_conflicts.ConflictList.from_stanzas(reader)
             finally:
                 confile.close()
 
@@ -1562,8 +1564,8 @@
         """Walk the directories of this tree.
 
         returns a generator which yields items in the form:
-                ((curren_directory_path, fileid),
-                 [(file1_path, file1_name, file1_kind, (lstat), file1_id,
+                (current_directory_path,
+                 [(file1_path, file1_name, file1_kind, (lstat),
                    file1_kind), ... ])
 
         This API returns a generator, which is only valid during the current
@@ -1626,20 +1628,20 @@
 
             if direction > 0:
                 # disk is before inventory - unknown
-                dirblock = [(relpath, basename, kind, stat, None, None) for
+                dirblock = [(relpath, basename, kind, stat, None) for
                             relpath, basename, kind, stat, top_path in
                             cur_disk_dir_content]
-                yield (cur_disk_dir_relpath, None), dirblock
+                yield cur_disk_dir_relpath, dirblock
                 try:
                     current_disk = next(disk_iterator)
                 except StopIteration:
                     disk_finished = True
             elif direction < 0:
                 # inventory is before disk - missing.
-                dirblock = [(relpath, basename, 'unknown', None, fileid, kind)
+                dirblock = [(relpath, basename, 'unknown', None, kind)
                             for relpath, basename, dkind, stat, fileid, kind in
                             current_inv[1]]
-                yield (current_inv[0][0], current_inv[0][1]), dirblock
+                yield current_inv[0][0], dirblock
                 try:
                     current_inv = next(inventory_iterator)
                 except StopIteration:
@@ -1657,23 +1659,20 @@
                         # versioned, present file
                         dirblock.append((inv_row[0],
                                          inv_row[1], disk_row[2],
-                                         disk_row[3], inv_row[4],
-                                         inv_row[5]))
+                                         disk_row[3], inv_row[5]))
                     elif len(path_elements[0]) == 5:
                         # unknown disk file
                         dirblock.append(
                             (path_elements[0][0], path_elements[0][1],
-                             path_elements[0][2], path_elements[0][3], None,
-                             None))
+                             path_elements[0][2], path_elements[0][3], None))
                     elif len(path_elements[0]) == 6:
                         # versioned, absent file.
                         dirblock.append(
                             (path_elements[0][0], path_elements[0][1],
-                             'unknown', None, path_elements[0][4],
-                             path_elements[0][5]))
+                             'unknown', None, path_elements[0][5]))
                     else:
                         raise NotImplementedError('unreachable code')
-                yield current_inv[0], dirblock
+                yield current_inv[0][0], dirblock
                 try:
                     current_inv = next(inventory_iterator)
                 except StopIteration:
@@ -1818,6 +1817,52 @@
             self.path2id(path),
             path, possible_transports=possible_transports)
 
+    def has_changes(self, _from_tree=None):
+        """Quickly check that the tree contains at least one commitable change.
+
+        :param _from_tree: tree to compare against to find changes (default to
+            the basis tree and is intended to be used by tests).
+
+        :return: True if a change is found. False otherwise
+        """
+        with self.lock_read():
+            # Check pending merges
+            if len(self.get_parent_ids()) > 1:
+                return True
+            if _from_tree is None:
+                _from_tree = self.basis_tree()
+            changes = self.iter_changes(_from_tree)
+            if self.supports_symlinks():
+                # Fast path for has_changes.
+                try:
+                    change = next(changes)
+                    # Exclude root (talk about black magic... --vila 20090629)
+                    if change.parent_id == (None, None):
+                        change = next(changes)
+                    return True
+                except StopIteration:
+                    # No changes
+                    return False
+            else:
+                # Slow path for has_changes.
+                # Handle platforms that do not support symlinks in the
+                # conditional below. This is slower than the try/except
+                # approach below that but we don't have a choice as we
+                # need to be sure that all symlinks are removed from the
+                # entire changeset. This is because in platforms that
+                # do not support symlinks, they show up as None in the
+                # working copy as compared to the repository.
+                # Also, exclude root as mention in the above fast path.
+                changes = filter(
+                    lambda c: c[6][0] != 'symlink' and c[4] != (None, None),
+                    changes)
+                try:
+                    next(iter(changes))
+                except StopIteration:
+                    return False
+                return True
+
+
 
 class WorkingTreeFormatMetaDir(bzrdir.BzrFormat, WorkingTreeFormat):
     """Base class for working trees that live in bzr meta directories."""

=== modified file 'breezy/bzr/workingtree_4.py'
--- breezy/bzr/workingtree_4.py	2020-07-18 23:14:00 +0000
+++ breezy/bzr/workingtree_4.py	2020-08-10 15:00:17 +0000
@@ -2157,16 +2157,15 @@
                 relroot = ""
             # FIXME: stash the node in pending
             entry = inv.get_entry(file_id)
+            subdirs = []
             for name, child in entry.sorted_children():
                 toppath = relroot + name
-                dirblock.append((toppath, name, child.kind, None,
-                                 child.file_id, child.kind
-                                 ))
-            yield (relpath, entry.file_id), dirblock
+                dirblock.append((toppath, name, child.kind, None, child.kind))
+                if child.kind == _directory:
+                    subdirs.append((toppath, child.file_id))
+            yield relpath, dirblock
             # push the user specified dirs from dirblock
-            for dir in reversed(dirblock):
-                if dir[2] == _directory:
-                    pending.append((dir[0], dir[4]))
+            pending.extend(reversed(subdirs))
 
 
 class InterDirStateTree(InterInventoryTree):

=== modified file 'breezy/commit.py'
--- breezy/commit.py	2020-06-19 21:26:53 +0000
+++ breezy/commit.py	2020-08-10 15:00:17 +0000
@@ -67,7 +67,6 @@
                       minimum_path_selection,
                       )
 from .trace import mutter, note, is_quiet
-from .tree import TreeChange
 from .urlutils import unescape_for_display
 from .i18n import gettext
 

=== modified file 'breezy/conflicts.py'
--- breezy/conflicts.py	2020-07-18 23:14:00 +0000
+++ breezy/conflicts.py	2020-08-10 15:00:17 +0000
@@ -17,18 +17,14 @@
 # TODO: 'brz resolve' should accept a directory name and work from that
 # point down
 
+import errno
 import os
 import re
 
 from .lazy_import import lazy_import
 lazy_import(globals(), """
-import errno
 
 from breezy import (
-    osutils,
-    rio,
-    trace,
-    transform,
     workingtree,
     )
 from breezy.i18n import gettext, ngettext
@@ -38,13 +34,12 @@
     errors,
     commands,
     option,
+    osutils,
     registry,
+    trace,
     )
 
 
-CONFLICT_SUFFIXES = ('.THIS', '.BASE', '.OTHER')
-
-
 class cmd_conflicts(commands.Command):
     __doc__ = """List files with conflicts.
 
@@ -127,7 +122,7 @@
         if all:
             if file_list:
                 raise errors.CommandError(gettext("If --all is specified,"
-                                                     " no FILE may be provided"))
+                                                  " no FILE may be provided"))
             if directory is None:
                 directory = u'.'
             tree = workingtree.WorkingTree.open_containing(directory)[0]
@@ -182,14 +177,14 @@
         tree_conflicts = tree.conflicts()
         nb_conflicts_before = len(tree_conflicts)
         if paths is None:
-            new_conflicts = ConflictList()
+            new_conflicts = []
             to_process = tree_conflicts
         else:
             new_conflicts, to_process = tree_conflicts.select_conflicts(
                 tree, paths, ignore_misses, recursive)
         for conflict in to_process:
             try:
-                conflict._do(action, tree)
+                conflict.do(action, tree)
                 conflict.cleanup(tree)
             except NotImplementedError:
                 new_conflicts.append(conflict)
@@ -235,8 +230,6 @@
     """List of conflicts.
 
     Typically obtained from WorkingTree.conflicts()
-
-    Can be instantiated from stanzas or from Conflict subclasses.
     """
 
     def __init__(self, conflicts=None):
@@ -270,19 +263,6 @@
     def __repr__(self):
         return "ConflictList(%r)" % self.__list
 
-    @staticmethod
-    def from_stanzas(stanzas):
-        """Produce a new ConflictList from an iterable of stanzas"""
-        conflicts = ConflictList()
-        for stanza in stanzas:
-            conflicts.append(Conflict.factory(**stanza.as_dict()))
-        return conflicts
-
-    def to_stanzas(self):
-        """Generator of stanzas"""
-        for conflict in self:
-            yield conflict.as_stanza()
-
     def to_strings(self):
         """Generate strings for the provided conflicts"""
         for conflict in self:
@@ -299,43 +279,23 @@
                          recurse=False):
         """Select the conflicts associated with paths in a tree.
 
-        File-ids are also used for this.
         :return: a pair of ConflictLists: (not_selected, selected)
         """
         path_set = set(paths)
-        ids = {}
         selected_paths = set()
         new_conflicts = ConflictList()
         selected_conflicts = ConflictList()
-        for path in paths:
-            file_id = tree.path2id(path)
-            if file_id is not None:
-                ids[file_id] = path
 
         for conflict in self:
             selected = False
-            for key in ('path', 'conflict_path'):
-                cpath = getattr(conflict, key, None)
-                if cpath is None:
-                    continue
-                if cpath in path_set:
+            if conflict.path in path_set:
+                selected = True
+                selected_paths.add(conflict.path)
+            if recurse:
+                if osutils.is_inside_any(path_set, conflict.path):
                     selected = True
-                    selected_paths.add(cpath)
-                if recurse:
-                    if osutils.is_inside_any(path_set, cpath):
-                        selected = True
-                        selected_paths.add(cpath)
+                    selected_paths.add(conflict.path)
 
-            for key in ('file_id', 'conflict_file_id'):
-                cfile_id = getattr(conflict, key, None)
-                if cfile_id is None:
-                    continue
-                try:
-                    cpath = ids[cfile_id]
-                except KeyError:
-                    continue
-                selected = True
-                selected_paths.add(cpath)
             if selected:
                 selected_conflicts.append(conflict)
             else:
@@ -350,84 +310,12 @@
 
 
 class Conflict(object):
-    """Base class for all types of conflict"""
-
-    # FIXME: cleanup should take care of that ? -- vila 091229
-    has_files = False
-
-    def __init__(self, path, file_id=None):
+    """Base class for conflicts."""
+
+    typestring = None
+
+    def __init__(self, path):
         self.path = path
-        # the factory blindly transfers the Stanza values to __init__ and
-        # Stanza is purely a Unicode api.
-        if isinstance(file_id, str):
-            file_id = cache_utf8.encode(file_id)
-        self.file_id = file_id
-
-    def as_stanza(self):
-        s = rio.Stanza(type=self.typestring, path=self.path)
-        if self.file_id is not None:
-            # Stanza requires Unicode apis
-            s.add('file_id', self.file_id.decode('utf8'))
-        return s
-
-    def _cmp_list(self):
-        return [type(self), self.path, self.file_id]
-
-    def __cmp__(self, other):
-        if getattr(other, "_cmp_list", None) is None:
-            return -1
-        x = self._cmp_list()
-        y = other._cmp_list()
-        return (x > y) - (x < y)
-
-    def __hash__(self):
-        return hash((type(self), self.path, self.file_id))
-
-    def __eq__(self, other):
-        return self.__cmp__(other) == 0
-
-    def __ne__(self, other):
-        return not self.__eq__(other)
-
-    def __unicode__(self):
-        return self.describe()
-
-    def __str__(self):
-        return self.describe()
-
-    def describe(self):
-        return self.format % self.__dict__
-
-    def __repr__(self):
-        rdict = dict(self.__dict__)
-        rdict['class'] = self.__class__.__name__
-        return self.rformat % rdict
-
-    @staticmethod
-    def factory(type, **kwargs):
-        global ctype
-        return ctype[type](**kwargs)
-
-    @staticmethod
-    def sort_key(conflict):
-        if conflict.path is not None:
-            return conflict.path, conflict.typestring
-        elif getattr(conflict, "conflict_path", None) is not None:
-            return conflict.conflict_path, conflict.typestring
-        else:
-            return None, conflict.typestring
-
-    def _do(self, action, tree):
-        """Apply the specified action to the conflict.
-
-        :param action: The method name to call.
-
-        :param tree: The tree passed as a parameter to the method.
-        """
-        meth = getattr(self, 'action_%s' % action, None)
-        if meth is None:
-            raise NotImplementedError(self.__class__.__name__ + '.' + action)
-        meth(tree)
 
     def associated_filenames(self):
         """The names of the files generated to help resolve the conflict."""
@@ -441,472 +329,14 @@
                 if e.errno != errno.ENOENT:
                     raise
 
-    def action_auto(self, tree):
-        raise NotImplementedError(self.action_auto)
-
-    def action_done(self, tree):
-        """Mark the conflict as solved once it has been handled."""
-        # This method does nothing but simplifies the design of upper levels.
-        pass
-
-    def action_take_this(self, tree):
-        raise NotImplementedError(self.action_take_this)
-
-    def action_take_other(self, tree):
-        raise NotImplementedError(self.action_take_other)
-
-    def _resolve_with_cleanups(self, tree, *args, **kwargs):
-        with tree.transform() as tt:
-            self._resolve(tt, *args, **kwargs)
-
-
-class PathConflict(Conflict):
-    """A conflict was encountered merging file paths"""
-
-    typestring = 'path conflict'
-
-    format = 'Path conflict: %(path)s / %(conflict_path)s'
-
-    rformat = '%(class)s(%(path)r, %(conflict_path)r, %(file_id)r)'
-
-    def __init__(self, path, conflict_path=None, file_id=None):
-        Conflict.__init__(self, path, file_id)
-        self.conflict_path = conflict_path
-
-    def as_stanza(self):
-        s = Conflict.as_stanza(self)
-        if self.conflict_path is not None:
-            s.add('conflict_path', self.conflict_path)
-        return s
-
-    def associated_filenames(self):
-        # No additional files have been generated here
-        return []
-
-    def _resolve(self, tt, file_id, path, winner):
-        """Resolve the conflict.
-
-        :param tt: The TreeTransform where the conflict is resolved.
-        :param file_id: The retained file id.
-        :param path: The retained path.
-        :param winner: 'this' or 'other' indicates which side is the winner.
-        """
-        path_to_create = None
-        if winner == 'this':
-            if self.path == '<deleted>':
-                return  # Nothing to do
-            if self.conflict_path == '<deleted>':
-                path_to_create = self.path
-                revid = tt._tree.get_parent_ids()[0]
-        elif winner == 'other':
-            if self.conflict_path == '<deleted>':
-                return  # Nothing to do
-            if self.path == '<deleted>':
-                path_to_create = self.conflict_path
-                # FIXME: If there are more than two parents we may need to
-                # iterate. Taking the last parent is the safer bet in the mean
-                # time. -- vila 20100309
-                revid = tt._tree.get_parent_ids()[-1]
-        else:
-            # Programmer error
-            raise AssertionError('bad winner: %r' % (winner,))
-        if path_to_create is not None:
-            tid = tt.trans_id_tree_path(path_to_create)
-            tree = self._revision_tree(tt._tree, revid)
-            transform.create_from_tree(
-                tt, tid, tree, tree.id2path(file_id))
-            tt.version_file(tid, file_id=file_id)
-        else:
-            tid = tt.trans_id_file_id(file_id)
-        # Adjust the path for the retained file id
-        parent_tid = tt.get_tree_parent(tid)
-        tt.adjust_path(osutils.basename(path), parent_tid, tid)
-        tt.apply()
-
-    def _revision_tree(self, tree, revid):
-        return tree.branch.repository.revision_tree(revid)
-
-    def _infer_file_id(self, tree):
-        # Prior to bug #531967, file_id wasn't always set, there may still be
-        # conflict files in the wild so we need to cope with them
-        # Establish which path we should use to find back the file-id
-        possible_paths = []
-        for p in (self.path, self.conflict_path):
-            if p == '<deleted>':
-                # special hard-coded path
-                continue
-            if p is not None:
-                possible_paths.append(p)
-        # Search the file-id in the parents with any path available
-        file_id = None
-        for revid in tree.get_parent_ids():
-            revtree = self._revision_tree(tree, revid)
-            for p in possible_paths:
-                file_id = revtree.path2id(p)
-                if file_id is not None:
-                    return revtree, file_id
-        return None, None
-
-    def action_take_this(self, tree):
-        if self.file_id is not None:
-            self._resolve_with_cleanups(tree, self.file_id, self.path,
-                                        winner='this')
-        else:
-            # Prior to bug #531967 we need to find back the file_id and restore
-            # the content from there
-            revtree, file_id = self._infer_file_id(tree)
-            tree.revert([revtree.id2path(file_id)],
-                        old_tree=revtree, backups=False)
-
-    def action_take_other(self, tree):
-        if self.file_id is not None:
-            self._resolve_with_cleanups(tree, self.file_id,
-                                        self.conflict_path,
-                                        winner='other')
-        else:
-            # Prior to bug #531967 we need to find back the file_id and restore
-            # the content from there
-            revtree, file_id = self._infer_file_id(tree)
-            tree.revert([revtree.id2path(file_id)],
-                        old_tree=revtree, backups=False)
-
-
-class ContentsConflict(PathConflict):
-    """The files are of different types (or both binary), or not present"""
-
-    has_files = True
-
-    typestring = 'contents conflict'
-
-    format = 'Contents conflict in %(path)s'
-
-    def associated_filenames(self):
-        return [self.path + suffix for suffix in ('.BASE', '.OTHER')]
-
-    def _resolve(self, tt, suffix_to_remove):
-        """Resolve the conflict.
-
-        :param tt: The TreeTransform where the conflict is resolved.
-        :param suffix_to_remove: Either 'THIS' or 'OTHER'
-
-        The resolution is symmetric: when taking THIS, OTHER is deleted and
-        item.THIS is renamed into item and vice-versa.
-        """
-        try:
-            # Delete 'item.THIS' or 'item.OTHER' depending on
-            # suffix_to_remove
-            tt.delete_contents(
-                tt.trans_id_tree_path(self.path + '.' + suffix_to_remove))
-        except errors.NoSuchFile:
-            # There are valid cases where 'item.suffix_to_remove' either
-            # never existed or was already deleted (including the case
-            # where the user deleted it)
-            pass
-        try:
-            this_path = tt._tree.id2path(self.file_id)
-        except errors.NoSuchId:
-            # The file is not present anymore. This may happen if the user
-            # deleted the file either manually or when resolving a conflict on
-            # the parent.  We may raise some exception to indicate that the
-            # conflict doesn't exist anymore and as such doesn't need to be
-            # resolved ? -- vila 20110615
-            this_tid = None
-        else:
-            this_tid = tt.trans_id_tree_path(this_path)
-        if this_tid is not None:
-            # Rename 'item.suffix_to_remove' (note that if
-            # 'item.suffix_to_remove' has been deleted, this is a no-op)
-            parent_tid = tt.get_tree_parent(this_tid)
-            tt.adjust_path(osutils.basename(self.path), parent_tid, this_tid)
-            tt.apply()
-
-    def action_take_this(self, tree):
-        self._resolve_with_cleanups(tree, 'OTHER')
-
-    def action_take_other(self, tree):
-        self._resolve_with_cleanups(tree, 'THIS')
-
-
-# TODO: There should be a base revid attribute to better inform the user about
-# how the conflicts were generated.
-class TextConflict(Conflict):
-    """The merge algorithm could not resolve all differences encountered."""
-
-    has_files = True
-
-    typestring = 'text conflict'
-
-    format = 'Text conflict in %(path)s'
-
-    rformat = '%(class)s(%(path)r, %(file_id)r)'
-
-    _conflict_re = re.compile(b'^(<{7}|={7}|>{7})')
-
-    def associated_filenames(self):
-        return [self.path + suffix for suffix in CONFLICT_SUFFIXES]
-
-    def _resolve(self, tt, winner_suffix):
-        """Resolve the conflict by copying one of .THIS or .OTHER into file.
-
-        :param tt: The TreeTransform where the conflict is resolved.
-        :param winner_suffix: Either 'THIS' or 'OTHER'
-
-        The resolution is symmetric, when taking THIS, item.THIS is renamed
-        into item and vice-versa. This takes one of the files as a whole
-        ignoring every difference that could have been merged cleanly.
-        """
-        # To avoid useless copies, we switch item and item.winner_suffix, only
-        # item will exist after the conflict has been resolved anyway.
-        item_tid = tt.trans_id_file_id(self.file_id)
-        item_parent_tid = tt.get_tree_parent(item_tid)
-        winner_path = self.path + '.' + winner_suffix
-        winner_tid = tt.trans_id_tree_path(winner_path)
-        winner_parent_tid = tt.get_tree_parent(winner_tid)
-        # Switch the paths to preserve the content
-        tt.adjust_path(osutils.basename(self.path),
-                       winner_parent_tid, winner_tid)
-        tt.adjust_path(osutils.basename(winner_path),
-                       item_parent_tid, item_tid)
-        # Associate the file_id to the right content
-        tt.unversion_file(item_tid)
-        tt.version_file(winner_tid, file_id=self.file_id)
-        tt.apply()
-
-    def action_auto(self, tree):
-        # GZ 2012-07-27: Using NotImplementedError to signal that a conflict
-        #                can't be auto resolved does not seem ideal.
-        try:
-            kind = tree.kind(self.path)
-        except errors.NoSuchFile:
-            return
-        if kind != 'file':
-            raise NotImplementedError("Conflict is not a file")
-        conflict_markers_in_line = self._conflict_re.search
-        # GZ 2012-07-27: What if not tree.has_id(self.file_id) due to removal?
-        with tree.get_file(self.path) as f:
-            for line in f:
-                if conflict_markers_in_line(line):
-                    raise NotImplementedError("Conflict markers present")
-
-    def action_take_this(self, tree):
-        self._resolve_with_cleanups(tree, 'THIS')
-
-    def action_take_other(self, tree):
-        self._resolve_with_cleanups(tree, 'OTHER')
-
-
-class HandledConflict(Conflict):
-    """A path problem that has been provisionally resolved.
-    This is intended to be a base class.
-    """
-
-    rformat = "%(class)s(%(action)r, %(path)r, %(file_id)r)"
-
-    def __init__(self, action, path, file_id=None):
-        Conflict.__init__(self, path, file_id)
-        self.action = action
-
-    def _cmp_list(self):
-        return Conflict._cmp_list(self) + [self.action]
-
-    def as_stanza(self):
-        s = Conflict.as_stanza(self)
-        s.add('action', self.action)
-        return s
-
-    def associated_filenames(self):
-        # Nothing has been generated here
-        return []
-
-
-class HandledPathConflict(HandledConflict):
-    """A provisionally-resolved path problem involving two paths.
-    This is intended to be a base class.
-    """
-
-    rformat = "%(class)s(%(action)r, %(path)r, %(conflict_path)r,"\
-        " %(file_id)r, %(conflict_file_id)r)"
-
-    def __init__(self, action, path, conflict_path, file_id=None,
-                 conflict_file_id=None):
-        HandledConflict.__init__(self, action, path, file_id)
-        self.conflict_path = conflict_path
-        # the factory blindly transfers the Stanza values to __init__,
-        # so they can be unicode.
-        if isinstance(conflict_file_id, str):
-            conflict_file_id = cache_utf8.encode(conflict_file_id)
-        self.conflict_file_id = conflict_file_id
-
-    def _cmp_list(self):
-        return HandledConflict._cmp_list(self) + [self.conflict_path,
-                                                  self.conflict_file_id]
-
-    def as_stanza(self):
-        s = HandledConflict.as_stanza(self)
-        s.add('conflict_path', self.conflict_path)
-        if self.conflict_file_id is not None:
-            s.add('conflict_file_id', self.conflict_file_id.decode('utf8'))
-
-        return s
-
-
-class DuplicateID(HandledPathConflict):
-    """Two files want the same file_id."""
-
-    typestring = 'duplicate id'
-
-    format = 'Conflict adding id to %(conflict_path)s.  %(action)s %(path)s.'
-
-
-class DuplicateEntry(HandledPathConflict):
-    """Two directory entries want to have the same name."""
-
-    typestring = 'duplicate'
-
-    format = 'Conflict adding file %(conflict_path)s.  %(action)s %(path)s.'
-
-    def action_take_this(self, tree):
-        tree.remove([self.conflict_path], force=True, keep_files=False)
-        tree.rename_one(self.path, self.conflict_path)
-
-    def action_take_other(self, tree):
-        tree.remove([self.path], force=True, keep_files=False)
-
-
-class ParentLoop(HandledPathConflict):
-    """An attempt to create an infinitely-looping directory structure.
-    This is rare, but can be produced like so:
-
-    tree A:
-      mv foo bar
-    tree B:
-      mv bar foo
-    merge A and B
-    """
-
-    typestring = 'parent loop'
-
-    format = 'Conflict moving %(path)s into %(conflict_path)s. %(action)s.'
-
-    def action_take_this(self, tree):
-        # just acccept brz proposal
-        pass
-
-    def action_take_other(self, tree):
-        with tree.transform() as tt:
-            p_tid = tt.trans_id_file_id(self.file_id)
-            parent_tid = tt.get_tree_parent(p_tid)
-            cp_tid = tt.trans_id_file_id(self.conflict_file_id)
-            cparent_tid = tt.get_tree_parent(cp_tid)
-            tt.adjust_path(osutils.basename(self.path), cparent_tid, cp_tid)
-            tt.adjust_path(osutils.basename(self.conflict_path),
-                           parent_tid, p_tid)
-            tt.apply()
-
-
-class UnversionedParent(HandledConflict):
-    """An attempt to version a file whose parent directory is not versioned.
-    Typically, the result of a merge where one tree unversioned the directory
-    and the other added a versioned file to it.
-    """
-
-    typestring = 'unversioned parent'
-
-    format = 'Conflict because %(path)s is not versioned, but has versioned'\
-             ' children.  %(action)s.'
-
-    # FIXME: We silently do nothing to make tests pass, but most probably the
-    # conflict shouldn't exist (the long story is that the conflict is
-    # generated with another one that can be resolved properly) -- vila 091224
-    def action_take_this(self, tree):
-        pass
-
-    def action_take_other(self, tree):
-        pass
-
-
-class MissingParent(HandledConflict):
-    """An attempt to add files to a directory that is not present.
-    Typically, the result of a merge where THIS deleted the directory and
-    the OTHER added a file to it.
-    See also: DeletingParent (same situation, THIS and OTHER reversed)
-    """
-
-    typestring = 'missing parent'
-
-    format = 'Conflict adding files to %(path)s.  %(action)s.'
-
-    def action_take_this(self, tree):
-        tree.remove([self.path], force=True, keep_files=False)
-
-    def action_take_other(self, tree):
-        # just acccept brz proposal
-        pass
-
-
-class DeletingParent(HandledConflict):
-    """An attempt to add files to a directory that is not present.
-    Typically, the result of a merge where one OTHER deleted the directory and
-    the THIS added a file to it.
-    """
-
-    typestring = 'deleting parent'
-
-    format = "Conflict: can't delete %(path)s because it is not empty.  "\
-             "%(action)s."
-
-    # FIXME: It's a bit strange that the default action is not coherent with
-    # MissingParent from the *user* pov.
-
-    def action_take_this(self, tree):
-        # just acccept brz proposal
-        pass
-
-    def action_take_other(self, tree):
-        tree.remove([self.path], force=True, keep_files=False)
-
-
-class NonDirectoryParent(HandledConflict):
-    """An attempt to add files to a directory that is not a directory or
-    an attempt to change the kind of a directory with files.
-    """
-
-    typestring = 'non-directory parent'
-
-    format = "Conflict: %(path)s is not a directory, but has files in it."\
-             "  %(action)s."
-
-    # FIXME: .OTHER should be used instead of .new when the conflict is created
-
-    def action_take_this(self, tree):
-        # FIXME: we should preserve that path when the conflict is generated !
-        if self.path.endswith('.new'):
-            conflict_path = self.path[:-(len('.new'))]
-            tree.remove([self.path], force=True, keep_files=False)
-            tree.add(conflict_path)
-        else:
-            raise NotImplementedError(self.action_take_this)
-
-    def action_take_other(self, tree):
-        # FIXME: we should preserve that path when the conflict is generated !
-        if self.path.endswith('.new'):
-            conflict_path = self.path[:-(len('.new'))]
-            tree.remove([conflict_path], force=True, keep_files=False)
-            tree.rename_one(self.path, conflict_path)
-        else:
-            raise NotImplementedError(self.action_take_other)
-
-
-ctype = {}
-
-
-def register_types(*conflict_types):
-    """Register a Conflict subclass for serialization purposes"""
-    global ctype
-    for conflict_type in conflict_types:
-        ctype[conflict_type.typestring] = conflict_type
-
-
-register_types(ContentsConflict, TextConflict, PathConflict, DuplicateID,
-               DuplicateEntry, ParentLoop, UnversionedParent, MissingParent,
-               DeletingParent, NonDirectoryParent)
+    def do(self, action, tree):
+        """Apply the specified action to the conflict.
+
+        :param action: The method name to call.
+
+        :param tree: The tree passed as a parameter to the method.
+        """
+        raise NotImplementedError(self.do)
+
+    def describe(self):
+        raise NotImplementedError(self.describe)

=== modified file 'breezy/delta.py'
--- breezy/delta.py	2020-02-18 01:57:45 +0000
+++ breezy/delta.py	2020-08-10 15:00:17 +0000
@@ -20,7 +20,7 @@
     osutils,
     trace,
     )
-from .tree import TreeChange
+from .bzr.inventorytree import InventoryTreeChange
 
 
 class TreeDelta(object):
@@ -397,7 +397,7 @@
         dec_new_path = decorate_path(item.path[1], item.kind[1], item.meta_modified())
         to_file.write(' => %s' % dec_new_path)
         if item.changed_content or item.meta_modified():
-            extra_modified.append(TreeChange(
+            extra_modified.append(InventoryTreeChange(
                 item.file_id, (item.path[1], item.path[1]),
                 item.changed_content,
                 item.versioned,

=== modified file 'breezy/errors.py'
--- breezy/errors.py	2020-07-28 02:11:05 +0000
+++ breezy/errors.py	2020-08-10 15:00:17 +0000
@@ -1282,15 +1282,15 @@
 
 class UnexpectedHttpStatus(InvalidHttpResponse):
 
-    _fmt = "Unexpected HTTP status %(code)d for %(path)s"
+    _fmt = "Unexpected HTTP status %(code)d for %(path)s: %(extra)s"
 
-    def __init__(self, path, code, msg=None):
+    def __init__(self, path, code, extra=None):
         self.path = path
         self.code = code
-        self.msg = msg
+        self.extra = extra or ''
         full_msg = 'status code %d unexpected' % code
-        if msg is not None:
-            full_msg += ': ' + msg
+        if extra is not None:
+            full_msg += ': ' + extra
         InvalidHttpResponse.__init__(
             self, path, full_msg)
 

=== modified file 'breezy/git/commit.py'
--- breezy/git/commit.py	2020-07-18 23:14:00 +0000
+++ breezy/git/commit.py	2020-08-10 15:00:17 +0000
@@ -75,11 +75,21 @@
     def record_iter_changes(self, workingtree, basis_revid, iter_changes):
         seen_root = False
         for change in iter_changes:
+            if change.versioned[0] and not change.copied:
+                file_id = self._mapping.generate_file_id(change.path[0])
+            elif change.versioned[1]:
+                file_id = self._mapping.generate_file_id(change.path[1])
+            else:
+                file_id = None
+            if change.path[1]:
+                parent_id_new = self._mapping.generate_file_id(osutils.dirname(change.path[1]))
+            else:
+                parent_id_new = None
             if change.kind[1] in ("directory",):
                 self._inv_delta.append(
-                    (change.path[0], change.path[1], change.file_id,
+                    (change.path[0], change.path[1], file_id,
                      entry_factory[change.kind[1]](
-                         change.file_id, change.name[1], change.parent_id[1])))
+                         file_id, change.name[1], parent_id_new)))
                 if change.kind[0] in ("file", "symlink"):
                     self._blobs[encode_git_path(change.path[0])] = None
                     self._any_changes = True
@@ -88,14 +98,14 @@
                 continue
             self._any_changes = True
             if change.path[1] is None:
-                self._inv_delta.append((change.path[0], change.path[1], change.file_id, None))
+                self._inv_delta.append((change.path[0], change.path[1], file_id, None))
                 self._deleted_paths.add(encode_git_path(change.path[0]))
                 continue
             try:
                 entry_kls = entry_factory[change.kind[1]]
             except KeyError:
                 raise KeyError("unknown kind %s" % change.kind[1])
-            entry = entry_kls(change.file_id, change.name[1], change.parent_id[1])
+            entry = entry_kls(file_id, change.name[1], parent_id_new)
             if change.kind[1] == "file":
                 entry.executable = change.executable[1]
                 blob = Blob()
@@ -104,10 +114,10 @@
                     blob.data = f.read()
                 finally:
                     f.close()
-                entry.text_size = len(blob.data)
-                entry.text_sha1 = osutils.sha_string(blob.data)
+                sha = blob.id
+                entry.text_size = st.st_size
+                entry.git_sha1 = sha
                 self.store.add_object(blob)
-                sha = blob.id
             elif change.kind[1] == "symlink":
                 symlink_target = workingtree.get_symlink_target(change.path[1])
                 blob = Blob()
@@ -124,12 +134,12 @@
             else:
                 raise AssertionError("Unknown kind %r" % change.kind[1])
             mode = object_mode(change.kind[1], change.executable[1])
-            self._inv_delta.append((change.path[0], change.path[1], change.file_id, entry))
+            self._inv_delta.append((change.path[0], change.path[1], file_id, entry))
             if change.path[0] is not None:
                 self._deleted_paths.add(encode_git_path(change.path[0]))
             self._blobs[encode_git_path(change.path[1])] = (mode, sha)
             if st is not None:
-                yield change.path[1], (entry.text_sha1, st)
+                yield change.path[1], (entry.git_sha1, st)
         if not seen_root and len(self.parents) == 0:
             raise RootMissing()
         if getattr(workingtree, "basis_tree", False):

=== modified file 'breezy/git/remote.py'
--- breezy/git/remote.py	2020-07-18 23:14:00 +0000
+++ breezy/git/remote.py	2020-08-10 15:00:17 +0000
@@ -201,6 +201,8 @@
         return PermissionDenied(url, message)
     if message.endswith(' does not appear to be a git repository'):
         return NotBranchError(url, message)
+    if message == 'pre-receive hook declined':
+        return PermissionDenied(url, message)
     if re.match('(.+) is not a valid repository name',
                 message.splitlines()[0]):
         return NotBranchError(url, message)
@@ -600,11 +602,16 @@
         push_result.branch_push_result = None
         repo = self.find_repository()
         refname = self._get_selected_ref(name)
-        ref_chain, old_sha = self.get_refs_container().follow(refname)
-        if ref_chain:
-            actual_refname = ref_chain[-1]
-        else:
+        try:
+            ref_chain, old_sha = self.get_refs_container().follow(refname)
+        except NotBranchError:
             actual_refname = refname
+            old_sha = None
+        else:
+            if ref_chain:
+                actual_refname = ref_chain[-1]
+            else:
+                actual_refname = refname
         if isinstance(source, GitBranch) and lossy:
             raise errors.LossyPushToSameVCS(source.controldir, self)
         source_store = get_object_store(source.repository)
@@ -624,6 +631,7 @@
                     raise errors.NoRoundtrippingSupport(
                         source, self.open_branch(name=name, nascent_ok=True))
             if not overwrite:
+                old_sha = remote_refs.get(actual_refname)
                 if remote_divergence(old_sha, new_sha, source_store):
                     raise DivergedBranches(
                         source, self.open_branch(name, nascent_ok=True))

=== modified file 'breezy/git/tests/test_remote.py'
--- breezy/git/tests/test_remote.py	2020-07-18 23:14:00 +0000
+++ breezy/git/tests/test_remote.py	2020-08-10 15:00:17 +0000
@@ -135,6 +135,14 @@
         self.assertEqual(e.path, 'porridge/gaduhistory.git')
         self.assertEqual(e.extra, ': denied to jelmer')
 
+    def test_pre_receive_hook_declined(self):
+        e = parse_git_error(
+            "url",
+            'pre-receive hook declined')
+        self.assertIsInstance(e, PermissionDenied)
+        self.assertEqual(e.path, "url")
+        self.assertEqual(e.extra, ': pre-receive hook declined')
+
     def test_invalid_repo_name(self):
         e = parse_git_error(
             "url",

=== modified file 'breezy/git/tests/test_tree.py'
--- breezy/git/tests/test_tree.py	2020-06-21 22:00:40 +0000
+++ breezy/git/tests/test_tree.py	2020-08-06 01:40:59 +0000
@@ -25,7 +25,7 @@
 from dulwich.objects import Tree, Blob
 
 from breezy.delta import TreeDelta
-from breezy.tree import TreeChange
+from breezy.bzr.inventorytree import InventoryTreeChange as TreeChange
 from breezy.git.tree import (
     changes_from_git_changes,
     tree_delta_from_git_changes,

=== modified file 'breezy/git/tests/test_workingtree.py'
--- breezy/git/tests/test_workingtree.py	2020-06-19 21:26:53 +0000
+++ breezy/git/tests/test_workingtree.py	2020-08-10 15:00:17 +0000
@@ -21,8 +21,11 @@
 import stat
 
 from dulwich import __version__ as dulwich_version
-from dulwich.diff_tree import RenameDetector
+from dulwich.diff_tree import RenameDetector, tree_changes
 from dulwich.index import IndexEntry
+from dulwich.object_store import (
+    OverlayObjectStore,
+    )
 from dulwich.objects import (
     S_IFGITLINK,
     Blob,
@@ -35,12 +38,11 @@
     workingtree as _mod_workingtree,
     )
 from ...delta import TreeDelta
-from ...tree import TreeChange
+from ...bzr.inventorytree import InventoryTreeChange as TreeChange
 from ..mapping import (
     default_mapping,
     )
 from ..tree import (
-    changes_between_git_tree_and_working_copy,
     tree_delta_from_git_changes,
     )
 from ..workingtree import (
@@ -52,6 +54,22 @@
     )
 
 
+def changes_between_git_tree_and_working_copy(source_store, from_tree_sha, target,
+                                              want_unchanged=False,
+                                              want_unversioned=False,
+                                              rename_detector=None,
+                                              include_trees=True):
+    """Determine the changes between a git tree and a working tree with index.
+
+    """
+    to_tree_sha, extras = target.git_snapshot(want_unversioned=want_unversioned)
+    store = OverlayObjectStore([source_store, target.store])
+    return tree_changes(
+        store, from_tree_sha, to_tree_sha, include_trees=include_trees,
+        rename_detector=rename_detector,
+        want_unchanged=want_unchanged, change_type_same=True), extras
+
+
 class GitWorkingTreeTests(TestCaseWithTransport):
 
     def setUp(self):

=== modified file 'breezy/git/transform.py'
--- breezy/git/transform.py	2020-07-20 02:02:23 +0000
+++ breezy/git/transform.py	2020-08-10 15:00:17 +0000
@@ -22,7 +22,7 @@
 from stat import S_ISREG
 import time
 
-from .. import errors, multiparent, osutils, trace, ui, urlutils
+from .. import conflicts, errors, multiparent, osutils, trace, ui, urlutils
 from ..i18n import gettext
 from ..mutabletree import MutableTree
 from ..transform import (
@@ -38,6 +38,7 @@
     ReusingTransform,
     MalformedTransform,
     )
+from ..bzr.inventorytree import InventoryTreeChange
 
 from ..bzr import inventory
 from ..bzr.transform import TransformPreview as GitTransformPreview
@@ -280,7 +281,7 @@
             if value == trans_id:
                 return key
 
-    def find_conflicts(self):
+    def find_raw_conflicts(self):
         """Find any violations of inventory or filesystem invariants"""
         if self._done is True:
             raise ReusingTransform()
@@ -289,7 +290,6 @@
         # all children of non-existent parents are known, by definition.
         self._add_tree_children()
         by_parent = self.by_parent()
-        conflicts.extend(self._unversioned_parents(by_parent))
         conflicts.extend(self._parent_loops())
         conflicts.extend(self._duplicate_entries(by_parent))
         conflicts.extend(self._parent_type_conflicts(by_parent))
@@ -299,7 +299,7 @@
         return conflicts
 
     def _check_malformed(self):
-        conflicts = self.find_conflicts()
+        conflicts = self.find_raw_conflicts()
         if len(conflicts) != 0:
             raise MalformedTransform(conflicts=conflicts)
 
@@ -385,20 +385,6 @@
                     break
         return conflicts
 
-    def _unversioned_parents(self, by_parent):
-        """If parent directories are versioned, children must be versioned."""
-        conflicts = []
-        for parent_id, children in by_parent.items():
-            if parent_id == ROOT_PARENT:
-                continue
-            if self.final_is_versioned(parent_id):
-                continue
-            for child_id in children:
-                if self.final_is_versioned(child_id):
-                    conflicts.append(('unversioned parent', parent_id))
-                    break
-        return conflicts
-
     def _improper_versioning(self):
         """Cannot version a file with no contents, or a bad type.
 
@@ -687,7 +673,7 @@
         """Produce output in the same format as Tree.iter_changes.
 
         Will produce nonsensical results if invoked while inventory/filesystem
-        conflicts (as reported by TreeTransform.find_conflicts()) are present.
+        conflicts (as reported by TreeTransform.find_raw_conflicts()) are present.
 
         This reads the Transform, but only reproduces changes involving a
         file_id.  Files that are not versioned in either of the FROM or TO
@@ -739,7 +725,7 @@
                     and from_executable == to_executable):
                 continue
             results.append(
-                TreeChange(
+                InventoryTreeChange(
                     file_id, (from_path, to_path), modified,
                     (from_versioned, to_versioned),
                     (from_parent, to_parent),
@@ -994,6 +980,25 @@
         """
         raise NotImplementedError(self.apply)
 
+    def cook_conflicts(self, raw_conflicts):
+        """Generate a list of cooked conflicts, sorted by file path"""
+        if not raw_conflicts:
+            return
+        fp = FinalPaths(self)
+        from .workingtree import TextConflict
+        for c in raw_conflicts:
+            if c[0] == 'text conflict':
+                yield TextConflict(fp.get_path(c[1]))
+            elif c[0] == 'duplicate':
+                yield TextConflict(fp.get_path(c[2]))
+            elif c[0] == 'contents conflict':
+                yield TextConflict(fp.get_path(c[1][0]))
+            elif c[0] == 'missing parent':
+                # TODO(jelmer): This should not make it to here
+                yield TextConflict(fp.get_path(c[2]))
+            else:
+                raise AssertionError('unknown conflict %s' % c[0])
+
 
 class DiskTreeTransform(TreeTransformBase):
     """Tree transform storing its contents on disk."""

=== modified file 'breezy/git/tree.py'
--- breezy/git/tree.py	2020-07-18 23:14:00 +0000
+++ breezy/git/tree.py	2020-08-10 15:00:17 +0000
@@ -76,18 +76,17 @@
     TransportObjectStore,
     TransportRepo,
     )
+from ..bzr.inventorytree import InventoryTreeChange
 
 
 class GitTreeDirectory(_mod_tree.TreeDirectory):
 
-    __slots__ = ['file_id', 'name', 'parent_id', 'children']
+    __slots__ = ['file_id', 'name', 'parent_id']
 
     def __init__(self, file_id, name, parent_id):
         self.file_id = file_id
         self.name = name
         self.parent_id = parent_id
-        # TODO(jelmer)
-        self.children = {}
 
     @property
     def kind(self):
@@ -115,16 +114,16 @@
 
 class GitTreeFile(_mod_tree.TreeFile):
 
-    __slots__ = ['file_id', 'name', 'parent_id', 'text_size', 'text_sha1',
-                 'executable']
+    __slots__ = ['file_id', 'name', 'parent_id', 'text_size',
+                 'executable', 'git_sha1']
 
     def __init__(self, file_id, name, parent_id, text_size=None,
-                 text_sha1=None, executable=None):
+                 git_sha1=None, executable=None):
         self.file_id = file_id
         self.name = name
         self.parent_id = parent_id
         self.text_size = text_size
-        self.text_sha1 = text_sha1
+        self.git_sha1 = git_sha1
         self.executable = executable
 
     @property
@@ -136,20 +135,20 @@
                 self.file_id == other.file_id and
                 self.name == other.name and
                 self.parent_id == other.parent_id and
-                self.text_sha1 == other.text_sha1 and
+                self.git_sha1 == other.git_sha1 and
                 self.text_size == other.text_size and
                 self.executable == other.executable)
 
     def __repr__(self):
         return ("%s(file_id=%r, name=%r, parent_id=%r, text_size=%r, "
-                "text_sha1=%r, executable=%r)") % (
+                "git_sha1=%r, executable=%r)") % (
             type(self).__name__, self.file_id, self.name, self.parent_id,
-            self.text_size, self.text_sha1, self.executable)
+            self.text_size, self.git_sha1, self.executable)
 
     def copy(self):
         ret = self.__class__(
             self.file_id, self.name, self.parent_id)
-        ret.text_sha1 = self.text_sha1
+        ret.git_sha1 = self.git_sha1
         ret.text_size = self.text_size
         ret.executable = self.executable
         return ret
@@ -257,7 +256,24 @@
     return path
 
 
-class GitRevisionTree(revisiontree.RevisionTree):
+class GitTree(_mod_tree.Tree):
+
+    def iter_git_objects(self):
+        """Iterate over all the objects in the tree.
+
+        :return :Yields tuples with (path, sha, mode)
+        """
+        raise NotImplementedError(self.iter_git_objects)
+
+    def git_snapshot(self, want_unversioned=False):
+        """Snapshot a tree, and return tree object.
+
+        :return: Tree sha and set of extras
+        """
+        raise NotImplementedError(self.snapshot)
+
+
+class GitRevisionTree(revisiontree.RevisionTree, GitTree):
     """Revision tree implementation based on Git objects."""
 
     def __init__(self, repository, revision_id):
@@ -279,6 +295,9 @@
                 raise errors.NoSuchRevision(repository, revision_id)
             self.tree = commit.tree
 
+    def git_snapshot(self, want_unversioned=False):
+        return self.tree, set()
+
     def _submodule_info(self):
         if self._submodules is None:
             try:
@@ -496,9 +515,8 @@
             ie.reference_revision = self.mapping.revision_id_foreign_to_bzr(
                 hexsha)
         else:
-            data = store[hexsha].data
-            ie.text_sha1 = osutils.sha_string(data)
-            ie.text_size = len(data)
+            ie.git_sha1 = hexsha
+            ie.text_size = None
             ie.executable = mode_is_executable(mode)
         return ie
 
@@ -699,9 +717,9 @@
     def walkdirs(self, prefix=u""):
         (store, mode, hexsha) = self._lookup_path(prefix)
         todo = deque(
-            [(store, encode_git_path(prefix), hexsha, self.path2id(prefix))])
+            [(store, encode_git_path(prefix), hexsha)])
         while todo:
-            store, path, tree_sha, parent_id = todo.popleft()
+            store, path, tree_sha = todo.popleft()
             path_decoded = decode_git_path(path)
             tree = store[tree_sha]
             children = []
@@ -709,14 +727,13 @@
                 if self.mapping.is_special_file(name):
                     continue
                 child_path = posixpath.join(path, name)
-                file_id = self.path2id(decode_git_path(child_path))
                 if stat.S_ISDIR(mode):
-                    todo.append((store, child_path, hexsha, file_id))
+                    todo.append((store, child_path, hexsha))
                 children.append(
                     (decode_git_path(child_path), decode_git_path(name),
                         mode_kind(mode), None,
-                        file_id, mode_kind(mode)))
-            yield (path_decoded, parent_id), children
+                        mode_kind(mode)))
+            yield path_decoded, children
 
     def preview_transform(self, pb=None):
         from .transform import GitTransformPreview
@@ -814,7 +831,7 @@
             newpath = None
         if oldpath is None and newpath is None:
             continue
-        change = _mod_tree.TreeChange(
+        change = InventoryTreeChange(
             fileid, (oldpath_decoded, newpath_decoded), (oldsha != newsha),
             (oldversioned, newversioned),
             (oldparent, newparent), (oldname, newname),
@@ -860,7 +877,7 @@
         parent_id = new_mapping.generate_file_id(parent_path)
         file_id = new_mapping.generate_file_id(path_decoded)
         ret.added.append(
-            _mod_tree.TreeChange(
+            InventoryTreeChange(
                 file_id, (None, path_decoded), True,
                 (False, True),
                 (None, parent_id),
@@ -956,7 +973,7 @@
             fileid = mapping.generate_file_id(newpath_decoded)
         else:
             fileid = None
-        yield _mod_tree.TreeChange(
+        yield InventoryTreeChange(
             fileid, (oldpath_decoded, newpath_decoded), (oldsha != newsha),
             (oldversioned, newversioned),
             (oldparent, newparent), (oldname, newname),
@@ -971,10 +988,18 @@
     _matching_to_tree_format = None
     _test_mutable_trees_to_test_trees = None
 
+    def __init__(self, source, target):
+        super(InterGitTrees, self).__init__(source, target)
+        if self.source.store == self.target.store:
+            self.store = self.source.store
+        else:
+            self.store = OverlayObjectStore(
+                [self.source.store, self.target.store])
+        self.rename_detector = RenameDetector(self.store)
+
     @classmethod
     def is_compatible(cls, source, target):
-        return (isinstance(source, GitRevisionTree) and
-                isinstance(target, GitRevisionTree))
+        return isinstance(source, GitTree) and isinstance(target, GitTree)
 
     def compare(self, want_unchanged=False, specific_files=None,
                 extra_trees=None, require_versioned=False, include_root=False,
@@ -1012,7 +1037,25 @@
     def _iter_git_changes(self, want_unchanged=False, specific_files=None,
                           require_versioned=False, extra_trees=None,
                           want_unversioned=False, include_trees=True):
-        raise NotImplementedError(self._iter_git_changes)
+        trees = [self.source]
+        if extra_trees is not None:
+            trees.extend(extra_trees)
+        if specific_files is not None:
+            specific_files = self.target.find_related_paths_across_trees(
+                specific_files, trees,
+                require_versioned=require_versioned)
+        # TODO(jelmer): Restrict to specific_files, for performance reasons.
+        with self.lock_read():
+            from_tree_sha, from_extras = self.source.git_snapshot(
+                want_unversioned=want_unversioned)
+            to_tree_sha, to_extras = self.target.git_snapshot(
+                want_unversioned=want_unversioned)
+            changes = tree_changes(
+                self.store, from_tree_sha, to_tree_sha,
+                include_trees=include_trees,
+                rename_detector=self.rename_detector,
+                want_unchanged=want_unchanged, change_type_same=True)
+            return changes, from_extras, to_extras
 
     def find_target_path(self, path, recurse='none'):
         ret = self.find_target_paths([path], recurse=recurse)
@@ -1067,48 +1110,10 @@
         return ret
 
 
-class InterGitRevisionTrees(InterGitTrees):
-    """InterTree that works between two git revision trees."""
-
-    _matching_from_tree_format = None
-    _matching_to_tree_format = None
-    _test_mutable_trees_to_test_trees = None
-
-    @classmethod
-    def is_compatible(cls, source, target):
-        return (isinstance(source, GitRevisionTree) and
-                isinstance(target, GitRevisionTree))
-
-    def _iter_git_changes(self, want_unchanged=False, specific_files=None,
-                          require_versioned=True, extra_trees=None,
-                          want_unversioned=False, include_trees=True):
-        trees = [self.source]
-        if extra_trees is not None:
-            trees.extend(extra_trees)
-        if specific_files is not None:
-            specific_files = self.target.find_related_paths_across_trees(
-                specific_files, trees,
-                require_versioned=require_versioned)
-
-        if (self.source._repository._git.object_store !=
-                self.target._repository._git.object_store):
-            store = OverlayObjectStore(
-                [self.source._repository._git.object_store,
-                    self.target._repository._git.object_store])
-        else:
-            store = self.source._repository._git.object_store
-        rename_detector = RenameDetector(store)
-        changes = tree_changes(
-            store, self.source.tree, self.target.tree,
-            want_unchanged=want_unchanged, include_trees=include_trees,
-            change_type_same=True, rename_detector=rename_detector)
-        return changes, set(), set()
-
-
-_mod_tree.InterTree.register_optimiser(InterGitRevisionTrees)
-
-
-class MutableGitIndexTree(mutabletree.MutableTree):
+_mod_tree.InterTree.register_optimiser(InterGitTrees)
+
+
+class MutableGitIndexTree(mutabletree.MutableTree, GitTree):
 
     def __init__(self):
         self._lock_mode = None
@@ -1117,6 +1122,9 @@
         self._index_dirty = False
         self._submodules = None
 
+    def git_snapshot(self, want_unversioned=False):
+        return snapshot_workingtree(self, want_unversioned=want_unversioned)
+
     def is_versioned(self, path):
         with self.lock_read():
             path = encode_git_path(path.rstrip('/'))
@@ -1136,7 +1144,7 @@
         if self._lock_mode is None:
             raise errors.ObjectNotLocked(self)
         self._versioned_dirs = set()
-        for p, i in self._recurse_index_entries():
+        for p, sha, mode in self.iter_git_objects():
             self._ensure_versioned_dir(posixpath.dirname(p))
 
     def _ensure_versioned_dir(self, dirname):
@@ -1295,6 +1303,10 @@
         if self._versioned_dirs is not None:
             self._ensure_versioned_dir(index_path)
 
+    def iter_git_objects(self):
+        for p, entry in self._recurse_index_entries():
+            yield p, entry.sha, entry.mode
+
     def _recurse_index_entries(self, index=None, basepath=b"",
                                recurse_nested=False):
         # Iterate over all index entries
@@ -1381,18 +1393,8 @@
         elif kind == 'tree-reference':
             ie.reference_revision = self.get_reference_revision(path)
         else:
-            try:
-                data = self.get_file_text(path)
-            except errors.NoSuchFile:
-                data = None
-            except IOError as e:
-                if e.errno != errno.ENOENT:
-                    raise
-                data = None
-            if data is None:
-                data = self.branch.repository._git.object_store[sha].data
-            ie.text_sha1 = osutils.sha_string(data)
-            ie.text_size = len(data)
+            ie.git_sha1 = sha
+            ie.text_size = size
             ie.executable = bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
         return ie
 
@@ -1636,130 +1638,49 @@
         from .transform import GitTransformPreview
         return GitTransformPreview(self, pb=pb)
 
-
-class InterToIndexGitTree(InterGitTrees):
-    """InterTree that works between a Git revision tree and an index."""
-
-    def __init__(self, source, target):
-        super(InterToIndexGitTree, self).__init__(source, target)
-        if self.source.store == self.target.store:
-            self.store = self.source.store
-        else:
-            self.store = OverlayObjectStore(
-                [self.source.store, self.target.store])
-        self.rename_detector = RenameDetector(self.store)
-
-    @classmethod
-    def is_compatible(cls, source, target):
-        return (isinstance(source, GitRevisionTree) and
-                isinstance(target, MutableGitIndexTree))
-
-    def _iter_git_changes(self, want_unchanged=False, specific_files=None,
-                          require_versioned=False, extra_trees=None,
-                          want_unversioned=False, include_trees=True):
-        trees = [self.source]
-        if extra_trees is not None:
-            trees.extend(extra_trees)
-        if specific_files is not None:
-            specific_files = self.target.find_related_paths_across_trees(
-                specific_files, trees,
-                require_versioned=require_versioned)
-        # TODO(jelmer): Restrict to specific_files, for performance reasons.
-        with self.lock_read():
-            changes, target_extras = changes_between_git_tree_and_working_copy(
-                self.source.store, self.source.tree,
-                self.target, want_unchanged=want_unchanged,
-                want_unversioned=want_unversioned,
-                rename_detector=self.rename_detector,
-                include_trees=include_trees)
-            return changes, set(), target_extras
-
-
-_mod_tree.InterTree.register_optimiser(InterToIndexGitTree)
-
-
-class InterFromIndexGitTree(InterGitTrees):
-    """InterTree that works between a Git revision tree and an index."""
-
-    def __init__(self, source, target):
-        super(InterFromIndexGitTree, self).__init__(source, target)
-        if self.source.store == self.target.store:
-            self.store = self.source.store
-        else:
-            self.store = OverlayObjectStore(
-                [self.source.store, self.target.store])
-        self.rename_detector = RenameDetector(self.store)
-
-    @classmethod
-    def is_compatible(cls, source, target):
-        return (isinstance(target, GitRevisionTree) and
-                isinstance(source, MutableGitIndexTree))
-
-    def _iter_git_changes(self, want_unchanged=False, specific_files=None,
-                          require_versioned=False, extra_trees=None,
-                          want_unversioned=False, include_trees=True):
-        trees = [self.source]
-        if extra_trees is not None:
-            trees.extend(extra_trees)
-        if specific_files is not None:
-            specific_files = self.target.find_related_paths_across_trees(
-                specific_files, trees,
-                require_versioned=require_versioned)
-        # TODO(jelmer): Restrict to specific_files, for performance reasons.
-        with self.lock_read():
-            from_tree_sha, extras = snapshot_workingtree(self.source, want_unversioned=want_unversioned)
-            return tree_changes(
-                self.store, from_tree_sha, self.target.tree,
-                include_trees=include_trees,
-                rename_detector=self.rename_detector,
-                want_unchanged=want_unchanged, change_type_same=True), extras
-
-
-_mod_tree.InterTree.register_optimiser(InterFromIndexGitTree)
-
-
-class InterIndexGitTree(InterGitTrees):
-    """InterTree that works between a Git revision tree and an index."""
-
-    def __init__(self, source, target):
-        super(InterIndexGitTree, self).__init__(source, target)
-        if self.source.store == self.target.store:
-            self.store = self.source.store
-        else:
-            self.store = OverlayObjectStore(
-                [self.source.store, self.target.store])
-        self.rename_detector = RenameDetector(self.store)
-
-    @classmethod
-    def is_compatible(cls, source, target):
-        return (isinstance(target, MutableGitIndexTree) and
-                isinstance(source, MutableGitIndexTree))
-
-    def _iter_git_changes(self, want_unchanged=False, specific_files=None,
-                          require_versioned=False, extra_trees=None,
-                          want_unversioned=False, include_trees=True):
-        trees = [self.source]
-        if extra_trees is not None:
-            trees.extend(extra_trees)
-        if specific_files is not None:
-            specific_files = self.target.find_related_paths_across_trees(
-                specific_files, trees,
-                require_versioned=require_versioned)
-        # TODO(jelmer): Restrict to specific_files, for performance reasons.
-        with self.lock_read():
-            from_tree_sha, from_extras = snapshot_workingtree(
-                self.source, want_unversioned=want_unversioned)
-            to_tree_sha, to_extras = snapshot_workingtree(
-                self.target, want_unversioned=want_unversioned)
-            changes = tree_changes(
-                self.store, from_tree_sha, to_tree_sha,
-                include_trees=include_trees,
-                rename_detector=self.rename_detector,
-                want_unchanged=want_unchanged, change_type_same=True)
-            return changes, from_extras, to_extras
-
-
-_mod_tree.InterTree.register_optimiser(InterIndexGitTree)
+    def has_changes(self, _from_tree=None):
+        """Quickly check that the tree contains at least one commitable change.
+
+        :param _from_tree: tree to compare against to find changes (default to
+            the basis tree and is intended to be used by tests).
+
+        :return: True if a change is found. False otherwise
+        """
+        with self.lock_read():
+            # Check pending merges
+            if len(self.get_parent_ids()) > 1:
+                return True
+            if _from_tree is None:
+                _from_tree = self.basis_tree()
+            changes = self.iter_changes(_from_tree)
+            if self.supports_symlinks():
+                # Fast path for has_changes.
+                try:
+                    change = next(changes)
+                    if change.path[1] == '':
+                        next(changes)
+                    return True
+                except StopIteration:
+                    # No changes
+                    return False
+            else:
+                # Slow path for has_changes.
+                # Handle platforms that do not support symlinks in the
+                # conditional below. This is slower than the try/except
+                # approach below that but we don't have a choice as we
+                # need to be sure that all symlinks are removed from the
+                # entire changeset. This is because in platforms that
+                # do not support symlinks, they show up as None in the
+                # working copy as compared to the repository.
+                # Also, exclude root as mention in the above fast path.
+                changes = filter(
+                    lambda c: c[6][0] != 'symlink' and c[4] != (None, None),
+                    changes)
+                try:
+                    next(iter(changes))
+                except StopIteration:
+                    return False
+                return True
 
 
 def snapshot_workingtree(target, want_unversioned=False):
@@ -1809,21 +1730,21 @@
                         target.store.add_object(blob)
                 blobs[path] = (live_entry.sha, cleanup_mode(live_entry.mode))
     if want_unversioned:
-        for e in target._iter_files_recursive(include_dirs=False):
+        for extra in target._iter_files_recursive(include_dirs=False):
             try:
-                e, accessible = osutils.normalized_filename(e)
+                extra, accessible = osutils.normalized_filename(extra)
             except UnicodeDecodeError:
                 raise errors.BadFilenameEncoding(
-                    e, osutils._fs_enc)
-            np = encode_git_path(e)
+                    extra, osutils._fs_enc)
+            np = encode_git_path(extra)
             if np in blobs:
                 continue
-            st = target._lstat(e)
+            st = target._lstat(extra)
             if stat.S_ISDIR(st.st_mode):
                 blob = Tree()
             elif stat.S_ISREG(st.st_mode) or stat.S_ISLNK(st.st_mode):
                 blob = blob_from_path_and_stat(
-                    target.abspath(e).encode(osutils._fs_enc), st)
+                    target.abspath(extra).encode(osutils._fs_enc), st)
             else:
                 continue
             target.store.add_object(blob)
@@ -1831,19 +1752,3 @@
             extras.add(np)
     return commit_tree(
         target.store, dirified + [(p, s, m) for (p, (s, m)) in blobs.items()]), extras
-
-
-def changes_between_git_tree_and_working_copy(source_store, from_tree_sha, target,
-                                              want_unchanged=False,
-                                              want_unversioned=False,
-                                              rename_detector=None,
-                                              include_trees=True):
-    """Determine the changes between a git tree and a working tree with index.
-
-    """
-    to_tree_sha, extras = snapshot_workingtree(target, want_unversioned=want_unversioned)
-    store = OverlayObjectStore([source_store, target.store])
-    return tree_changes(
-        store, from_tree_sha, to_tree_sha, include_trees=include_trees,
-        rename_detector=rename_detector,
-        want_unchanged=want_unchanged, change_type_same=True), extras

=== modified file 'breezy/git/workingtree.py'
--- breezy/git/workingtree.py	2020-07-18 23:14:00 +0000
+++ breezy/git/workingtree.py	2020-08-10 15:00:17 +0000
@@ -44,6 +44,7 @@
     )
 import os
 import posixpath
+import re
 import stat
 import sys
 
@@ -84,6 +85,95 @@
     )
 
 
+CONFLICT_SUFFIXES = ['.BASE', '.OTHER', '.THIS']
+
+
+# TODO: There should be a base revid attribute to better inform the user about
+# how the conflicts were generated.
+class TextConflict(_mod_conflicts.Conflict):
+    """The merge algorithm could not resolve all differences encountered."""
+
+    has_files = True
+
+    typestring = 'text conflict'
+
+    _conflict_re = re.compile(b'^(<{7}|={7}|>{7})')
+
+    def associated_filenames(self):
+        return [self.path + suffix for suffix in CONFLICT_SUFFIXES]
+
+    def _resolve(self, tt, winner_suffix):
+        """Resolve the conflict by copying one of .THIS or .OTHER into file.
+
+        :param tt: The TreeTransform where the conflict is resolved.
+        :param winner_suffix: Either 'THIS' or 'OTHER'
+
+        The resolution is symmetric, when taking THIS, item.THIS is renamed
+        into item and vice-versa. This takes one of the files as a whole
+        ignoring every difference that could have been merged cleanly.
+        """
+        # To avoid useless copies, we switch item and item.winner_suffix, only
+        # item will exist after the conflict has been resolved anyway.
+        item_tid = tt.trans_id_tree_path(self.path)
+        item_parent_tid = tt.get_tree_parent(item_tid)
+        winner_path = self.path + '.' + winner_suffix
+        winner_tid = tt.trans_id_tree_path(winner_path)
+        winner_parent_tid = tt.get_tree_parent(winner_tid)
+        # Switch the paths to preserve the content
+        tt.adjust_path(osutils.basename(self.path),
+                       winner_parent_tid, winner_tid)
+        tt.adjust_path(osutils.basename(winner_path),
+                       item_parent_tid, item_tid)
+        tt.unversion_file(item_tid)
+        tt.version_file(winner_tid)
+        tt.apply()
+
+    def action_auto(self, tree):
+        # GZ 2012-07-27: Using NotImplementedError to signal that a conflict
+        #                can't be auto resolved does not seem ideal.
+        try:
+            kind = tree.kind(self.path)
+        except errors.NoSuchFile:
+            return
+        if kind != 'file':
+            raise NotImplementedError("Conflict is not a file")
+        conflict_markers_in_line = self._conflict_re.search
+        with tree.get_file(self.path) as f:
+            for line in f:
+                if conflict_markers_in_line(line):
+                    raise NotImplementedError("Conflict markers present")
+
+    def _resolve_with_cleanups(self, tree, *args, **kwargs):
+        with tree.transform() as tt:
+            self._resolve(tt, *args, **kwargs)
+
+    def action_take_this(self, tree):
+        self._resolve_with_cleanups(tree, 'THIS')
+
+    def action_take_other(self, tree):
+        self._resolve_with_cleanups(tree, 'OTHER')
+
+    def do(self, action, tree):
+        """Apply the specified action to the conflict.
+
+        :param action: The method name to call.
+
+        :param tree: The tree passed as a parameter to the method.
+        """
+        meth = getattr(self, 'action_%s' % action, None)
+        if meth is None:
+            raise NotImplementedError(self.__class__.__name__ + '.' + action)
+        meth(tree)
+
+    def action_done(self, tree):
+        """Mark the conflict as solved once it has been handled."""
+        # This method does nothing but simplifies the design of upper levels.
+        pass
+
+    def describe(self):
+        return 'Text conflict in %(path)s' % self.__dict__
+
+
 class GitWorkingTree(MutableGitIndexTree, workingtree.WorkingTree):
     """A Git working tree."""
 
@@ -330,8 +420,8 @@
         def recurse_directory_to_add_files(directory):
             # Recurse directory and add all files
             # so we can check if they have changed.
-            for parent_info, file_infos in self.walkdirs(directory):
-                for relpath, basename, kind, lstat, fileid, kind in file_infos:
+            for parent_path, file_infos in self.walkdirs(directory):
+                for relpath, basename, kind, lstat, kind in file_infos:
                     # Is it versioned or ignored?
                     if self.is_versioned(relpath):
                         # Add nested content for deletion.
@@ -569,7 +659,7 @@
         """
         with self.lock_read():
             index_paths = set(
-                [decode_git_path(p) for p, i in self._recurse_index_entries()])
+                [decode_git_path(p) for p, sha, mode in self.iter_git_objects()])
             all_paths = set(self._iter_files_recursive(include_dirs=False))
             return iter(all_paths - index_paths)
 
@@ -867,8 +957,7 @@
             conflicts = _mod_conflicts.ConflictList()
             for item_path, value in self.index.iteritems():
                 if value.flags & FLAG_STAGEMASK:
-                    conflicts.append(_mod_conflicts.TextConflict(
-                        decode_git_path(item_path)))
+                    conflicts.append(TextConflict(decode_git_path(item_path)))
             return conflicts
 
     def set_conflicts(self, conflicts):
@@ -909,8 +998,8 @@
         """Walk the directories of this tree.
 
         returns a generator which yields items in the form:
-                ((curren_directory_path, fileid),
-                 [(file1_path, file1_name, file1_kind, (lstat), file1_id,
+                (current_directory_path,
+                 [(file1_path, file1_name, file1_kind, (lstat),
                    file1_kind), ... ])
 
         This API returns a generator, which is only valid during the current
@@ -974,20 +1063,20 @@
                              - (current_inv[0][0] < cur_disk_dir_relpath))
             if direction > 0:
                 # disk is before inventory - unknown
-                dirblock = [(relpath, basename, kind, stat, None, None) for
+                dirblock = [(relpath, basename, kind, stat, None) for
                             relpath, basename, kind, stat, top_path in
                             cur_disk_dir_content]
-                yield (cur_disk_dir_relpath, None), dirblock
+                yield cur_disk_dir_relpath, dirblock
                 try:
                     current_disk = next(disk_iterator)
                 except StopIteration:
                     disk_finished = True
             elif direction < 0:
                 # inventory is before disk - missing.
-                dirblock = [(relpath, basename, 'unknown', None, fileid, kind)
+                dirblock = [(relpath, basename, 'unknown', None, kind)
                             for relpath, basename, dkind, stat, fileid, kind in
                             current_inv[1]]
-                yield (current_inv[0][0], current_inv[0][1]), dirblock
+                yield current_inv[0][0], dirblock
                 try:
                     current_inv = next(inventory_iterator)
                 except StopIteration:
@@ -1005,23 +1094,22 @@
                         # versioned, present file
                         dirblock.append((inv_row[0],
                                          inv_row[1], disk_row[2],
-                                         disk_row[3], inv_row[4],
-                                         inv_row[5]))
+                                         disk_row[3], inv_row[5]))
                     elif len(path_elements[0]) == 5:
                         # unknown disk file
                         dirblock.append(
                             (path_elements[0][0], path_elements[0][1],
                                 path_elements[0][2], path_elements[0][3],
-                                None, None))
+                                None))
                     elif len(path_elements[0]) == 6:
                         # versioned, absent file.
                         dirblock.append(
                             (path_elements[0][0], path_elements[0][1],
-                                'unknown', None, path_elements[0][4],
+                                'unknown', None,
                                 path_elements[0][5]))
                     else:
                         raise NotImplementedError('unreachable code')
-                yield current_inv[0], dirblock
+                yield current_inv[0][0], dirblock
                 try:
                     current_inv = next(inventory_iterator)
                 except StopIteration:

=== modified file 'breezy/merge.py'
--- breezy/merge.py	2020-07-18 23:14:00 +0000
+++ breezy/merge.py	2020-08-10 15:00:17 +0000
@@ -36,11 +36,13 @@
     workingtree,
     )
 from breezy.bzr import (
+    conflicts as _mod_bzr_conflicts,
     generate_ids,
     versionedfile,
     )
 from breezy.i18n import gettext
 """)
+from breezy.bzr.conflicts import Conflict as BzrConflict
 from . import (
     decorators,
     errors,
@@ -1302,8 +1304,10 @@
                 # This is a contents conflict, because none of the available
                 # functions could merge it.
                 file_group = self._dump_conflicts(
-                    name, (base_path, other_path, this_path), parent_id,
-                    file_id, set_version=True)
+                    name, (base_path, other_path, this_path), parent_id)
+                for tid in file_group:
+                    self.tt.version_file(tid, file_id=file_id)
+                    break
                 self._raw_conflicts.append(('contents conflict', file_group))
         elif hook_status == 'success':
             self.tt.create_file(lines, trans_id)
@@ -1315,7 +1319,7 @@
             name = self.tt.final_name(trans_id)
             parent_id = self.tt.final_parent(trans_id)
             self._dump_conflicts(
-                name, (base_path, other_path, this_path), parent_id, file_id)
+                name, (base_path, other_path, this_path), parent_id)
         elif hook_status == 'delete':
             self.tt.unversion_file(trans_id)
             result = "deleted"
@@ -1421,10 +1425,9 @@
             self._raw_conflicts.append(('text conflict', trans_id))
             name = self.tt.final_name(trans_id)
             parent_id = self.tt.final_parent(trans_id)
-            file_id = self.tt.final_file_id(trans_id)
-            file_group = self._dump_conflicts(name, paths, parent_id, file_id,
-                                              this_lines, base_lines,
-                                              other_lines)
+            file_group = self._dump_conflicts(
+                name, paths, parent_id,
+                lines=(base_lines, other_lines, this_lines))
             file_group.append(trans_id)
 
     def _get_filter_tree_path(self, path):
@@ -1441,8 +1444,7 @@
         # Skip the lookup for older formats
         return None
 
-    def _dump_conflicts(self, name, paths, parent_id, file_id, this_lines=None,
-                        base_lines=None, other_lines=None, set_version=False,
+    def _dump_conflicts(self, name, paths, parent_id, lines=None,
                         no_base=False):
         """Emit conflict files.
         If this_lines, base_lines, or other_lines are omitted, they will be
@@ -1450,6 +1452,10 @@
         or .BASE (in that order) will be created as versioned files.
         """
         base_path, other_path, this_path = paths
+        if lines:
+            base_lines, other_lines, this_lines = lines
+        else:
+            base_lines = other_lines = this_lines = None
         data = [('OTHER', self.other_tree, other_path, other_lines),
                 ('THIS', self.this_tree, this_path, this_lines)]
         if not no_base:
@@ -1462,7 +1468,6 @@
             # Skip the id2path lookup for older formats
             filter_tree_path = None
 
-        versioned = False
         file_group = []
         for suffix, tree, path, lines in data:
             if path is not None:
@@ -1470,9 +1475,6 @@
                     name, parent_id, path, tree, suffix, lines,
                     filter_tree_path)
                 file_group.append(trans_id)
-                if set_version and not versioned:
-                    self.tt.version_file(trans_id, file_id=file_id)
-                    versioned = True
         return file_group
 
     def _conflict_file(self, name, parent_id, path, tree, suffix,
@@ -1519,73 +1521,8 @@
 
     def cook_conflicts(self, fs_conflicts):
         """Convert all conflicts into a form that doesn't depend on trans_id"""
-        content_conflict_file_ids = set()
-        cooked_conflicts = transform.cook_conflicts(fs_conflicts, self.tt)
-        fp = transform.FinalPaths(self.tt)
-        for conflict in self._raw_conflicts:
-            conflict_type = conflict[0]
-            if conflict_type == 'path conflict':
-                (trans_id, file_id,
-                 this_parent, this_name,
-                 other_parent, other_name) = conflict[1:]
-                if this_parent is None or this_name is None:
-                    this_path = '<deleted>'
-                else:
-                    parent_path = fp.get_path(
-                        self.tt.trans_id_file_id(this_parent))
-                    this_path = osutils.pathjoin(parent_path, this_name)
-                if other_parent is None or other_name is None:
-                    other_path = '<deleted>'
-                else:
-                    if other_parent == self.other_tree.path2id(''):
-                        # The tree transform doesn't know about the other root,
-                        # so we special case here to avoid a NoFinalPath
-                        # exception
-                        parent_path = ''
-                    else:
-                        parent_path = fp.get_path(
-                            self.tt.trans_id_file_id(other_parent))
-                    other_path = osutils.pathjoin(parent_path, other_name)
-                c = _mod_conflicts.Conflict.factory(
-                    'path conflict', path=this_path,
-                    conflict_path=other_path,
-                    file_id=file_id)
-            elif conflict_type == 'contents conflict':
-                for trans_id in conflict[1]:
-                    file_id = self.tt.final_file_id(trans_id)
-                    if file_id is not None:
-                        # Ok we found the relevant file-id
-                        break
-                path = fp.get_path(trans_id)
-                for suffix in ('.BASE', '.THIS', '.OTHER'):
-                    if path.endswith(suffix):
-                        # Here is the raw path
-                        path = path[:-len(suffix)]
-                        break
-                c = _mod_conflicts.Conflict.factory(conflict_type,
-                                                    path=path, file_id=file_id)
-                content_conflict_file_ids.add(file_id)
-            elif conflict_type == 'text conflict':
-                trans_id = conflict[1]
-                path = fp.get_path(trans_id)
-                file_id = self.tt.final_file_id(trans_id)
-                c = _mod_conflicts.Conflict.factory(conflict_type,
-                                                    path=path, file_id=file_id)
-            else:
-                raise AssertionError('bad conflict type: %r' % (conflict,))
-            cooked_conflicts.append(c)
-
-        self.cooked_conflicts = []
-        # We want to get rid of path conflicts when a corresponding contents
-        # conflict exists. This can occur when one branch deletes a file while
-        # the other renames *and* modifies it. In this case, the content
-        # conflict is enough.
-        for c in cooked_conflicts:
-            if (c.typestring == 'path conflict'
-                    and c.file_id in content_conflict_file_ids):
-                continue
-            self.cooked_conflicts.append(c)
-        self.cooked_conflicts.sort(key=_mod_conflicts.Conflict.sort_key)
+        self.cooked_conflicts = list(self.tt.cook_conflicts(
+            list(fs_conflicts) + self._raw_conflicts))
 
 
 class WeaveMerger(Merge3Merger):
@@ -1642,10 +1579,9 @@
             self._raw_conflicts.append(('text conflict', trans_id))
             name = self.tt.final_name(trans_id)
             parent_id = self.tt.final_parent(trans_id)
-            file_id = self.tt.final_file_id(trans_id)
-            file_group = self._dump_conflicts(name, paths, parent_id, file_id,
-                                              no_base=False,
-                                              base_lines=base_lines)
+            file_group = self._dump_conflicts(name, paths, parent_id,
+                                              (base_lines, None, None),
+                                              no_base=False)
             file_group.append(trans_id)
 
 
@@ -1695,8 +1631,7 @@
             if status == 1:
                 name = self.tt.final_name(trans_id)
                 parent_id = self.tt.final_parent(trans_id)
-                file_id = self.tt.final_file_id(trans_id)
-                self._dump_conflicts(name, paths, parent_id, file_id)
+                self._dump_conflicts(name, paths, parent_id)
                 self._raw_conflicts.append(('text conflict', trans_id))
         finally:
             osutils.rmtree(temp_dir)

=== modified file 'breezy/mutabletree.py'
--- breezy/mutabletree.py	2020-07-18 23:14:00 +0000
+++ breezy/mutabletree.py	2020-08-10 15:00:17 +0000
@@ -185,42 +185,7 @@
 
         :return: True if a change is found. False otherwise
         """
-        with self.lock_read():
-            # Check pending merges
-            if len(self.get_parent_ids()) > 1:
-                return True
-            if _from_tree is None:
-                _from_tree = self.basis_tree()
-            changes = self.iter_changes(_from_tree)
-            if self.supports_symlinks():
-                # Fast path for has_changes.
-                try:
-                    change = next(changes)
-                    # Exclude root (talk about black magic... --vila 20090629)
-                    if change.parent_id == (None, None):
-                        change = next(changes)
-                    return True
-                except StopIteration:
-                    # No changes
-                    return False
-            else:
-                # Slow path for has_changes.
-                # Handle platforms that do not support symlinks in the
-                # conditional below. This is slower than the try/except
-                # approach below that but we don't have a choice as we
-                # need to be sure that all symlinks are removed from the
-                # entire changeset. This is because in platforms that
-                # do not support symlinks, they show up as None in the
-                # working copy as compared to the repository.
-                # Also, exclude root as mention in the above fast path.
-                changes = filter(
-                    lambda c: c[6][0] != 'symlink' and c[4] != (None, None),
-                    changes)
-                try:
-                    next(iter(changes))
-                except StopIteration:
-                    return False
-                return True
+        raise NotImplementedError(self.has_changes)
 
     def check_changed_or_out_of_date(self, strict, opt_name,
                                      more_error, more_warning):
@@ -243,7 +208,7 @@
                 strict = self.branch.get_config_stack().get(opt_name)
             if strict is not False:
                 err_class = None
-                if (self.has_changes()):
+                if self.has_changes():
                     err_class = errors.UncommittedChanges
                 elif self.last_revision() != self.branch.last_revision():
                     # The tree has lost sync with its branch, there is little

=== modified file 'breezy/plugins/darcs/__init__.py'
--- breezy/plugins/darcs/__init__.py	2020-07-18 23:14:00 +0000
+++ breezy/plugins/darcs/__init__.py	2020-08-10 15:00:17 +0000
@@ -29,8 +29,7 @@
 class DarcsUnsupportedError(errors.UnsupportedFormatError):
 
     _fmt = ('Darcs branches are not yet supported. '
-            'To convert darcs branches to Bazaar branches or vice versa, '
-            'use bzr-fastimport. See http://bazaar-vcs.org/BzrMigration.')
+            'To interoperate with darcs branches, use fastimport.')
 
 
 class DarcsDirFormat(controldir.ControlDirFormat):

=== modified file 'breezy/plugins/fastimport/revision_store.py'
--- breezy/plugins/fastimport/revision_store.py	2020-02-18 01:57:45 +0000
+++ breezy/plugins/fastimport/revision_store.py	2020-08-10 15:00:17 +0000
@@ -23,7 +23,7 @@
     osutils,
     revision as _mod_revision,
     )
-from ...tree import TreeChange
+from ...bzr.inventorytree import InventoryTreeChange
 from ...bzr import (
     inventory,
     )
@@ -117,7 +117,7 @@
                 old_ie = None
                 if ie is None:
                     raise AssertionError('How is both old and new None?')
-                    change = TreeChange(
+                    change = InventoryTreeChange(
                         file_id,
                         (old_path, new_path),
                         False,
@@ -127,7 +127,7 @@
                         (None, None),
                         (None, None),
                         )
-                change = TreeChange(
+                change = InventoryTreeChange(
                     file_id,
                     (old_path, new_path),
                     True,
@@ -139,7 +139,7 @@
                     )
             else:
                 if ie is None:
-                    change = TreeChange(
+                    change = InventoryTreeChange(
                         file_id,
                         (old_path, new_path),
                         True,
@@ -154,7 +154,7 @@
                                         ie.text_size != old_ie.text_size)
                     # TODO: ie.kind != old_ie.kind
                     # TODO: symlinks changing targets, content_modified?
-                    change = TreeChange(
+                    change = InventoryTreeChange(
                         file_id,
                         (old_path, new_path),
                         content_modified,

=== modified file 'breezy/plugins/fossil/__init__.py'
--- breezy/plugins/fossil/__init__.py	2020-02-18 01:57:45 +0000
+++ breezy/plugins/fossil/__init__.py	2020-08-10 15:00:17 +0000
@@ -30,8 +30,7 @@
 class FossilUnsupportedError(errors.UnsupportedFormatError):
 
     _fmt = ('Fossil branches are not yet supported. '
-            'To convert Fossil branches to Bazaar branches or vice versa, '
-            'use fastimport.')
+            'To interoperate with Fossil branches, use fastimport.')
 
 
 class FossilDirFormat(controldir.ControlDirFormat):

=== modified file 'breezy/plugins/github/hoster.py'
--- breezy/plugins/github/hoster.py	2020-07-28 02:11:05 +0000
+++ breezy/plugins/github/hoster.py	2020-08-10 15:00:17 +0000
@@ -275,9 +275,15 @@
             'Accept': 'application/vnd.github.v3+json'}
         if self._token:
             headers['Authorization'] = 'token %s' % self._token
-        response = self.transport.request(
-            method, urlutils.join(self.transport.base, path),
-            headers=headers, body=body, retries=3)
+        try:
+            response = self.transport.request(
+                method, urlutils.join(self.transport.base, path),
+                headers=headers, body=body, retries=3)
+        except UnexpectedHttpStatus as e:
+            if e.code == 401:
+                raise GitHubLoginRequired(self)
+            else:
+                raise
         if response.status == 401:
             raise GitHubLoginRequired(self)
         return response
@@ -561,7 +567,9 @@
         raise UnexpectedHttpStatus(path, response.status)
 
     def get_current_user(self):
-        return self.current_user['login']
+        if self._token is not None:
+            return self.current_user['login']
+        return None
 
     def get_user_url(self, username):
         return urlutils.join(self.base_url, username)

=== modified file 'breezy/plugins/gitlab/hoster.py'
--- breezy/plugins/gitlab/hoster.py	2020-07-28 02:11:05 +0000
+++ breezy/plugins/gitlab/hoster.py	2020-08-10 15:00:17 +0000
@@ -39,6 +39,7 @@
     MergeProposalExists,
     NoSuchProject,
     PrerequisiteBranchUnsupported,
+    SourceNotDerivedFromTarget,
     UnsupportedHoster,
     )
 
@@ -74,6 +75,14 @@
         self.url = url
 
 
+class GitLabUnprocessable(errors.BzrError):
+
+    _fmt = "GitLab can not process request: %(error)s."
+
+    def __init__(self, error):
+        errors.BzrError.__init__(self, error=error)
+
+
 class DifferentGitLabInstances(errors.BzrError):
 
     _fmt = ("Can't create merge proposals across GitLab instances: "
@@ -193,6 +202,11 @@
     return host, project_name, int(parts[-1])
 
 
+def _unexpected_status(path, response):
+    raise errors.UnexpectedHttpStatus(
+        path, response.status, response.data.decode('utf-8', 'replace'))
+
+
 class GitLabMergeProposal(MergeProposal):
 
     def __init__(self, gl, mr):
@@ -341,7 +355,7 @@
             raise KeyError('no such user %s' % username)
         if response.status == 200:
             return json.loads(response.data)
-        raise errors.UnexpectedHttpStatus(path, response.status)
+        _unexpected_status(path, response)
 
     def _get_user_by_email(self, email):
         path = 'users?search=%s' % urlutils.quote(str(email), '')
@@ -353,7 +367,7 @@
             if len(ret) != 1:
                 raise ValueError('unexpected number of results; %r' % ret)
             return ret[0]
-        raise errors.UnexpectedHttpStatus(path, response.status)
+        _unexpected_status(path, response)
 
     def _get_project(self, project_name):
         path = 'projects/%s' % urlutils.quote(str(project_name), '')
@@ -362,7 +376,7 @@
             raise NoSuchProject(project_name)
         if response.status == 200:
             return json.loads(response.data)
-        raise errors.UnexpectedHttpStatus(path, response.status)
+        _unexpected_status(path, response)
 
     def _fork_project(self, project_name, timeout=50, interval=5, owner=None):
         path = 'projects/%s/fork' % urlutils.quote(str(project_name), '')
@@ -376,7 +390,7 @@
             resp = json.loads(response.data)
             raise GitLabConflict(resp.get('message'))
         if response.status not in (200, 201):
-            raise errors.UnexpectedHttpStatus(path, response.status)
+            _unexpected_status(path, response)
         # The response should be valid JSON, but let's ignore it
         project = json.loads(response.data)
         # Spin and wait until import_status for new project
@@ -412,7 +426,7 @@
             if response.status == 403:
                 raise errors.PermissionDenied(response.text)
             if response.status != 200:
-                raise errors.UnexpectedHttpStatus(path, response.status)
+                _unexpected_status(path, response)
             page = response.getheader("X-Next-Page")
             for entry in json.loads(response.data):
                 yield entry
@@ -435,7 +449,7 @@
         if response.status == 403:
             raise errors.PermissionDenied(response.text)
         if response.status != 200:
-            raise errors.UnexpectedHttpStatus(path, response.status)
+            _unexpected_status(path, response)
         return json.loads(response.data)
 
     def _list_projects(self, owner):
@@ -449,7 +463,9 @@
         response = self._api_request('PUT', path, fields=mr)
         if response.status == 200:
             return json.loads(response.data)
-        raise errors.UnexpectedHttpStatus(path, response.status)
+        if response.status == 403:
+            raise errors.PermissionDenied(response.text)
+        _unexpected_status(path, response)
 
     def _post_merge_request_note(self, project_id, iid, kwargs):
         path = 'projects/%s/merge_requests/%s/notes' % (
@@ -458,7 +474,9 @@
         if response.status == 201:
             json.loads(response.data)
             return
-        raise errors.UnexpectedHttpStatus(path, response.status)
+        if response.status == 403:
+            raise errors.PermissionDenied(response.text)
+        _unexpected_status(path, response)
 
     def _create_mergerequest(
             self, title, source_project_id, target_project_id,
@@ -480,8 +498,11 @@
             raise errors.PermissionDenied(response.text)
         if response.status == 409:
             raise MergeRequestExists()
+        if response.status == 422:
+            data = json.loads(response.data)
+            raise GitLabUnprocessable(data['error'])
         if response.status != 201:
-            raise errors.UnexpectedHttpStatus(path, response.status)
+            _unexpected_status(path, response)
         return json.loads(response.data)
 
     def get_push_url(self, branch):
@@ -629,7 +650,7 @@
         if response.status == 404:
             raise NoSuchProject(project)
         if response.status != 202:
-            raise errors.UnexpectedHttpStatus(path, response.status)
+            _unexpected_status(path, response)
 
 
 class GitlabMergeProposalBuilder(MergeProposalBuilder):
@@ -698,6 +719,11 @@
             merge_request = self.gl._create_mergerequest(**kwargs)
         except MergeRequestExists:
             raise MergeProposalExists(self.source_branch.user_url)
+        except GitLabUnprocessable as e:
+            if e.error == [
+                    "Source project is not a fork of the target project"]:
+                raise SourceNotDerivedFromTarget(
+                    self.source_branch, self.target_branch)
         return GitLabMergeProposal(self.gl, merge_request)
 
 

=== modified file 'breezy/plugins/hg/__init__.py'
--- breezy/plugins/hg/__init__.py	2020-07-18 23:14:00 +0000
+++ breezy/plugins/hg/__init__.py	2020-08-10 15:00:17 +0000
@@ -30,8 +30,7 @@
 class MercurialUnsupportedError(errors.UnsupportedFormatError):
 
     _fmt = ('Mercurial branches are not yet supported. '
-            'To convert Mercurial branches to Bazaar branches or vice versa, '
-            'use the fastimport format. ')
+            'To interoperate with Mercurial, use the fastimport format.')
 
 
 class LocalHgDirFormat(controldir.ControlDirFormat):
@@ -145,6 +144,9 @@
             'GET', url, headers={'Accept': 'application/mercurial-0.1'})
         if resp.status == 404:
             return False
+        if resp.status == 406:
+            # The server told us that it can't handle our Accept header.
+            return False
         ct = resp.getheader("Content-Type")
         if ct is None:
             return False

=== modified file 'breezy/plugins/launchpad/hoster.py'
--- breezy/plugins/launchpad/hoster.py	2020-07-28 02:11:05 +0000
+++ breezy/plugins/launchpad/hoster.py	2020-08-10 15:00:17 +0000
@@ -721,7 +721,8 @@
                 revid=self.source_branch.last_revision())
 
     def create_proposal(self, description, reviewers=None, labels=None,
-                        prerequisite_branch=None, commit_message=None):
+                        prerequisite_branch=None, commit_message=None,
+                        work_in_progress=False, allow_collaboration=False):
         """Perform the submission."""
         if labels:
             raise LabelsUnsupported(self)
@@ -739,7 +740,7 @@
                 merge_prerequisite=prereq_branch_lp,
                 initial_comment=description.strip(),
                 commit_message=commit_message,
-                needs_review=True,
+                needs_review=(not work_in_progress),
                 reviewers=[self.launchpad.people[reviewer].self_link
                            for reviewer in reviewers],
                 review_types=[None for reviewer in reviewers])

=== modified file 'breezy/plugins/mtn/__init__.py'
--- breezy/plugins/mtn/__init__.py	2020-02-18 01:57:45 +0000
+++ breezy/plugins/mtn/__init__.py	2020-08-10 15:00:17 +0000
@@ -30,8 +30,8 @@
 class MonotoneUnsupportedError(errors.UnsupportedFormatError):
 
     _fmt = ('Monotone branches are not yet supported. '
-            'To convert monotone branches to Bazaar branches or vice versa, '
-            'use bzr-fastimport. See http://bazaar-vcs.org/BzrMigration.')
+            'To interoperate with Monotone branches, '
+            'use fastimport.')
 
 
 class MonotoneDirFormat(controldir.ControlDirFormat):

=== modified file 'breezy/plugins/propose/cmds.py'
--- breezy/plugins/propose/cmds.py	2020-07-28 02:11:05 +0000
+++ breezy/plugins/propose/cmds.py	2020-08-10 15:00:17 +0000
@@ -34,7 +34,7 @@
     Option,
     RegistryOption,
     )
-from ...trace import note
+from ...trace import note, warning
 from ... import (
     propose as _mod_propose,
     )
@@ -268,19 +268,22 @@
 
     def run(self, status='open', verbose=False):
         for instance in _mod_propose.iter_hoster_instances():
-            for mp in instance.iter_my_proposals(status=status):
-                self.outf.write('%s\n' % mp.url)
-                if verbose:
-                    self.outf.write(
-                        '(Merging %s into %s)\n' %
-                        (mp.get_source_branch_url(),
-                         mp.get_target_branch_url()))
-                    description = mp.get_description()
-                    if description:
-                        self.outf.writelines(
-                            ['\t%s\n' % l
-                             for l in description.splitlines()])
-                    self.outf.write('\n')
+            try:
+                for mp in instance.iter_my_proposals(status=status):
+                    self.outf.write('%s\n' % mp.url)
+                    if verbose:
+                        self.outf.write(
+                            '(Merging %s into %s)\n' %
+                            (mp.get_source_branch_url(),
+                             mp.get_target_branch_url()))
+                        description = mp.get_description()
+                        if description:
+                            self.outf.writelines(
+                                ['\t%s\n' % l
+                                 for l in description.splitlines()])
+                        self.outf.write('\n')
+            except _mod_propose.HosterLoginRequired as e:
+                warning('Skipping %r, login required.', instance)
 
 
 class cmd_land_merge_proposal(Command):
@@ -303,7 +306,12 @@
     def run(self):
         for instance in _mod_propose.iter_hoster_instances():
             current_user = instance.get_current_user()
-            self.outf.write(
-                gettext('%s (%s) - user: %s (%s)\n') % (
-                    instance.name, instance.base_url,
-                    current_user, instance.get_user_url(current_user)))
+            if current_user is not None:
+                self.outf.write(
+                    gettext('%s (%s) - user: %s (%s)\n') % (
+                        instance.name, instance.base_url,
+                        current_user, instance.get_user_url(current_user)))
+            else:
+                self.outf.write(
+                    gettext('%s (%s) - not logged in\n') % (
+                        instance.name, instance.base_url))

=== modified file 'breezy/plugins/rewrite/rebase.py'
--- breezy/plugins/rewrite/rebase.py	2020-02-18 01:57:45 +0000
+++ breezy/plugins/rewrite/rebase.py	2020-08-10 15:00:17 +0000
@@ -30,12 +30,12 @@
     UnrelatedBranches,
     )
 from ...bzr.generate_ids import gen_revision_id
+from ...bzr.inventorytree import InventoryTreeChange
 from ...graph import FrozenHeadsCache
 from ...merge import Merger
 from ...revision import NULL_REVISION
 from ...trace import mutter
 from ...tsort import topo_sort
-from ...tree import TreeChange
 from ... import ui
 
 from .maptree import (
@@ -389,7 +389,7 @@
             new_parent = map_tree.new_id(change.parent_id[1])
         else:
             new_parent = change.parent_id[1]
-        yield TreeChange(
+        yield InventoryTreeChange(
             map_tree.new_id(change.file_id), change.path,
             change.changed_content, change.versioned,
             (old_parent, new_parent), change.name, change.kind,

=== modified file 'breezy/plugins/rewrite/tests/test_rebase.py'
--- breezy/plugins/rewrite/tests/test_rebase.py	2020-01-23 00:05:05 +0000
+++ breezy/plugins/rewrite/tests/test_rebase.py	2020-08-09 18:10:01 +0000
@@ -653,7 +653,7 @@
         newwt.unlock()
         with open("new/afile", 'r') as f:
             self.assertEquals("E\n" + "A\n" * 10 + "C\n", f.read())
-        newwt.set_conflicts(ConflictList())
+        newwt.set_conflicts([])
         oldrev = newwt.branch.repository.get_revision(b"E")
         replayer.commit_rebase(oldrev, b"E'")
         newrev = newwt.branch.repository.get_revision(b"E'")

=== modified file 'breezy/plugins/svn/__init__.py'
--- breezy/plugins/svn/__init__.py	2020-07-28 02:11:05 +0000
+++ breezy/plugins/svn/__init__.py	2020-08-10 15:00:17 +0000
@@ -34,8 +34,7 @@
 class SubversionUnsupportedError(errors.UnsupportedFormatError):
 
     _fmt = ('Subversion branches are not yet supported. '
-            'To convert Subversion branches to Bazaar branches or vice versa, '
-            'use the fastimport format.')
+            'To interoperate with Subversion branches, use fastimport.')
 
 
 class SvnWorkingTreeDirFormat(controldir.ControlDirFormat):

=== modified file 'breezy/plugins/weave_fmt/test_workingtree.py'
--- breezy/plugins/weave_fmt/test_workingtree.py	2020-02-18 01:57:45 +0000
+++ breezy/plugins/weave_fmt/test_workingtree.py	2020-08-10 15:00:17 +0000
@@ -21,6 +21,9 @@
 
 from ... import (
     conflicts,
+    )
+from ...bzr.conflicts import TextConflict, ContentsConflict
+from ... import (
     errors,
     )
 
@@ -45,23 +48,23 @@
                           None)
         with open('lala.BASE', 'wb') as f:
             f.write(b'labase')
-        expected = conflicts.ContentsConflict('lala')
+        expected = ContentsConflict('lala')
         self.assertEqual(list(tree.conflicts()), [expected])
         with open('lala', 'wb') as f:
             f.write(b'la')
         tree.add('lala', b'lala-id')
-        expected = conflicts.ContentsConflict('lala', file_id='lala-id')
+        expected = ContentsConflict('lala', file_id='lala-id')
         self.assertEqual(list(tree.conflicts()), [expected])
         with open('lala.THIS', 'wb') as f:
             f.write(b'lathis')
         with open('lala.OTHER', 'wb') as f:
             f.write(b'laother')
         # When "text conflict"s happen, stem, THIS and OTHER are text
-        expected = conflicts.TextConflict('lala', file_id='lala-id')
+        expected = TextConflict('lala', file_id='lala-id')
         self.assertEqual(list(tree.conflicts()), [expected])
         os.unlink('lala.OTHER')
         os.mkdir('lala.OTHER')
-        expected = conflicts.ContentsConflict('lala', file_id='lala-id')
+        expected = ContentsConflict('lala', file_id='lala-id')
         self.assertEqual(list(tree.conflicts()), [expected])
 
     def test_detect_conflicts(self):

=== modified file 'breezy/plugins/weave_fmt/workingtree.py'
--- breezy/plugins/weave_fmt/workingtree.py	2020-02-18 01:57:45 +0000
+++ breezy/plugins/weave_fmt/workingtree.py	2020-08-10 15:00:17 +0000
@@ -27,6 +27,7 @@
     transform,
     )
 from ...bzr import (
+    conflicts as _mod_bzr_conflicts,
     inventory,
     xml5,
     )
@@ -41,7 +42,7 @@
 
 
 def get_conflicted_stem(path):
-    for suffix in _mod_conflicts.CONFLICT_SUFFIXES:
+    for suffix in _mod_bzr_conflicts.CONFLICT_SUFFIXES:
         if path.endswith(suffix):
             return path[:-len(suffix)]
 
@@ -230,7 +231,7 @@
                             break
                 ctype = {True: 'text conflict',
                          False: 'contents conflict'}[text]
-                conflicts.append(_mod_conflicts.Conflict.factory(ctype,
+                conflicts.append(_mod_bzr_conflicts.Conflict.factory(ctype,
                                                                  path=conflicted,
                                                                  file_id=self.path2id(conflicted)))
             return conflicts

=== modified file 'breezy/propose.py'
--- breezy/propose.py	2020-07-28 02:11:05 +0000
+++ breezy/propose.py	2020-08-10 15:00:17 +0000
@@ -97,6 +97,18 @@
         self.hoster = hoster
 
 
+class SourceNotDerivedFromTarget(errors.BzrError):
+    """Source branch is not derived from target branch."""
+
+    _fmt = ("Source %(source_branch)r not derived from "
+            "target %(target_branch)r.")
+
+    def __init__(self, source_branch, target_branch):
+        errors.BzrError.__init__(
+            self, source_branch=source_branch,
+            target_branch=target_branch)
+
+
 class MergeProposal(object):
     """A merge proposal.
 

=== modified file 'breezy/tests/blackbox/test_merge.py'
--- breezy/tests/blackbox/test_merge.py	2018-11-29 23:42:41 +0000
+++ breezy/tests/blackbox/test_merge.py	2020-08-09 18:10:01 +0000
@@ -26,7 +26,6 @@
 
 from breezy import (
     branch,
-    conflicts,
     controldir,
     merge_directive,
     osutils,
@@ -34,6 +33,9 @@
     urlutils,
     workingtree,
     )
+from breezy.bzr import (
+    conflicts,
+    )
 from breezy.tests import (
     scenarios,
     script,
@@ -328,7 +330,7 @@
                                               tree_b.get_parent_ids()[0])
         self.assertEqualDiff(testament_a.as_text(),
                              testament_b.as_text())
-        tree_a.set_conflicts(conflicts.ConflictList())
+        tree_a.set_conflicts([])
         tree_a.commit('message')
         # it is legal to attempt to merge an already-merged bundle
         err = self.run_bzr('merge ../bundle', working_dir='branch_a')[1]
@@ -427,8 +429,7 @@
         tree_b.commit('content change')
         self.run_bzr('merge ../tree_a', retcode=1, working_dir='tree_b')
         self.assertEqual(tree_b.conflicts(),
-                         [conflicts.ContentsConflict('file',
-                                                     file_id='file-id')])
+                         [conflicts.ContentsConflict('file', file_id='file-id')])
 
     def test_directive_cherrypick(self):
         source = self.make_branch_and_tree('source')
@@ -632,11 +633,11 @@
         tree_a.merge_from_branch(tree_b.branch)
         self.build_tree_contents([('a/file',
                                    b'base-contents\nthis-contents\n')])
-        tree_a.set_conflicts(conflicts.ConflictList())
+        tree_a.set_conflicts([])
         tree_b.merge_from_branch(tree_a.branch)
         self.build_tree_contents([('b/file',
                                    b'base-contents\nother-contents\n')])
-        tree_b.set_conflicts(conflicts.ConflictList())
+        tree_b.set_conflicts([])
         tree_a.commit('', rev_id=b'rev3a')
         tree_b.commit('', rev_id=b'rev3b')
         out, err = self.run_bzr(['merge', '-d', 'a', 'b', '--lca'], retcode=1)

=== modified file 'breezy/tests/blackbox/test_resolve.py'
--- breezy/tests/blackbox/test_resolve.py	2018-11-16 18:33:17 +0000
+++ breezy/tests/blackbox/test_resolve.py	2020-08-09 18:10:01 +0000
@@ -18,6 +18,7 @@
     conflicts,
     tests,
     )
+from breezy.bzr import conflicts as _mod_bzr_conflicts
 from breezy.tests import (
     script,
     KnownFailure,
@@ -178,8 +179,8 @@
                                    b'<<<<<<<\na\n=======\n>>>>>>>\n')])
         tree.add('file', b'file_id')
         self.assertEqual(tree.kind('file'), 'file')
-        file_conflict = conflicts.TextConflict('file', file_id=b'file_id')
-        tree.set_conflicts(conflicts.ConflictList([file_conflict]))
+        file_conflict = _mod_bzr_conflicts.TextConflict('file', file_id=b'file_id')
+        tree.set_conflicts([file_conflict])
         note = self.run_bzr('resolve', retcode=1, working_dir='tree')[1]
         self.assertContainsRe(note, '0 conflicts auto-resolved.')
         self.assertContainsRe(note,

=== modified file 'breezy/tests/blackbox/test_status.py'
--- breezy/tests/blackbox/test_status.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/blackbox/test_status.py	2020-08-10 15:00:17 +0000
@@ -31,13 +31,13 @@
 import sys
 
 from ... import (
-    conflicts,
     errors,
     osutils,
     status,
     )
 from breezy.bzr import (
     bzrdir,
+    conflicts,
     )
 import breezy.branch
 from ...osutils import pathjoin
@@ -297,20 +297,17 @@
         self.build_tree(['dir2/'])
         tree.add('dir2')
         tree.commit('added dir2')
-        tree.set_conflicts(conflicts.ConflictList(
-            [conflicts.ContentsConflict('foo')]))
+        tree.set_conflicts([conflicts.ContentsConflict('foo')])
         tof = BytesIO()
         show_tree_status(tree, specific_files=['dir2'], to_file=tof)
         self.assertEqualDiff(b'', tof.getvalue())
-        tree.set_conflicts(conflicts.ConflictList(
-            [conflicts.ContentsConflict('dir2')]))
+        tree.set_conflicts([conflicts.ContentsConflict('dir2')])
         tof = StringIO()
         show_tree_status(tree, specific_files=['dir2'], to_file=tof)
         self.assertEqualDiff('conflicts:\n  Contents conflict in dir2\n',
                              tof.getvalue())
 
-        tree.set_conflicts(conflicts.ConflictList(
-            [conflicts.ContentsConflict('dir2/file1')]))
+        tree.set_conflicts([conflicts.ContentsConflict('dir2/file1')])
         tof = StringIO()
         show_tree_status(tree, specific_files=['dir2'], to_file=tof)
         self.assertEqualDiff('conflicts:\n  Contents conflict in dir2/file1\n',

=== modified file 'breezy/tests/features.py'
--- breezy/tests/features.py	2020-02-18 01:57:45 +0000
+++ breezy/tests/features.py	2020-08-10 15:00:17 +0000
@@ -405,6 +405,8 @@
 lsprof_feature = ModuleAvailableFeature('breezy.lsprof')
 pkg_resources_feature = ModuleAvailableFeature('pkg_resources')
 
+pyinotify = ModuleAvailableFeature('pyinotify')
+
 
 class _BackslashDirSeparatorFeature(Feature):
 

=== modified file 'breezy/tests/per_intertree/test_compare.py'
--- breezy/tests/per_intertree/test_compare.py	2020-07-04 12:29:00 +0000
+++ breezy/tests/per_intertree/test_compare.py	2020-08-06 01:40:59 +0000
@@ -25,7 +25,7 @@
     tests,
     )
 from breezy.osutils import has_symlinks
-from breezy.tree import TreeChange
+from breezy.bzr.inventorytree import InventoryTreeChange
 from breezy.tests.per_intertree import TestCaseWithTwoTrees
 from breezy.tests import (
     features,
@@ -572,7 +572,7 @@
 
     def added(self, tree, path):
         entry = self.get_path_entry(tree, path)
-        return TreeChange(
+        return InventoryTreeChange(
             entry.file_id, (None, path), True, (False, True), (None, entry.parent_id),
             (None, entry.name), (None, entry.kind),
             (None, entry.executable))
@@ -587,7 +587,7 @@
 
     def changed_content(self, tree, path):
         entry = self.get_path_entry(tree, path)
-        return TreeChange(
+        return InventoryTreeChange(
             entry.file_id, (path, path), True, (True, True),
             (entry.parent_id, entry.parent_id),
             (entry.name, entry.name), (entry.kind, entry.kind),
@@ -596,7 +596,7 @@
     def kind_changed(self, from_tree, to_tree, from_path, to_path):
         old_entry = self.get_path_entry(from_tree, from_path)
         new_entry = self.get_path_entry(to_tree, to_path)
-        return TreeChange(
+        return InventoryTreeChange(
             new_entry.file_id, (from_path, to_path), True, (True, True),
             (old_entry.parent_id, new_entry.parent_id),
             (old_entry.name, new_entry.name),
@@ -607,14 +607,14 @@
         _, from_basename = os.path.split(from_path)
         _, to_basename = os.path.split(to_path)
         # missing files have both paths, but no kind.
-        return TreeChange(
+        return InventoryTreeChange(
             file_id, (from_path, to_path), True, (True, True),
             (parent_id, parent_id),
             (from_basename, to_basename), (kind, None), (False, False))
 
     def deleted(self, tree, path):
         entry = self.get_path_entry(tree, path)
-        return TreeChange(
+        return InventoryTreeChange(
             entry.file_id, (path, None), True, (True, False), (entry.parent_id, None),
             (entry.name, None), (entry.kind, None),
             (entry.executable, None))
@@ -622,7 +622,7 @@
     def renamed(self, from_tree, to_tree, from_path, to_path, content_changed):
         from_entry = self.get_path_entry(from_tree, from_path)
         to_entry = self.get_path_entry(to_tree, to_path)
-        return TreeChange(
+        return InventoryTreeChange(
             to_entry.file_id, (from_path, to_path), content_changed, (True, True),
             (from_entry.parent_id, to_entry.parent_id),
             (from_entry.name, to_entry.name),
@@ -635,7 +635,7 @@
         name = entry.name
         kind = entry.kind
         executable = entry.executable
-        return TreeChange(
+        return InventoryTreeChange(
             entry.file_id, (path, path), False, (True, True),
             (parent, parent), (name, name), (kind, kind),
             (executable, executable))
@@ -644,7 +644,7 @@
         """Create an unversioned result."""
         _, basename = os.path.split(path)
         kind = tree._comparison_data(None, path)[0]
-        return TreeChange(
+        return InventoryTreeChange(
             None, (None, path), True, (False, False), (None, None),
             (None, basename), (None, kind),
             (None, False))
@@ -1111,8 +1111,9 @@
         self.not_applicable_if_missing_in('file', tree1)
         root_id = tree1.path2id('')
         expected = [
-            TreeChange(b'file-id', ('file', None), False, (True, False),
-                       (root_id, None), ('file', None), (None, None), (False, None))]
+            InventoryTreeChange(
+                b'file-id', ('file', None), False, (True, False),
+                (root_id, None), ('file', None), (None, None), (False, None))]
         self.assertEqual(expected, self.do_iter_changes(tree1, tree2))
 
     def test_only_in_target_and_missing(self):
@@ -1126,8 +1127,9 @@
         self.not_applicable_if_missing_in('file', tree2)
         root_id = tree1.path2id('')
         expected = [
-            TreeChange(b'file-id', (None, 'file'), False, (False, True),
-                       (None, root_id), (None, 'file'), (None, None), (None, False))]
+            InventoryTreeChange(
+                b'file-id', (None, 'file'), False, (False, True),
+                (None, root_id), (None, 'file'), (None, None), (None, False))]
         self.assertEqual(expected, self.do_iter_changes(tree1, tree2))
 
     def test_only_in_target_missing_subtree_specific_bug_367632(self):
@@ -1142,10 +1144,10 @@
         self.not_applicable_if_missing_in('a-dir', tree2)
         root_id = tree1.path2id('')
         expected = [
-            TreeChange(
+            InventoryTreeChange(
                 b'dir-id', (None, 'a-dir'), False, (False, True),
                 (None, root_id), (None, 'a-dir'), (None, None), (None, False)),
-            TreeChange(
+            InventoryTreeChange(
                 b'file-id', (None, 'a-dir/a-file'), False, (False, True),
                 (None, b'dir-id'), (None, 'a-file'), (None, None), (None, False))
             ]
@@ -1164,7 +1166,7 @@
         tree1, tree2 = self.mutable_trees_to_locked_test_trees(tree1, tree2)
         self.assertEqual(sorted([self.unchanged(tree1, ''),
                                  self.unchanged(tree1, 'b'),
-                                 TreeChange(
+                                 InventoryTreeChange(
                                      b'a-id', ('a', 'd'), True, (True, True),
                                      (b'root-id', b'root-id'), ('a', 'd'),
                                      ('file', 'file'),
@@ -1192,7 +1194,7 @@
         self.assertEqual([], list(tree2.iter_changes(tree1)))
         subtree1.commit('commit', rev_id=b'commit-a')
         self.assertEqual([
-            TreeChange(
+            InventoryTreeChange(
                 b'root-id',
                 (u'', u''),
                 False,
@@ -1201,7 +1203,7 @@
                 (u'', u''),
                 ('directory', 'directory'),
                 (False, False)),
-            TreeChange(
+            InventoryTreeChange(
                 b'subtree-id',
                 ('sub', 'sub',),
                 False,

=== modified file 'breezy/tests/per_merger.py'
--- breezy/tests/per_merger.py	2020-07-04 12:29:00 +0000
+++ breezy/tests/per_merger.py	2020-08-09 18:10:01 +0000
@@ -18,7 +18,7 @@
 
 import os
 
-from ..conflicts import TextConflict
+from ..bzr.conflicts import TextConflict
 from .. import (
     errors,
     merge as _mod_merge,

=== modified file 'breezy/tests/per_repository/test_check.py'
--- breezy/tests/per_repository/test_check.py	2019-06-15 15:59:17 +0000
+++ breezy/tests/per_repository/test_check.py	2020-08-06 01:40:59 +0000
@@ -20,7 +20,7 @@
 from breezy import (
     revision as _mod_revision,
     )
-from breezy.tree import TreeChange
+from breezy.bzr.inventorytree import InventoryTreeChange
 from breezy.tests.per_repository import TestCaseWithRepository
 
 
@@ -33,7 +33,7 @@
         self.overrideEnv('BRZ_EMAIL', 'foo@sample.com')
         builder = branch.get_commit_builder([], branch.get_config_stack())
         list(builder.record_iter_changes(None, _mod_revision.NULL_REVISION, [
-            TreeChange(
+            InventoryTreeChange(
                 b'TREE_ROOT', (None, ''), True, (False, True), (None, None),
                 (None, ''), (None, 'directory'), (None, False))]))
         builder.finish_inventory()

=== modified file 'breezy/tests/per_repository/test_commit_builder.py'
--- breezy/tests/per_repository/test_commit_builder.py	2020-07-20 02:02:23 +0000
+++ breezy/tests/per_repository/test_commit_builder.py	2020-08-10 15:00:17 +0000
@@ -26,10 +26,10 @@
     revision as _mod_revision,
     tests,
     )
-from breezy.tree import TreeChange
 from breezy.bzr import (
     inventorytree,
     )
+from breezy.bzr.inventorytree import InventoryTreeChange
 from breezy.tests import per_repository
 from breezy.tests import (
     features,
@@ -196,7 +196,7 @@
         with tree.lock_write():
             builder = tree.branch.get_commit_builder([rev_id])
             try:
-                delete_change = TreeChange(
+                delete_change = InventoryTreeChange(
                     foo_id, ('foo', None), True, (True, False),
                     (tree.path2id(''), None),
                     ('foo', None), ('file', None),
@@ -527,8 +527,9 @@
                 self.assertEqual(delta_entry.name, new_entry.name)
                 self.assertEqual(delta_entry.parent_id, new_entry.parent_id)
                 if delta_entry.kind == 'file':
-                    self.assertEqual(delta_entry.text_size, new_entry.text_size)
-                    self.assertEqual(delta_entry.text_sha1, new_entry.text_sha1)
+                    self.assertEqual(delta_entry.text_size, revtree.get_file_size(new_name))
+                    if getattr(delta_entry, 'text_sha1', None):
+                        self.assertEqual(delta_entry.text_sha1, revtree.get_file_sha1(new_name))
                 elif delta_entry.kind == 'symlink':
                     self.assertEqual(delta_entry.symlink_target, new_entry.symlink_target)
             else:

=== modified file 'breezy/tests/per_tree/test_transform.py'
--- breezy/tests/per_tree/test_transform.py	2020-07-06 01:43:04 +0000
+++ breezy/tests/per_tree/test_transform.py	2020-08-06 22:38:03 +0000
@@ -610,8 +610,7 @@
         preview.fixup_new_roots()
         preview_tree = preview.get_preview_tree()
         preview.new_file('a', preview.root, [b'contents'], b'a-id')
-        expected = [(('', b'tree-root'),
-                     [('a', 'a', 'file', None, b'a-id', 'file')])]
+        expected = [('', [('a', 'a', 'file', None, 'file')])]
         self.assertEqual(expected, list(preview_tree.walkdirs()))
 
     def test_extras(self):

=== modified file 'breezy/tests/per_tree/test_walkdirs.py'
--- breezy/tests/per_tree/test_walkdirs.py	2018-11-11 04:08:32 +0000
+++ breezy/tests/per_tree/test_walkdirs.py	2020-08-06 22:38:03 +0000
@@ -33,48 +33,42 @@
         dirblocks = {}
 
         dirblocks[''] = [
-            ('0file', '0file', 'file', None,
-                tree.path2id('0file'), 'file'),
-            ('1top-dir', '1top-dir', 'directory', None,
-                tree.path2id('1top-dir'), 'directory'),
-            (u'2utf\u1234file', u'2utf\u1234file', 'file', None,
-                tree.path2id(u'2utf\u1234file'), 'file')]
+            ('0file', '0file', 'file', None, 'file'),
+            ('1top-dir', '1top-dir', 'directory', None, 'directory'),
+            (u'2utf\u1234file', u'2utf\u1234file', 'file', None, 'file')]
 
         dirblocks['1top-dir'] = [
             ('1top-dir/0file-in-1topdir', '0file-in-1topdir',
-             'file', None, tree.path2id('1top-dir/0file-in-1topdir'), 'file')]
+             'file', None, 'file')]
         if empty_dirs_present:
             dirblocks['1top-dir'].append(
                 ('1top-dir/1dir-in-1topdir', '1dir-in-1topdir', 'directory',
                  None if empty_dirs_are_versioned else os.stat(
                      tree.abspath('1top-dir/1dir-in-1topdir')),
-                 tree.path2id('1top-dir/1dir-in-1topdir'),
                  'directory' if empty_dirs_are_versioned else None))
             dirblocks['1top-dir/1dir-in-1topdir'] = []
         if symlinks:
             dirblocks[''].append(
-                ('symlink', 'symlink', 'symlink', None,
-                 tree.path2id('symlink'), 'symlink'))
-        return [((path, tree.path2id(path)), list(sorted(entries)))
+                ('symlink', 'symlink', 'symlink', None, 'symlink'))
+        return [(path, list(sorted(entries)))
                 for (path, entries) in sorted(dirblocks.items())]
 
     def test_walkdir_root(self):
         tree = self.get_tree_with_subdirs_and_all_supported_content_types(
             has_symlinks())
-        tree.lock_read()
-        expected_dirblocks = self.get_all_subdirs_expected(
-            tree, has_symlinks())
-        # test that its iterable by iterating
-        result = []
-        for dirinfo, block in tree.walkdirs():
-            newblock = []
-            for row in block:
-                if row[4] is not None:
-                    newblock.append(row[0:3] + (None,) + row[4:])
-                else:
-                    newblock.append(row)
-            result.append((dirinfo, newblock))
-        tree.unlock()
+        with tree.lock_read():
+            expected_dirblocks = self.get_all_subdirs_expected(
+                tree, has_symlinks())
+            # test that its iterable by iterating
+            result = []
+            for dirinfo, block in tree.walkdirs():
+                newblock = []
+                for row in block:
+                    if row[4] is not None:
+                        newblock.append(row[0:3] + (None,) + row[4:])
+                    else:
+                        newblock.append(row)
+                result.append((dirinfo, newblock))
         # check each return value for debugging ease.
         for pos, item in enumerate(expected_dirblocks):
             self.assertEqual(item, result[pos])
@@ -85,18 +79,17 @@
             has_symlinks())
         # test that its iterable by iterating
         result = []
-        tree.lock_read()
-        expected_dirblocks = self.get_all_subdirs_expected(
-            tree, has_symlinks())[1:]
-        for dirinfo, block in tree.walkdirs('1top-dir'):
-            newblock = []
-            for row in block:
-                if row[4] is not None:
-                    newblock.append(row[0:3] + (None,) + row[4:])
-                else:
-                    newblock.append(row)
-            result.append((dirinfo, newblock))
-        tree.unlock()
+        with tree.lock_read():
+            expected_dirblocks = self.get_all_subdirs_expected(
+                tree, has_symlinks())[1:]
+            for dirinfo, block in tree.walkdirs('1top-dir'):
+                newblock = []
+                for row in block:
+                    if row[4] is not None:
+                        newblock.append(row[0:3] + (None,) + row[4:])
+                    else:
+                        newblock.append(row)
+                result.append((dirinfo, newblock))
         # check each return value for debugging ease.
         for pos, item in enumerate(expected_dirblocks):
             self.assertEqual(item, result[pos])
@@ -106,8 +99,6 @@
         work_tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/file', 'tree/dir/'])
         work_tree.add(['file', 'dir'])
-        file_id = work_tree.path2id('file')
-        dir_id = work_tree.path2id('dir')
         os.unlink('tree/file')
         os.rmdir('tree/dir')
         tree = self._convert_tree(work_tree)
@@ -116,10 +107,10 @@
         if tree.path2id('file') is None:
             raise tests.TestNotApplicable(
                 'Tree type cannot represent dangling ids.')
-        expected = [(('', work_tree.path2id('')), ([
-            ('dir', 'dir', 'unknown', None, dir_id, 'directory')]
+        expected = [('', ([
+            ('dir', 'dir', 'unknown', None, 'directory')]
             if tree.has_versioned_directories() else []) +
-            [('file', 'file', 'unknown', None, file_id, 'file')])]
+            [('file', 'file', 'unknown', None, 'file')])]
         if tree.has_versioned_directories():
-            expected.append((('dir', dir_id), []))
+            expected.append(('dir', []))
         self.assertEqual(expected, list(tree.walkdirs()))

=== modified file 'breezy/tests/per_workingtree/test_commit.py'
--- breezy/tests/per_workingtree/test_commit.py	2020-06-21 02:15:25 +0000
+++ breezy/tests/per_workingtree/test_commit.py	2020-08-10 03:38:37 +0000
@@ -102,7 +102,10 @@
         # Merging from A should introduce conflicts because 'n' was modified
         # (in A) and removed (in B), so 'a' needs to be restored.
         num_conflicts = tree_b.merge_from_branch(tree_a.branch)
-        self.assertEqual(3, num_conflicts)
+        if tree_b.has_versioned_directories():
+            self.assertEqual(3, num_conflicts)
+        else:
+            self.assertEqual(2, num_conflicts)
 
         self.assertThat(
             tree_b, HasPathRelations(
@@ -113,7 +116,7 @@
         osutils.rmtree('B/a')
         try:
             # bzr resolve --all
-            tree_b.set_conflicts(conflicts.ConflictList())
+            tree_b.set_conflicts([])
         except errors.UnsupportedOperation:
             # On WT2, set_conflicts is unsupported, but the rmtree has the same
             # effect.

=== modified file 'breezy/tests/per_workingtree/test_merge_from_branch.py'
--- breezy/tests/per_workingtree/test_merge_from_branch.py	2019-06-29 13:16:26 +0000
+++ breezy/tests/per_workingtree/test_merge_from_branch.py	2020-08-10 03:38:37 +0000
@@ -265,7 +265,7 @@
         else:
             self.assertEqual(0, nb_conflicts)
         try:
-            outer.set_conflicts(conflicts.ConflictList())
+            outer.set_conflicts([])
         except errors.UnsupportedOperation:
             # WT2 doesn't have a separate list of conflicts to clear. It
             # actually says there is a conflict, but happily forgets all about
@@ -284,7 +284,10 @@
                                    'foo'],
                                   outer)
         else:
-            self.assertEqual(2, nb_conflicts)
+            if outer.has_versioned_directories():
+                self.assertEqual(2, nb_conflicts)
+            else:
+                self.assertEqual(1, nb_conflicts)
             self.assertTreeLayout(['dir',
                                    'dir-outer',
                                    'dir-outer/dir',

=== modified file 'breezy/tests/per_workingtree/test_transform.py'
--- breezy/tests/per_workingtree/test_transform.py	2020-07-20 02:02:23 +0000
+++ breezy/tests/per_workingtree/test_transform.py	2020-08-10 15:00:17 +0000
@@ -28,7 +28,7 @@
     transform,
     urlutils,
     )
-from ...conflicts import (
+from ...bzr.conflicts import (
     DeletingParent,
     DuplicateEntry,
     DuplicateID,
@@ -58,7 +58,6 @@
     )
 from ...transform import (
     create_from_tree,
-    cook_conflicts,
     FinalPaths,
     resolve_conflicts,
     resolve_checkout,
@@ -383,9 +382,9 @@
         transform.new_file('toto', dorothy, [b'toto-contents'], b'toto-id',
                            False)
 
-        self.assertEqual(len(transform.find_conflicts()), 0)
+        self.assertEqual(len(transform.find_raw_conflicts()), 0)
         transform.apply()
-        self.assertRaises(ReusingTransform, transform.find_conflicts)
+        self.assertRaises(ReusingTransform, transform.find_raw_conflicts)
         with open(self.wt.abspath('name'), 'r') as f:
             self.assertEqual('contents', f.read())
         self.assertEqual(self.wt.path2id('name'), b'my_pretties')
@@ -418,46 +417,46 @@
         transform, root = self.transform()
         trans_id = transform.new_file('name', root, [b'contents'],
                                       b'my_pretties')
-        self.assertEqual(len(transform.find_conflicts()), 0)
+        self.assertEqual(len(transform.find_raw_conflicts()), 0)
         trans_id2 = transform.new_file('name', root, [b'Crontents'], b'toto')
-        self.assertEqual(transform.find_conflicts(),
+        self.assertEqual(transform.find_raw_conflicts(),
                          [('duplicate', trans_id, trans_id2, 'name')])
         self.assertRaises(MalformedTransform, transform.apply)
         transform.adjust_path('name', trans_id, trans_id2)
-        self.assertEqual(transform.find_conflicts(),
+        self.assertEqual(transform.find_raw_conflicts(),
                          [('non-directory parent', trans_id)])
         tinman_id = transform.trans_id_tree_path('tinman')
         transform.adjust_path('name', tinman_id, trans_id2)
-        self.assertEqual(transform.find_conflicts(),
+        self.assertEqual(transform.find_raw_conflicts(),
                          [('unversioned parent', tinman_id),
                           ('missing parent', tinman_id)])
         lion_id = transform.create_path('lion', root)
-        self.assertEqual(transform.find_conflicts(),
+        self.assertEqual(transform.find_raw_conflicts(),
                          [('unversioned parent', tinman_id),
                           ('missing parent', tinman_id)])
         transform.adjust_path('name', lion_id, trans_id2)
-        self.assertEqual(transform.find_conflicts(),
+        self.assertEqual(transform.find_raw_conflicts(),
                          [('unversioned parent', lion_id),
                           ('missing parent', lion_id)])
         transform.version_file(lion_id, file_id=b"Courage")
-        self.assertEqual(transform.find_conflicts(),
+        self.assertEqual(transform.find_raw_conflicts(),
                          [('missing parent', lion_id),
                           ('versioning no contents', lion_id)])
         transform.adjust_path('name2', root, trans_id2)
-        self.assertEqual(transform.find_conflicts(),
+        self.assertEqual(transform.find_raw_conflicts(),
                          [('versioning no contents', lion_id)])
         transform.create_file([b'Contents, okay?'], lion_id)
         transform.adjust_path('name2', trans_id2, trans_id2)
-        self.assertEqual(transform.find_conflicts(),
+        self.assertEqual(transform.find_raw_conflicts(),
                          [('parent loop', trans_id2),
                           ('non-directory parent', trans_id2)])
         transform.adjust_path('name2', root, trans_id2)
         oz_id = transform.new_directory('oz', root)
         transform.set_executability(True, oz_id)
-        self.assertEqual(transform.find_conflicts(),
+        self.assertEqual(transform.find_raw_conflicts(),
                          [('unversioned executability', oz_id)])
         transform.version_file(oz_id, file_id=b'oz-id')
-        self.assertEqual(transform.find_conflicts(),
+        self.assertEqual(transform.find_raw_conflicts(),
                          [('non-file executability', oz_id)])
         transform.set_executability(None, oz_id)
         tip_id = transform.new_file('tip', oz_id, [b'ozma'], b'tip-id')
@@ -468,7 +467,7 @@
         transform2, root = self.transform()
         oz_id = transform2.trans_id_tree_path('oz')
         newtip = transform2.new_file('tip', oz_id, [b'other'], b'tip-id')
-        result = transform2.find_conflicts()
+        result = transform2.find_raw_conflicts()
         fp = FinalPaths(transform2)
         self.assertTrue('oz/tip' in transform2._tree_path_ids)
         self.assertEqual(fp.get_path(newtip), pathjoin('oz', 'tip'))
@@ -482,7 +481,7 @@
         self.addCleanup(transform3.finalize)
         oz_id = transform3.trans_id_tree_path('oz')
         transform3.delete_contents(oz_id)
-        self.assertEqual(transform3.find_conflicts(),
+        self.assertEqual(transform3.find_raw_conflicts(),
                          [('missing parent', oz_id)])
         root_id = transform3.root
         tip_id = transform3.trans_id_tree_path('oz/tip')
@@ -499,7 +498,7 @@
         self.addCleanup(transform.finalize)
         transform.new_file('file', transform.root, [b'content'])
         transform.new_file('FiLe', transform.root, [b'content'])
-        result = transform.find_conflicts()
+        result = transform.find_raw_conflicts()
         self.assertEqual([], result)
         transform.finalize()
         # Force the tree to report that it is case insensitive, for conflict
@@ -509,7 +508,7 @@
         self.addCleanup(transform.finalize)
         transform.new_file('file', transform.root, [b'content'])
         transform.new_file('FiLe', transform.root, [b'content'])
-        result = transform.find_conflicts()
+        result = transform.find_raw_conflicts()
         self.assertEqual([('duplicate', 'new-1', 'new-2', 'file')], result)
 
     def test_conflict_on_case_insensitive_existing(self):
@@ -522,7 +521,7 @@
         transform = tree.transform()
         self.addCleanup(transform.finalize)
         transform.new_file('file', transform.root, [b'content'])
-        result = transform.find_conflicts()
+        result = transform.find_raw_conflicts()
         self.assertEqual([], result)
         transform.finalize()
         # Force the tree to report that it is case insensitive, for conflict
@@ -531,7 +530,7 @@
         transform = tree.transform()
         self.addCleanup(transform.finalize)
         transform.new_file('file', transform.root, [b'content'])
-        result = transform.find_conflicts()
+        result = transform.find_raw_conflicts()
         self.assertEqual([('duplicate', 'new-1', 'new-2', 'file')], result)
 
     def test_resolve_case_insensitive_conflict(self):
@@ -627,7 +626,7 @@
         self.addCleanup(unversion.finalize)
         parent = unversion.trans_id_tree_path('parent')
         unversion.unversion_file(parent)
-        self.assertEqual(unversion.find_conflicts(),
+        self.assertEqual(unversion.find_raw_conflicts(),
                          [('unversioned parent', parent_id)])
         file_id = unversion.trans_id_tree_path('parent/child')
         unversion.unversion_file(file_id)
@@ -766,7 +765,7 @@
         replace = self.wt.transform()
         self.addCleanup(replace.finalize)
         name2 = replace.new_file('name2', root, [b'hello2'], b'name1')
-        conflicts = replace.find_conflicts()
+        conflicts = replace.find_raw_conflicts()
         name1 = replace.trans_id_tree_path('name1')
         self.assertEqual(conflicts, [('duplicate id', name1, name2)])
         resolve_conflicts(replace)
@@ -786,7 +785,7 @@
         transform.create_symlink(link_target2, wiz_id)
         transform.version_file(wiz_id, file_id=b'wiz-id2')
         transform.set_executability(True, wiz_id)
-        self.assertEqual(transform.find_conflicts(),
+        self.assertEqual(transform.find_raw_conflicts(),
                          [('non-file executability', wiz_id)])
         transform.set_executability(None, wiz_id)
         transform.apply()
@@ -815,7 +814,7 @@
             tt = wt.transform()
             self.addCleanup(tt.finalize)
             tt.new_symlink('foo', tt.root, 'bar')
-            result = tt.find_conflicts()
+            result = tt.find_raw_conflicts()
             self.assertEqual([], result)
         os_symlink = getattr(os, 'symlink', None)
         os.symlink = None
@@ -862,7 +861,7 @@
     def test_cook_conflicts(self):
         tt, emerald, oz, old_dorothy, new_dorothy = self.get_conflicted()
         raw_conflicts = resolve_conflicts(tt)
-        cooked_conflicts = cook_conflicts(raw_conflicts, tt)
+        cooked_conflicts = tt.cook_conflicts(raw_conflicts)
         duplicate = DuplicateEntry('Moved existing file to', 'dorothy.moved',
                                    'dorothy', None, b'dorothy-id')
         self.assertEqual(cooked_conflicts[0], duplicate)
@@ -892,7 +891,7 @@
     def test_string_conflicts(self):
         tt, emerald, oz, old_dorothy, new_dorothy = self.get_conflicted()
         raw_conflicts = resolve_conflicts(tt)
-        cooked_conflicts = cook_conflicts(raw_conflicts, tt)
+        cooked_conflicts = tt.cook_conflicts(raw_conflicts)
         tt.finalize()
         conflicts_s = [str(c) for c in cooked_conflicts]
         self.assertEqual(len(cooked_conflicts), len(conflicts_s))
@@ -925,16 +924,16 @@
         tt.new_file('child,', parent_id, [b'contents2'], b'file-id')
         return tt
 
-    def test_find_conflicts_wrong_parent_kind(self):
+    def test_find_raw_conflicts_wrong_parent_kind(self):
         tt = self.prepare_wrong_parent_kind()
-        tt.find_conflicts()
+        tt.find_raw_conflicts()
 
     def test_resolve_conflicts_wrong_existing_parent_kind(self):
         tt = self.prepare_wrong_parent_kind()
         raw_conflicts = resolve_conflicts(tt)
         self.assertEqual({('non-directory parent', 'Created directory',
                            'new-3')}, raw_conflicts)
-        cooked_conflicts = cook_conflicts(raw_conflicts, tt)
+        cooked_conflicts = tt.cook_conflicts(raw_conflicts)
         self.assertEqual([NonDirectoryParent('Created directory', 'parent.new',
                                              b'parent-id')], cooked_conflicts)
         tt.apply()

=== modified file 'breezy/tests/per_workingtree/test_unversion.py'
--- breezy/tests/per_workingtree/test_unversion.py	2018-11-11 04:08:32 +0000
+++ breezy/tests/per_workingtree/test_unversion.py	2020-08-10 03:38:37 +0000
@@ -176,7 +176,10 @@
         # and removed, so 'a' needs to be restored. We also have a conflict
         # because 'a' is still an existing directory
         num_conflicts = tree_b.merge_from_branch(tree_a.branch)
-        self.assertEqual(4, num_conflicts)
+        if tree_b.has_versioned_directories():
+            self.assertEqual(4, num_conflicts)
+        else:
+            self.assertEqual(3, num_conflicts)
 
         self.assertThat(
             tree_b,

=== modified file 'breezy/tests/per_workingtree/test_walkdirs.py'
--- breezy/tests/per_workingtree/test_walkdirs.py	2020-07-04 12:29:00 +0000
+++ breezy/tests/per_workingtree/test_walkdirs.py	2020-08-06 22:01:46 +0000
@@ -28,7 +28,7 @@
 class DirBlock:
     """Object representation of the tuples returned by dirstate."""
 
-    def __init__(self, tree, file_path, file_name=None, id=None,
+    def __init__(self, tree, file_path, file_name=None,
                  inventory_kind=None, stat=None, disk_kind='unknown'):
         self.file_path = file_path
         self.abspath = tree.abspath(file_path)
@@ -38,17 +38,16 @@
             if len(file_name) == 0:
                 file_name = os.path.split(file_path)[-2]
         self.file_name = file_name
-        self.id = id
         self.inventory_kind = inventory_kind
         self.stat = stat
         self.disk_kind = disk_kind
 
     def as_tuple(self):
         return (self.relpath, self.file_name, self.disk_kind,
-                self.stat, self.id, self.inventory_kind)
+                self.stat, self.inventory_kind)
 
     def as_dir_tuple(self):
-        return (self.relpath, self.id)
+        return self.relpath
 
     def __str__(self):
         return """
@@ -56,11 +55,10 @@
 abspath        = %r
 relpath        = %r
 file_name      = %r
-id             = %r
 inventory_kind = %r
 stat           = %r
 disk_kind      = %r""" % (self.file_path, self.abspath, self.relpath,
-                          self.file_name, self.id, self.inventory_kind, self.stat,
+                          self.file_name, self.inventory_kind, self.stat,
                           self.disk_kind)
 
 
@@ -97,9 +95,6 @@
 
         if file_status != self.unknown:
             tree.add(paths)
-            for dirblock in dirblocks:
-                if file_status != self.unknown:
-                    dirblock.id = tree.path2id(dirblock.relpath)
 
         if file_status == self.missing:
             # now make the files be missing
@@ -108,7 +103,7 @@
             tree.controldir.root_transport.delete_tree(dirblocks[3].relpath)
 
         expected_dirblocks = [
-            (('', tree.path2id('')),
+            ('',
              [dirblocks[1].as_tuple()] +
              ([dirblocks[3].as_tuple()]
                  if (tree.has_versioned_directories() or file_status == self.unknown) else []) +
@@ -120,22 +115,18 @@
             ]
         if (tree.has_versioned_directories() or
                 file_status != self.missing):
-            expected_dirblocks.append(
-                (dirblocks[3].as_dir_tuple(),
-                 []
-                 ))
+            expected_dirblocks.append((dirblocks[3].as_dir_tuple(), []))
         if prefix:
             expected_dirblocks = [e for e in expected_dirblocks
-                                  if len(e) > 0 and len(e[0]) > 0 and e[0][0] == prefix]
+                                  if len(e) > 0 and len(e[0]) > 0 and e[0] == prefix]
         return tree, expected_dirblocks
 
     def _test_walkdir(self, file_status, prefix=""):
         result = []
         tree, expected_dirblocks = self.get_tree(file_status, prefix)
-        tree.lock_read()
-        for dirinfo, dirblock in tree.walkdirs(prefix):
-            result.append((dirinfo, list(dirblock)))
-        tree.unlock()
+        with tree.lock_read():
+            for dirpath, dirblock in tree.walkdirs(prefix):
+                result.append((dirpath, list(dirblock)))
 
         # check each return value for debugging ease.
         for pos, item in enumerate(expected_dirblocks):
@@ -200,40 +191,25 @@
         link1_stat = os.lstat('link1')
         link2_stat = os.lstat('link2')
         expected_dirblocks = [
-            (('', tree.path2id('')),
-             [('dir1', 'dir1', 'file', dir1_stat, tree.path2id('dir1'),
+            ('',
+             [('dir1', 'dir1', 'file', dir1_stat,
                'directory' if tree.has_versioned_directories() else None),
-              ('dir2', 'dir2', 'symlink', dir2_stat, tree.path2id('dir2'),
+              ('dir2', 'dir2', 'symlink', dir2_stat,
                'directory' if tree.has_versioned_directories() else None),
               ('file1', 'file1', 'directory', file1_stat,
-               tree.path2id('file1'), 'file'),
-              ('file2', 'file2', 'symlink', file2_stat,
-               tree.path2id('file2'), 'file'),
-              ('link1', 'link1', 'file', link1_stat,
-               tree.path2id('link1'), 'symlink'),
-              ('link2', 'link2', 'directory', link2_stat,
-               tree.path2id('link2'), 'symlink'),
+               'file'),
+              ('file2', 'file2', 'symlink', file2_stat, 'file'),
+              ('link1', 'link1', 'file', link1_stat, 'symlink'),
+              ('link2', 'link2', 'directory', link2_stat, 'symlink'),
               ]
              )]
         if tree.has_versioned_directories():
             expected_dirblocks.extend([
-                (('dir1', tree.path2id('dir1')),
-                 [
-                    ]
-                 ),
-                (('dir2', tree.path2id('dir2')),
-                    [
-                    ]
-                 )])
+                ('dir1', []),
+                ('dir2', [])])
         expected_dirblocks.extend([
-            (('file1', None),
-             [
-                ]
-             ),
-            (('link2', None),
-             [
-                ]
-             ),
+            ('file1', []),
+            ('link2', []),
             ])
         with tree.lock_read():
             result = list(tree.walkdirs())
@@ -257,33 +233,21 @@
         file1_stat = os.lstat('file1')
         if tree.has_versioned_directories():
             expected_dirblocks = [
-                (('', tree.path2id('')),
-                 [('dir1', 'dir1', 'file', dir1_stat, tree.path2id('dir1'), 'directory'),
-                  ('file1', 'file1', 'directory', file1_stat,
-                   tree.path2id('file1'), 'file'),
-                  ]
-                 ),
-                (('dir1', tree.path2id('dir1')),
-                 [
-                    ]
-                 ),
-                (('file1', None),
-                 [
-                    ]
-                 ),
+                ('',
+                 [('dir1', 'dir1', 'file', dir1_stat, 'directory'),
+                  ('file1', 'file1', 'directory', file1_stat, 'file'),
+                 ]),
+                ('dir1', []),
+                ('file1', []),
                 ]
         else:
             expected_dirblocks = [
-                (('', tree.path2id('')),
-                 [('dir1', 'dir1', 'file', dir1_stat, tree.path2id('dir1'), None),
-                  ('file1', 'file1', 'directory', file1_stat,
-                   tree.path2id('file1'), 'file'),
+                ('',
+                 [('dir1', 'dir1', 'file', dir1_stat, None),
+                  ('file1', 'file1', 'directory', file1_stat, 'file'),
                   ]
                  ),
-                (('file1', None),
-                 [
-                    ]
-                 ),
+                ('file1', []),
                 ]
         with tree.lock_read():
             result = list(tree.walkdirs())

=== modified file 'breezy/tests/per_workingtree/test_workingtree.py'
--- breezy/tests/per_workingtree/test_workingtree.py	2020-07-18 23:14:00 +0000
+++ breezy/tests/per_workingtree/test_workingtree.py	2020-08-10 15:00:17 +0000
@@ -56,7 +56,7 @@
     TreeFile,
     TreeLink,
     )
-from ...conflicts import ConflictList, TextConflict, ContentsConflict
+from ...bzr.conflicts import ConflictList, TextConflict, ContentsConflict
 from ...workingtree import (
     SettingFileIdUnsupported,
     WorkingTree,
@@ -797,7 +797,7 @@
         tree = self.make_merge_conflicts()
         self.assertEqual(len(tree.conflicts()), 1)
         try:
-            tree.set_conflicts(ConflictList())
+            tree.set_conflicts([])
         except UnsupportedOperation:
             raise TestSkipped('unsupported operation')
         self.assertEqual(tree.conflicts(), ConflictList())
@@ -929,11 +929,11 @@
             present_stat = os.lstat('present')
             unknown_stat = os.lstat('unknown')
             expected_results = [
-                (('', tree.path2id('')),
-                 [('missing', 'missing', 'unknown', None, b'missing-id', 'file'),
+                ('',
+                 [('missing', 'missing', 'unknown', None, 'file'),
                   ('present', 'present', 'file',
-                   present_stat, b'present-id', 'file'),
-                  ('unknown', 'unknown', 'file', unknown_stat, None, None),
+                   present_stat, 'file'),
+                  ('unknown', 'unknown', 'file', unknown_stat, None),
                   ]
                  )]
             self.assertEqual(expected_results, list(tree.walkdirs()))

=== modified file 'breezy/tests/test_commit.py'
--- breezy/tests/test_commit.py	2020-06-10 21:13:00 +0000
+++ breezy/tests/test_commit.py	2020-08-06 01:40:59 +0000
@@ -38,7 +38,7 @@
     BzrError,
     LockContention,
     )
-from ..tree import TreeChange
+from ..bzr.inventorytree import InventoryTreeChange
 from . import (
     TestCase,
     TestCaseWithTransport,
@@ -892,7 +892,7 @@
 
     def test_add_file_not_excluded(self):
         changes = [
-            TreeChange(
+            InventoryTreeChange(
                 'fid', (None, 'newpath'),
                 0, (False, False), ('pid', 'pid'), ('newpath', 'newpath'),
                 ('file', 'file'), (True, True))]
@@ -901,7 +901,7 @@
 
     def test_add_file_excluded(self):
         changes = [
-            TreeChange(
+            InventoryTreeChange(
                 'fid', (None, 'newpath'),
                 0, (False, False), ('pid', 'pid'), ('newpath', 'newpath'),
                 ('file', 'file'), (True, True))]
@@ -909,7 +909,7 @@
 
     def test_delete_file_excluded(self):
         changes = [
-            TreeChange(
+            InventoryTreeChange(
                 'fid', ('somepath', None),
                 0, (False, None), ('pid', None), ('newpath', None),
                 ('file', None), (True, None))]
@@ -917,7 +917,7 @@
 
     def test_move_from_or_to_excluded(self):
         changes = [
-            TreeChange(
+            InventoryTreeChange(
                 'fid', ('oldpath', 'newpath'),
                 0, (False, False), ('pid', 'pid'), ('oldpath', 'newpath'),
                 ('file', 'file'), (True, True))]

=== modified file 'breezy/tests/test_conflicts.py'
--- breezy/tests/test_conflicts.py	2020-07-18 23:14:00 +0000
+++ breezy/tests/test_conflicts.py	2020-08-10 15:00:17 +0000
@@ -14,7 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 import os
 
 from .. import (
@@ -25,6 +24,7 @@
     tests,
     transform,
     )
+from ..bzr import conflicts as bzr_conflicts
 from . import (
     script,
     scenarios,
@@ -42,24 +42,24 @@
 # u'\xe5' == a with circle
 # '\xc3\xae' == u'\xee' == i with hat
 # So these are u'path' and 'id' only with a circle and a hat. (shappo?)
-example_conflicts = conflicts.ConflictList(
-    [conflicts.MissingParent('Not deleting', u'p\xe5thg', b'\xc3\xaedg'),
-     conflicts.ContentsConflict(u'p\xe5tha', None, b'\xc3\xaeda'),
-     conflicts.TextConflict(u'p\xe5tha'),
-     conflicts.PathConflict(u'p\xe5thb', u'p\xe5thc', b'\xc3\xaedb'),
-     conflicts.DuplicateID('Unversioned existing file',
-                           u'p\xe5thc', u'p\xe5thc2',
-                           b'\xc3\xaedc', b'\xc3\xaedc'),
-     conflicts.DuplicateEntry('Moved existing file to',
-                              u'p\xe5thdd.moved', u'p\xe5thd',
-                              b'\xc3\xaedd', None),
-     conflicts.ParentLoop('Cancelled move', u'p\xe5the', u'p\xe5th2e',
-                          None, b'\xc3\xaed2e'),
-     conflicts.UnversionedParent('Versioned directory',
-                                 u'p\xe5thf', b'\xc3\xaedf'),
-     conflicts.NonDirectoryParent('Created directory',
-                                  u'p\xe5thg', b'\xc3\xaedg'),
-     ])
+example_conflicts = [
+     bzr_conflicts.MissingParent('Not deleting', u'p\xe5thg', b'\xc3\xaedg'),
+     bzr_conflicts.ContentsConflict(u'p\xe5tha', None, b'\xc3\xaeda'),
+     bzr_conflicts.TextConflict(u'p\xe5tha'),
+     bzr_conflicts.PathConflict(u'p\xe5thb', u'p\xe5thc', b'\xc3\xaedb'),
+     bzr_conflicts.DuplicateID('Unversioned existing file',
+                               u'p\xe5thc', u'p\xe5thc2',
+                               b'\xc3\xaedc', b'\xc3\xaedc'),
+     bzr_conflicts.DuplicateEntry('Moved existing file to',
+                                  u'p\xe5thdd.moved', u'p\xe5thd',
+                                  b'\xc3\xaedd', None),
+     bzr_conflicts.ParentLoop('Cancelled move', u'p\xe5the', u'p\xe5th2e',
+                              None, b'\xc3\xaed2e'),
+     bzr_conflicts.UnversionedParent('Versioned directory',
+                                     u'p\xe5thf', b'\xc3\xaedf'),
+     bzr_conflicts.NonDirectoryParent('Created directory',
+                                      u'p\xe5thg', b'\xc3\xaedg'),
+     ]
 
 
 def vary_by_conflicts():
@@ -77,7 +77,7 @@
                                   ])
         os.mkdir('hello.OTHER')
         tree.add('hello', b'q')
-        l = conflicts.ConflictList([conflicts.TextConflict('hello')])
+        l = conflicts.ConflictList([bzr_conflicts.TextConflict('hello')])
         l.remove_files(tree)
 
     def test_select_conflicts(self):
@@ -89,24 +89,24 @@
                 (not_selected, selected),
                 tree_conflicts.select_conflicts(tree, paths, **kwargs))
 
-        foo = conflicts.ContentsConflict('foo')
-        bar = conflicts.ContentsConflict('bar')
+        foo = bzr_conflicts.ContentsConflict('foo')
+        bar = bzr_conflicts.ContentsConflict('bar')
         tree_conflicts = clist([foo, bar])
 
         check_select(clist([bar]), clist([foo]), ['foo'])
         check_select(clist(), tree_conflicts,
                      [''], ignore_misses=True, recurse=True)
 
-        foobaz = conflicts.ContentsConflict('foo/baz')
+        foobaz = bzr_conflicts.ContentsConflict('foo/baz')
         tree_conflicts = clist([foobaz, bar])
 
         check_select(clist([bar]), clist([foobaz]),
                      ['foo'], ignore_misses=True, recurse=True)
 
-        qux = conflicts.PathConflict('qux', 'foo/baz')
+        qux = bzr_conflicts.PathConflict('qux', 'foo/baz')
         tree_conflicts = clist([qux])
 
-        check_select(clist(), tree_conflicts,
+        check_select(tree_conflicts, clist(),
                      ['foo'], ignore_misses=True, recurse=True)
         check_select(tree_conflicts, clist(), ['foo'], ignore_misses=True)
 
@@ -115,8 +115,7 @@
         self.build_tree(['dir/', 'dir/hello'])
         tree.add(['dir', 'dir/hello'])
 
-        dirhello = conflicts.ConflictList(
-            [conflicts.TextConflict('dir/hello')])
+        dirhello = [bzr_conflicts.TextConflict('dir/hello')]
         tree.set_conflicts(dirhello)
 
         conflicts.resolve(tree, ['dir'], recursive=False, ignore_misses=True)
@@ -137,46 +136,19 @@
         self.assertContainsString(repr(self.conflict),
                                   self.conflict.__class__.__name__)
 
-    def test_stanza_roundtrip(self):
-        p = self.conflict
-        o = conflicts.Conflict.factory(**p.as_stanza().as_dict())
-        self.assertEqual(o, p)
-
-        self.assertIsInstance(o.path, str)
-
-        if o.file_id is not None:
-            self.assertIsInstance(o.file_id, bytes)
-
-        conflict_path = getattr(o, 'conflict_path', None)
-        if conflict_path is not None:
-            self.assertIsInstance(conflict_path, str)
-
-        conflict_file_id = getattr(o, 'conflict_file_id', None)
-        if conflict_file_id is not None:
-            self.assertIsInstance(conflict_file_id, bytes)
-
-    def test_stanzification(self):
-        stanza = self.conflict.as_stanza()
-        if 'file_id' in stanza:
-            # In Stanza form, the file_id has to be unicode.
-            self.assertStartsWith(stanza['file_id'], u'\xeed')
-        self.assertStartsWith(stanza['path'], u'p\xe5th')
-        if 'conflict_path' in stanza:
-            self.assertStartsWith(stanza['conflict_path'], u'p\xe5th')
-        if 'conflict_file_id' in stanza:
-            self.assertStartsWith(stanza['conflict_file_id'], u'\xeed')
-
 
 class TestConflictList(tests.TestCase):
 
     def test_stanzas_roundtrip(self):
-        stanzas_iter = example_conflicts.to_stanzas()
-        processed = conflicts.ConflictList.from_stanzas(stanzas_iter)
+        stanzas_iter = bzr_conflicts.ConflictList(example_conflicts).to_stanzas()
+        processed = bzr_conflicts.ConflictList.from_stanzas(stanzas_iter)
         self.assertEqual(example_conflicts, processed)
 
     def test_stringification(self):
-        for text, o in zip(example_conflicts.to_strings(), example_conflicts):
-            self.assertEqual(text, str(o))
+        for text, o in zip(
+                bzr_conflicts.ConflictList(example_conflicts).to_strings(),
+                example_conflicts):
+            self.assertEqual(text, text_type(o))
 
 
 # FIXME: The shell-like tests should be converted to real whitebox tests... or
@@ -367,7 +339,7 @@
 
 class TestResolveTextConflicts(TestParametrizedResolveConflicts):
 
-    _conflict_type = conflicts.TextConflict
+    _conflict_type = bzr_conflicts.TextConflict
 
     # Set by the scenarios
     # path and file-id for the file involved in the conflict
@@ -438,7 +410,7 @@
 
 class TestResolveContentsConflict(TestParametrizedResolveConflicts):
 
-    _conflict_type = conflicts.ContentsConflict
+    _conflict_type = bzr_conflicts.ContentsConflict
 
     # Set by the scenarios
     # path and file-id for the file involved in the conflict
@@ -524,7 +496,7 @@
 
 class TestResolvePathConflict(TestParametrizedResolveConflicts):
 
-    _conflict_type = conflicts.PathConflict
+    _conflict_type = bzr_conflicts.PathConflict
 
     def do_nothing(self):
         return []
@@ -674,14 +646,14 @@
         # We create a conflict object as it was created before the fix and
         # inject it into the working tree, the test will exercise the
         # compatibility code.
-        old_c = conflicts.PathConflict('<deleted>', self._item_path,
+        old_c = bzr_conflicts.PathConflict('<deleted>', self._item_path,
                                        file_id=None)
-        wt.set_conflicts(conflicts.ConflictList([old_c]))
+        wt.set_conflicts([old_c])
 
 
 class TestResolveDuplicateEntry(TestParametrizedResolveConflicts):
 
-    _conflict_type = conflicts.DuplicateEntry
+    _conflict_type = bzr_conflicts.DuplicateEntry
 
     scenarios = mirror_scenarios(
         [
@@ -920,7 +892,7 @@
 
 class TestResolveParentLoop(TestParametrizedResolveConflicts):
 
-    _conflict_type = conflicts.ParentLoop
+    _conflict_type = bzr_conflicts.ParentLoop
 
     _this_args = None
     _other_args = None

=== modified file 'breezy/tests/test_delta.py'
--- breezy/tests/test_delta.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/test_delta.py	2020-08-10 15:00:17 +0000
@@ -22,7 +22,7 @@
     revision as _mod_revision,
     tests,
     )
-from ..tree import TreeChange
+from ..bzr.inventorytree import InventoryTreeChange
 
 
 class InstrumentedReporter(object):
@@ -161,7 +161,7 @@
                            exe_change=False):
         reporter = InstrumentedReporter()
         _mod_delta.report_changes([
-            TreeChange(
+            InventoryTreeChange(
                 file_id, paths, content_change, versioned, parent_id,
                 name, kind, executable, copied)], reporter)
         output = reporter.calls[0]
@@ -258,7 +258,7 @@
         other_delta = _mod_delta.TreeDelta()
         self.assertNotEqual(other_delta, delta)
         other_delta.kind_changed = [
-            TreeChange(
+            InventoryTreeChange(
                 b'file-id',
                 ('filename', 'filename'), True, (True, True),
                 (tree.path2id(''), tree.path2id('')),
@@ -266,7 +266,7 @@
                 ('file', 'symlink'), (False, False))]
         self.assertNotEqual(other_delta, delta)
         other_delta.kind_changed = [
-            TreeChange(
+            InventoryTreeChange(
                 b'file-id',
                 ('filename', 'filename'), True, (True, True),
                 (tree.path2id(''), tree.path2id('')), ('filename', 'filename'),
@@ -281,7 +281,7 @@
         self.assertEqual([], delta.kind_changed)
         # This loses the fact that kind changed, remembering it as a
         # modification
-        self.assertEqual([TreeChange(
+        self.assertEqual([InventoryTreeChange(
             b'file-id', ('filename', 'dirname'), True,
             (True, True), (tree.path2id(''), tree.path2id('')),
             ('filename', 'dirname'), ('file', 'directory'), (False, False))],

=== modified file 'breezy/tests/test_merge.py'
--- breezy/tests/test_merge.py	2020-07-18 23:14:00 +0000
+++ breezy/tests/test_merge.py	2020-08-10 15:00:17 +0000
@@ -33,7 +33,8 @@
     knit,
     versionedfile,
     )
-from ..conflicts import ConflictList, TextConflict
+from ..conflicts import ConflictList
+from ..bzr.conflicts import TextConflict, MissingParent, UnversionedParent, DeletingParent, ContentsConflict
 from ..errors import UnrelatedBranches, NoCommits
 from ..merge import transform_tree, merge_inner, _PlanMerge
 from ..osutils import basename, pathjoin, file_kind
@@ -134,7 +135,7 @@
                                          this_branch=wt.branch,
                                          do_merge=False)
         with merger.make_preview_transform() as tt:
-            self.assertEqual([], tt.find_conflicts())
+            self.assertEqual([], tt.find_raw_conflicts())
             preview = tt.get_preview_tree()
             self.assertEqual(wt.path2id(''), preview.path2id(''))
 
@@ -197,7 +198,7 @@
 
     def test_merge_inner_conflicts(self):
         tree_a = self.make_branch_and_tree('a')
-        tree_a.set_conflicts(ConflictList([TextConflict('patha')]))
+        tree_a.set_conflicts([TextConflict('patha')])
         merge_inner(tree_a.branch, tree_a, tree_a, this_tree=tree_a)
         self.assertEqual(1, len(tree_a.conflicts()))
 
@@ -219,14 +220,14 @@
         tree_z.commit('removed b')
         merge_inner(tree_z.branch, tree_a, base_tree, this_tree=tree_z)
         self.assertEqual([
-            conflicts.MissingParent('Created directory', 'b', b'b-id'),
-            conflicts.UnversionedParent('Versioned directory', 'b', b'b-id')],
+            MissingParent('Created directory', 'b', b'b-id'),
+            UnversionedParent('Versioned directory', 'b', b'b-id')],
             tree_z.conflicts())
         merge_inner(tree_a.branch, tree_z.basis_tree(), base_tree,
                     this_tree=tree_a)
         self.assertEqual([
-            conflicts.DeletingParent('Not deleting', 'b', b'b-id'),
-            conflicts.UnversionedParent('Versioned directory', 'b', b'b-id')],
+            DeletingParent('Not deleting', 'b', b'b-id'),
+            UnversionedParent('Versioned directory', 'b', b'b-id')],
             tree_a.conflicts())
 
     def test_nested_merge(self):
@@ -287,8 +288,7 @@
         tree_b.commit('content change')
         tree_b.merge_from_branch(tree_a.branch)
         self.assertEqual(tree_b.conflicts(),
-                         [conflicts.ContentsConflict('file',
-                                                     file_id=b'file-id')])
+                         [ContentsConflict('file', file_id=b'file-id')])
 
     def test_merge_type_registry(self):
         merge_type_option = option.Option.OPTIONS['merge-type']

=== modified file 'breezy/tests/test_merge_core.py'
--- breezy/tests/test_merge_core.py	2020-07-04 12:29:00 +0000
+++ breezy/tests/test_merge_core.py	2020-08-09 18:10:01 +0000
@@ -24,7 +24,7 @@
     merge_directive,
     osutils,
     )
-from ..conflicts import (
+from ..bzr.conflicts import (
     ContentsConflict,
     TextConflict,
     PathConflict,

=== modified file 'breezy/tests/test_transform.py'
--- breezy/tests/test_transform.py	2020-07-18 23:14:00 +0000
+++ breezy/tests/test_transform.py	2020-08-10 15:00:17 +0000
@@ -36,7 +36,7 @@
 from ..bzr import (
     generate_ids,
     )
-from ..conflicts import (
+from ..bzr.conflicts import (
     DeletingParent,
     DuplicateEntry,
     DuplicateID,
@@ -72,7 +72,6 @@
 from ..transform import (
     build_tree,
     create_from_tree,
-    cook_conflicts,
     _FileMover,
     FinalPaths,
     resolve_conflicts,
@@ -1414,7 +1413,7 @@
         tt.delete_contents(dir_tid)
         tt.unversion_file(dir_tid)
         # There should be a conflict because dir still contain foo
-        raw_conflicts = tt.find_conflicts()
+        raw_conflicts = tt.find_raw_conflicts()
         self.assertLength(1, raw_conflicts)
         self.assertEqual(('missing parent', 'new-1'), raw_conflicts[0])
         return tt, orphan_tid

=== modified file 'breezy/tests/test_workingtree.py'
--- breezy/tests/test_workingtree.py	2019-06-18 16:48:35 +0000
+++ breezy/tests/test_workingtree.py	2020-08-09 18:10:01 +0000
@@ -28,6 +28,7 @@
     )
 from ..bzr import (
     bzrdir,
+    conflicts as _mod_bzr_conflicts,
     workingtree as bzrworkingtree,
     workingtree_3,
     workingtree_4,
@@ -430,28 +431,28 @@
         self.build_tree_contents([('this/hello', b'Hello World')])
         this.commit('Add World')
         this.merge_from_branch(other.branch)
-        self.assertEqual([conflicts.TextConflict('hello', b'hello_id')],
+        self.assertEqual([_mod_bzr_conflicts.TextConflict('hello', b'hello_id')],
                          this.conflicts())
         self._auto_resolve(this)
-        self.assertEqual([conflicts.TextConflict('hello', b'hello_id')],
+        self.assertEqual([_mod_bzr_conflicts.TextConflict('hello', b'hello_id')],
                          this.conflicts())
         self.build_tree_contents([('this/hello', '<<<<<<<')])
         self._auto_resolve(this)
-        self.assertEqual([conflicts.TextConflict('hello', b'hello_id')],
+        self.assertEqual([_mod_bzr_conflicts.TextConflict('hello', b'hello_id')],
                          this.conflicts())
         self.build_tree_contents([('this/hello', '=======')])
         self._auto_resolve(this)
-        self.assertEqual([conflicts.TextConflict('hello', b'hello_id')],
+        self.assertEqual([_mod_bzr_conflicts.TextConflict('hello', b'hello_id')],
                          this.conflicts())
         self.build_tree_contents([('this/hello', '\n>>>>>>>')])
         remaining, resolved = self._auto_resolve(this)
-        self.assertEqual([conflicts.TextConflict('hello', b'hello_id')],
+        self.assertEqual([_mod_bzr_conflicts.TextConflict('hello', b'hello_id')],
                          this.conflicts())
         self.assertEqual([], resolved)
         self.build_tree_contents([('this/hello', b'hELLO wORLD')])
         remaining, resolved = self._auto_resolve(this)
         self.assertEqual([], this.conflicts())
-        self.assertEqual([conflicts.TextConflict('hello', b'hello_id')],
+        self.assertEqual([_mod_bzr_conflicts.TextConflict('hello', b'hello_id')],
                          resolved)
         self.assertPathDoesNotExist('this/hello.BASE')
 
@@ -487,23 +488,23 @@
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/hello/'])
         tree.add('hello', b'hello-id')
-        file_conflict = conflicts.TextConflict('hello', b'hello-id')
-        tree.set_conflicts(conflicts.ConflictList([file_conflict]))
+        file_conflict = _mod_bzr_conflicts.TextConflict('hello', b'hello-id')
+        tree.set_conflicts([file_conflict])
         remaining, resolved = self._auto_resolve(tree)
         self.assertEqual(
             remaining,
-            conflicts.ConflictList([conflicts.TextConflict(u'hello', 'hello-id')]))
+            conflicts.ConflictList([_mod_bzr_conflicts.TextConflict(u'hello', 'hello-id')]))
         self.assertEqual(resolved, [])
 
     def test_auto_resolve_missing(self):
         tree = self.make_branch_and_tree('tree')
-        file_conflict = conflicts.TextConflict('hello', b'hello-id')
-        tree.set_conflicts(conflicts.ConflictList([file_conflict]))
+        file_conflict = _mod_bzr_conflicts.TextConflict('hello', b'hello-id')
+        tree.set_conflicts([file_conflict])
         remaining, resolved = self._auto_resolve(tree)
         self.assertEqual(remaining, [])
         self.assertEqual(
             resolved,
-            conflicts.ConflictList([conflicts.TextConflict(u'hello', 'hello-id')]))
+            conflicts.ConflictList([_mod_bzr_conflicts.TextConflict(u'hello', 'hello-id')]))
 
 
 class TestStoredUncommitted(TestCaseWithTransport):

=== modified file 'breezy/tests/test_workspace.py'
--- breezy/tests/test_workspace.py	2020-07-27 22:32:15 +0000
+++ breezy/tests/test_workspace.py	2020-08-06 21:52:51 +0000
@@ -19,6 +19,7 @@
 from . import (
     TestCaseWithTransport,
     multiply_scenarios,
+    features,
     )
 from .scenarios import load_tests_apply_scenarios
 
@@ -117,6 +118,11 @@
         vary_by_format(),
     )
 
+    def setUp(self):
+        super(WorkspaceTests, self).setUp()
+        if self._use_inotify:
+            self.requireFeature(features.pyinotify)
+
     def test_root_add(self):
         tree = self.make_branch_and_tree('.', format=self._format)
         with Workspace(tree, use_inotify=self._use_inotify) as ws:

=== modified file 'breezy/transform.py'
--- breezy/transform.py	2020-07-20 02:02:23 +0000
+++ breezy/transform.py	2020-08-10 15:00:17 +0000
@@ -32,7 +32,6 @@
     )
 lazy_import.lazy_import(globals(), """
 from breezy import (
-    conflicts,
     multiparent,
     revision as _mod_revision,
     ui,
@@ -56,7 +55,6 @@
 from .progress import ProgressPhase
 from .tree import (
     InterTree,
-    TreeChange,
     find_previous_path,
     )
 
@@ -392,9 +390,9 @@
     def new_contents(self, trans_id):
         return (trans_id in self._new_contents)
 
-    def find_conflicts(self):
+    def find_raw_conflicts(self):
         """Find any violations of inventory or filesystem invariants"""
-        raise NotImplementedError(self.find_conflicts)
+        raise NotImplementedError(self.find_raw_conflicts)
 
     def new_file(self, name, parent_id, contents, file_id=None,
                  executable=None, sha1=None):
@@ -444,7 +442,7 @@
         """Produce output in the same format as Tree.iter_changes.
 
         Will produce nonsensical results if invoked while inventory/filesystem
-        conflicts (as reported by TreeTransform.find_conflicts()) are present.
+        conflicts (as reported by TreeTransform.find_raw_conflicts()) are present.
 
         This reads the Transform, but only reproduces changes involving a
         file_id.  Files that are not versioned in either of the FROM or TO
@@ -524,6 +522,11 @@
         """Cancel the creation of new file contents."""
         raise NotImplementedError(self.cancel_creation)
 
+    def cook_conflicts(self, raw_conflicts):
+        """Cook conflicts.
+        """
+        raise NotImplementedError(self.cook_conflicts)
+
 
 class OrphaningError(errors.BzrError):
 
@@ -768,7 +771,7 @@
         raw_conflicts = resolve_conflicts(tt, pass_func=resolver)
         if len(raw_conflicts) > 0:
             precomputed_delta = None
-        conflicts = cook_conflicts(raw_conflicts, tt)
+        conflicts = tt.cook_conflicts(raw_conflicts)
         for conflict in conflicts:
             trace.warning(str(conflict))
         try:
@@ -987,7 +990,7 @@
     with ui.ui_factory.nested_progress_bar() as child_pb:
         raw_conflicts = resolve_conflicts(
             tt, child_pb, lambda t, c: conflict_pass(t, c, target_tree))
-    conflicts = cook_conflicts(raw_conflicts, tt)
+    conflicts = tt.cook_conflicts(raw_conflicts)
     return conflicts, merge_modified
 
 
@@ -1008,16 +1011,15 @@
     try:
         deferred_files = []
         for id_num, change in enumerate(change_list):
-            file_id = change.file_id
             target_path, wt_path = change.path
             target_versioned, wt_versioned = change.versioned
-            target_parent, wt_parent = change.parent_id
+            target_parent = change.parent_id[0]
             target_name, wt_name = change.name
             target_kind, wt_kind = change.kind
             target_executable, wt_executable = change.executable
-            if skip_root and wt_parent is None:
+            if skip_root and wt_path == '':
                 continue
-            trans_id = tt.trans_id_file_id(file_id)
+            trans_id = tt.trans_id_file_id(change.file_id)
             mode_id = None
             if change.changed_content:
                 keep_content = False
@@ -1042,14 +1044,15 @@
                     if not keep_content:
                         tt.delete_contents(trans_id)
                     elif target_kind is not None:
-                        parent_trans_id = tt.trans_id_file_id(wt_parent)
+                        parent_trans_id = tt.trans_id_tree_path(osutils.dirname(wt_path))
                         backup_name = tt._available_backup_name(
                             wt_name, parent_trans_id)
                         tt.adjust_path(backup_name, parent_trans_id, trans_id)
                         new_trans_id = tt.create_path(wt_name, parent_trans_id)
                         if wt_versioned and target_versioned:
                             tt.unversion_file(trans_id)
-                            tt.version_file(new_trans_id, file_id=file_id)
+                            tt.version_file(
+                                new_trans_id, file_id=getattr(change, 'file_id', None))
                         # New contents should have the same unix perms as old
                         # contents
                         mode_id = trans_id
@@ -1065,7 +1068,7 @@
                         target_path), trans_id)
                 elif target_kind == 'file':
                     deferred_files.append(
-                        (target_path, (trans_id, mode_id, file_id)))
+                        (target_path, (trans_id, mode_id, target_path)))
                     if basis_tree is None:
                         basis_tree = working_tree.basis_tree()
                         basis_tree.lock_read()
@@ -1087,35 +1090,35 @@
                 elif target_kind is not None:
                     raise AssertionError(target_kind)
             if not wt_versioned and target_versioned:
-                tt.version_file(trans_id, file_id=file_id)
+                tt.version_file(
+                    trans_id, file_id=getattr(change, 'file_id', None))
             if wt_versioned and not target_versioned:
                 tt.unversion_file(trans_id)
             if (target_name is not None
-                    and (wt_name != target_name or wt_parent != target_parent)):
-                if target_name == '' and target_parent is None:
+                    and (wt_name != target_name or change.is_reparented())):
+                if target_path == '':
                     parent_trans = ROOT_PARENT
                 else:
                     parent_trans = tt.trans_id_file_id(target_parent)
-                if wt_parent is None and wt_versioned:
+                if wt_path == '' and wt_versioned:
                     tt.adjust_root_path(target_name, parent_trans)
                 else:
                     tt.adjust_path(target_name, parent_trans, trans_id)
             if wt_executable != target_executable and target_kind == "file":
                 tt.set_executability(target_executable, trans_id)
         if working_tree.supports_content_filtering():
-            for (trans_id, mode_id, file_id), bytes in (
+            for (trans_id, mode_id, target_path), bytes in (
                     target_tree.iter_files_bytes(deferred_files)):
                 # We're reverting a tree to the target tree so using the
                 # target tree to find the file path seems the best choice
                 # here IMO - Ian C 27/Oct/2009
-                filter_tree_path = target_tree.id2path(file_id)
-                filters = working_tree._content_filter_stack(filter_tree_path)
+                filters = working_tree._content_filter_stack(target_path)
                 bytes = filtered_output_bytes(
                     bytes, filters,
-                    ContentFilterContext(filter_tree_path, working_tree))
+                    ContentFilterContext(target_path, working_tree))
                 tt.create_file(bytes, trans_id, mode_id)
         else:
-            for (trans_id, mode_id, file_id), bytes in target_tree.iter_files_bytes(
+            for (trans_id, mode_id, target_path), bytes in target_tree.iter_files_bytes(
                     deferred_files):
                 tt.create_file(bytes, trans_id, mode_id)
         tt.fixup_new_roots()
@@ -1133,13 +1136,128 @@
     with ui.ui_factory.nested_progress_bar() as pb:
         for n in range(10):
             pb.update(gettext('Resolution pass'), n + 1, 10)
-            conflicts = tt.find_conflicts()
+            conflicts = tt.find_raw_conflicts()
             if len(conflicts) == 0:
                 return new_conflicts
             new_conflicts.update(pass_func(tt, conflicts))
         raise MalformedTransform(conflicts=conflicts)
 
 
+def resolve_duplicate_id(tt, path_tree, c_type, old_trans_id, trans_id):
+    tt.unversion_file(old_trans_id)
+    yield (c_type, 'Unversioned existing file', old_trans_id, trans_id)
+
+
+def resolve_duplicate(tt, path_tree, c_type, last_trans_id, trans_id, name):
+    # files that were renamed take precedence
+    final_parent = tt.final_parent(last_trans_id)
+    if tt.path_changed(last_trans_id):
+        existing_file, new_file = trans_id, last_trans_id
+    else:
+        existing_file, new_file = last_trans_id, trans_id
+    new_name = tt.final_name(existing_file) + '.moved'
+    tt.adjust_path(new_name, final_parent, existing_file)
+    yield (c_type, 'Moved existing file to', existing_file, new_file)
+
+
+def resolve_parent_loop(tt, path_tree, c_type, cur):
+    # break the loop by undoing one of the ops that caused the loop
+    while not tt.path_changed(cur):
+        cur = tt.final_parent(cur)
+    yield (c_type, 'Cancelled move', cur, tt.final_parent(cur),)
+    tt.adjust_path(tt.final_name(cur), tt.get_tree_parent(cur), cur)
+
+
+def resolve_missing_parent(tt, path_tree, c_type, trans_id):
+    if trans_id in tt._removed_contents:
+        cancel_deletion = True
+        orphans = tt._get_potential_orphans(trans_id)
+        if orphans:
+            cancel_deletion = False
+            # All children are orphans
+            for o in orphans:
+                try:
+                    tt.new_orphan(o, trans_id)
+                except OrphaningError:
+                    # Something bad happened so we cancel the directory
+                    # deletion which will leave it in place with a
+                    # conflict. The user can deal with it from there.
+                    # Note that this also catch the case where we don't
+                    # want to create orphans and leave the directory in
+                    # place.
+                    cancel_deletion = True
+                    break
+        if cancel_deletion:
+            # Cancel the directory deletion
+            tt.cancel_deletion(trans_id)
+            yield ('deleting parent', 'Not deleting', trans_id)
+    else:
+        create = True
+        try:
+            tt.final_name(trans_id)
+        except NoFinalPath:
+            if path_tree is not None:
+                file_id = tt.final_file_id(trans_id)
+                if file_id is None:
+                    file_id = tt.inactive_file_id(trans_id)
+                _, entry = next(path_tree.iter_entries_by_dir(
+                    specific_files=[path_tree.id2path(file_id)]))
+                # special-case the other tree root (move its
+                # children to current root)
+                if entry.parent_id is None:
+                    create = False
+                    moved = _reparent_transform_children(
+                        tt, trans_id, tt.root)
+                    for child in moved:
+                        yield (c_type, 'Moved to root', child)
+                else:
+                    parent_trans_id = tt.trans_id_file_id(
+                        entry.parent_id)
+                    tt.adjust_path(entry.name, parent_trans_id,
+                                   trans_id)
+        if create:
+            tt.create_directory(trans_id)
+            yield (c_type, 'Created directory', trans_id)
+
+
+def resolve_unversioned_parent(tt, path_tree, c_type, trans_id):
+    file_id = tt.inactive_file_id(trans_id)
+    # special-case the other tree root (move its children instead)
+    if path_tree and path_tree.path2id('') == file_id:
+        # This is the root entry, skip it
+        return
+    tt.version_file(trans_id, file_id=file_id)
+    yield (c_type, 'Versioned directory', trans_id)
+
+
+def resolve_non_directory_parent(tt, path_tree, c_type, parent_id):
+    parent_parent = tt.final_parent(parent_id)
+    parent_name = tt.final_name(parent_id)
+    parent_file_id = tt.final_file_id(parent_id)
+    new_parent_id = tt.new_directory(parent_name + '.new',
+                                     parent_parent, parent_file_id)
+    _reparent_transform_children(tt, parent_id, new_parent_id)
+    if parent_file_id is not None:
+        tt.unversion_file(parent_id)
+    yield (c_type, 'Created directory', new_parent_id)
+
+
+def resolve_versioning_no_contents(tt, path_tree, c_type, trans_id):
+    tt.cancel_versioning(trans_id)
+    return []
+
+
+CONFLICT_RESOLVERS = {
+    'duplicate id': resolve_duplicate_id,
+    'duplicate': resolve_duplicate,
+    'parent loop': resolve_parent_loop,
+    'missing parent': resolve_missing_parent,
+    'unversioned parent': resolve_unversioned_parent,
+    'non-directory parent': resolve_non_directory_parent,
+    'versioning no contents': resolve_versioning_no_contents,
+}
+
+
 def conflict_pass(tt, conflicts, path_tree=None):
     """Resolve some classes of conflicts.
 
@@ -1148,135 +1266,14 @@
     :param path_tree: A Tree to get supplemental paths from
     """
     new_conflicts = set()
-    for c_type, conflict in ((c[0], c) for c in conflicts):
-        if c_type == 'duplicate id':
-            tt.unversion_file(conflict[1])
-            new_conflicts.add((c_type, 'Unversioned existing file',
-                               conflict[1], conflict[2], ))
-        elif c_type == 'duplicate':
-            # files that were renamed take precedence
-            final_parent = tt.final_parent(conflict[1])
-            if tt.path_changed(conflict[1]):
-                existing_file, new_file = conflict[2], conflict[1]
-            else:
-                existing_file, new_file = conflict[1], conflict[2]
-            new_name = tt.final_name(existing_file) + '.moved'
-            tt.adjust_path(new_name, final_parent, existing_file)
-            new_conflicts.add((c_type, 'Moved existing file to',
-                               existing_file, new_file))
-        elif c_type == 'parent loop':
-            # break the loop by undoing one of the ops that caused the loop
-            cur = conflict[1]
-            while not tt.path_changed(cur):
-                cur = tt.final_parent(cur)
-            new_conflicts.add((c_type, 'Cancelled move', cur,
-                               tt.final_parent(cur),))
-            tt.adjust_path(tt.final_name(cur), tt.get_tree_parent(cur), cur)
-
-        elif c_type == 'missing parent':
-            trans_id = conflict[1]
-            if trans_id in tt._removed_contents:
-                cancel_deletion = True
-                orphans = tt._get_potential_orphans(trans_id)
-                if orphans:
-                    cancel_deletion = False
-                    # All children are orphans
-                    for o in orphans:
-                        try:
-                            tt.new_orphan(o, trans_id)
-                        except OrphaningError:
-                            # Something bad happened so we cancel the directory
-                            # deletion which will leave it in place with a
-                            # conflict. The user can deal with it from there.
-                            # Note that this also catch the case where we don't
-                            # want to create orphans and leave the directory in
-                            # place.
-                            cancel_deletion = True
-                            break
-                if cancel_deletion:
-                    # Cancel the directory deletion
-                    tt.cancel_deletion(trans_id)
-                    new_conflicts.add(('deleting parent', 'Not deleting',
-                                       trans_id))
-            else:
-                create = True
-                try:
-                    tt.final_name(trans_id)
-                except NoFinalPath:
-                    if path_tree is not None:
-                        file_id = tt.final_file_id(trans_id)
-                        if file_id is None:
-                            file_id = tt.inactive_file_id(trans_id)
-                        _, entry = next(path_tree.iter_entries_by_dir(
-                            specific_files=[path_tree.id2path(file_id)]))
-                        # special-case the other tree root (move its
-                        # children to current root)
-                        if entry.parent_id is None:
-                            create = False
-                            moved = _reparent_transform_children(
-                                tt, trans_id, tt.root)
-                            for child in moved:
-                                new_conflicts.add((c_type, 'Moved to root',
-                                                   child))
-                        else:
-                            parent_trans_id = tt.trans_id_file_id(
-                                entry.parent_id)
-                            tt.adjust_path(entry.name, parent_trans_id,
-                                           trans_id)
-                if create:
-                    tt.create_directory(trans_id)
-                    new_conflicts.add((c_type, 'Created directory', trans_id))
-        elif c_type == 'unversioned parent':
-            file_id = tt.inactive_file_id(conflict[1])
-            # special-case the other tree root (move its children instead)
-            if path_tree and path_tree.path2id('') == file_id:
-                # This is the root entry, skip it
-                continue
-            tt.version_file(conflict[1], file_id=file_id)
-            new_conflicts.add((c_type, 'Versioned directory', conflict[1]))
-        elif c_type == 'non-directory parent':
-            parent_id = conflict[1]
-            parent_parent = tt.final_parent(parent_id)
-            parent_name = tt.final_name(parent_id)
-            parent_file_id = tt.final_file_id(parent_id)
-            new_parent_id = tt.new_directory(parent_name + '.new',
-                                             parent_parent, parent_file_id)
-            _reparent_transform_children(tt, parent_id, new_parent_id)
-            if parent_file_id is not None:
-                tt.unversion_file(parent_id)
-            new_conflicts.add((c_type, 'Created directory', new_parent_id))
-        elif c_type == 'versioning no contents':
-            tt.cancel_versioning(conflict[1])
+    for conflict in conflicts:
+        resolver = CONFLICT_RESOLVERS.get(conflict[0])
+        if resolver is None:
+            continue
+        new_conflicts.update(resolver(tt, path_tree, *conflict))
     return new_conflicts
 
 
-def cook_conflicts(raw_conflicts, tt):
-    """Generate a list of cooked conflicts, sorted by file path"""
-    conflict_iter = iter_cook_conflicts(raw_conflicts, tt)
-    return sorted(conflict_iter, key=conflicts.Conflict.sort_key)
-
-
-def iter_cook_conflicts(raw_conflicts, tt):
-    fp = FinalPaths(tt)
-    for conflict in raw_conflicts:
-        c_type = conflict[0]
-        action = conflict[1]
-        modified_path = fp.get_path(conflict[2])
-        modified_id = tt.final_file_id(conflict[2])
-        if len(conflict) == 3:
-            yield conflicts.Conflict.factory(
-                c_type, action=action, path=modified_path, file_id=modified_id)
-
-        else:
-            conflicting_path = fp.get_path(conflict[3])
-            conflicting_id = tt.final_file_id(conflict[3])
-            yield conflicts.Conflict.factory(
-                c_type, action=action, path=modified_path,
-                file_id=modified_id,
-                conflict_path=conflicting_path,
-                conflict_file_id=conflicting_id)
-
-
 class _FileMover(object):
     """Moves and deletes files for TreeTransform, tracking operations"""
 

=== modified file 'breezy/transport/http/__init__.py'
--- breezy/transport/http/__init__.py	2020-07-18 23:14:00 +0000
+++ breezy/transport/http/__init__.py	2020-08-10 15:00:17 +0000
@@ -1201,7 +1201,8 @@
         if not server_headers:
             # The http error MUST have the associated
             # header. This must never happen in production code.
-            raise KeyError('%s not found' % self.auth_required_header)
+            trace.mutter('%s not found', self.auth_required_header)
+            return None
 
         auth = self.get_auth(request)
         auth['modified'] = False
@@ -1700,6 +1701,7 @@
                        403,
                        404,  # Not found
                        405,  # Method not allowed
+                       406,  # Not Acceptable
                        409,  # Conflict
                        416,  # Range not satisfiable
                        422,  # Unprocessible entity
@@ -1731,9 +1733,9 @@
                 'Server refuses to fulfill the request (403 Forbidden)'
                 ' for %s' % req.get_full_url())
         else:
-            raise errors.InvalidHttpResponse(req.get_full_url(),
-                                             'Unable to handle http code %d: %s'
-                                             % (code, msg))
+            raise errors.UnexpectedHttpStatus(
+                req.get_full_url(), code,
+                'Unable to handle http code: %s' % msg)
 
 
 class Opener(object):

=== modified file 'breezy/tree.py'
--- breezy/tree.py	2020-07-18 23:14:00 +0000
+++ breezy/tree.py	2020-08-10 15:00:17 +0000
@@ -17,11 +17,6 @@
 """Tree classes, representing directory at point in time.
 """
 
-from .lazy_import import lazy_import
-lazy_import(globals(), """
-from breezy.i18n import gettext
-""")
-
 from . import (
     errors,
     lock,
@@ -122,16 +117,14 @@
 class TreeChange(object):
     """Describes the changes between the same item in two different trees."""
 
-    __slots__ = ['file_id', 'path', 'changed_content', 'versioned', 'parent_id',
+    __slots__ = ['path', 'changed_content', 'versioned',
                  'name', 'kind', 'executable', 'copied']
 
-    def __init__(self, file_id, path, changed_content, versioned, parent_id,
+    def __init__(self, path, changed_content, versioned,
                  name, kind, executable, copied=False):
-        self.file_id = file_id
         self.path = path
         self.changed_content = changed_content
         self.versioned = versioned
-        self.parent_id = parent_id
         self.name = name
         self.kind = kind
         self.executable = executable
@@ -140,12 +133,9 @@
     def __repr__(self):
         return "%s%r" % (self.__class__.__name__, self._as_tuple())
 
-    def __len__(self):
-        return len(self.__slots__)
-
     def _as_tuple(self):
-        return (self.file_id, self.path, self.changed_content, self.versioned,
-                self.parent_id, self.name, self.kind, self.executable, self.copied)
+        return (self.path, self.changed_content, self.versioned,
+                self.name, self.kind, self.executable, self.copied)
 
     def __eq__(self, other):
         if isinstance(other, TreeChange):
@@ -163,12 +153,12 @@
         return False
 
     def is_reparented(self):
-        return self.parent_id[0] != self.parent_id[1]
+        return os.path.dirname(self.path[0]) != os.path.dirname(self.path[1])
 
     def discard_new(self):
         return self.__class__(
-            self.file_id, (self.path[0], None), self.changed_content,
-            (self.versioned[0], None), (self.parent_id[0], None),
+            (self.path[0], None), self.changed_content,
+            (self.versioned[0], None),
             (self.name[0], None), (self.kind[0], None),
             (self.executable[0], None),
             copied=False)
@@ -637,12 +627,10 @@
         list to exclude some directories, they are then not descended into.
 
         The data yielded is of the form:
-        ((directory-relpath, directory-path-from-root, directory-fileid),
-        [(relpath, basename, kind, lstat, path_from_tree_root, file_id,
+        (directory-relpath,
+        [(relpath, basename, kind, lstat, path_from_tree_root,
           versioned_kind), ...]),
-         - directory-relpath is the containing dirs relpath from prefix
          - directory-path-from-root is the containing dirs path from /
-         - directory-fileid is the id of the directory if it is versioned.
          - relpath is the relative path within the subtree being walked.
          - basename is the basename
          - kind is the kind of the file now. If unknonwn then the file is not
@@ -650,7 +638,6 @@
            versioned_kind.
          - lstat is the stat data *if* the file was statted.
          - path_from_tree_root is the path from the root of the tree.
-         - file_id is the file_id if the entry is versioned.
          - versioned_kind is the kind of the file as last recorded in the
            versioning system. If 'unknown' the file is not versioned.
         One of 'kind' and 'versioned_kind' must not be 'unknown'.
@@ -683,8 +670,7 @@
         prefs = next(self.iter_search_rules([path], filter_pref_names))
         stk = filters._get_filter_stack_for(prefs)
         if 'filters' in debug.debug_flags:
-            trace.note(
-                gettext("*** {0} content-filter: {1} => {2!r}").format(path, prefs, stk))
+            trace.note("*** {0} content-filter: {1} => {2!r}").format(path, prefs, stk)
         return stk
 
     def _content_filter_stack_provider(self):

=== modified file 'breezy/upstream_import.py'
--- breezy/upstream_import.py	2020-07-18 23:14:00 +0000
+++ breezy/upstream_import.py	2020-08-10 15:00:17 +0000
@@ -33,7 +33,7 @@
 from .osutils import (pathjoin, isdir, file_iterator, basename,
                       file_kind, splitpath)
 from .trace import warning
-from .transform import resolve_conflicts, cook_conflicts
+from .transform import resolve_conflicts
 from .transport import get_transport
 from .workingtree import WorkingTree
 
@@ -300,7 +300,7 @@
     for path in removed.difference(added):
         tt.unversion_file(tt.trans_id_tree_path(path))
 
-    for conflict in cook_conflicts(resolve_conflicts(tt), tt):
+    for conflict in tt.cook_conflicts(resolve_conflicts(tt)):
         warning(conflict)
 
 

=== modified file 'breezy/workingtree.py'
--- breezy/workingtree.py	2020-07-18 23:14:00 +0000
+++ breezy/workingtree.py	2020-08-10 15:00:17 +0000
@@ -1219,8 +1219,8 @@
         """Walk the directories of this tree.
 
         returns a generator which yields items in the form:
-                ((curren_directory_path, fileid),
-                 [(file1_path, file1_name, file1_kind, (lstat), file1_id,
+                (current_directory_path,
+                 [(file1_path, file1_name, file1_kind, (lstat),
                    file1_kind), ... ])
 
         This API returns a generator, which is only valid during the current
@@ -1240,11 +1240,11 @@
         into files that have text conflicts.  The corresponding .THIS .BASE and
         .OTHER files are deleted, as per 'resolve'.
 
-        :return: a tuple of ConflictLists: (un_resolved, resolved).
+        :return: a tuple of lists: (un_resolved, resolved).
         """
         with self.lock_tree_write():
-            un_resolved = _mod_conflicts.ConflictList()
-            resolved = _mod_conflicts.ConflictList()
+            un_resolved = []
+            resolved = []
             for conflict in self.conflicts():
                 try:
                     conflict.action_auto(self)

=== modified file 'breezy/workspace.py'
--- breezy/workspace.py	2020-07-27 22:52:21 +0000
+++ breezy/workspace.py	2020-08-06 22:27:46 +0000
@@ -37,8 +37,8 @@
 class WorkspaceDirty(BzrError):
     _fmt = "The directory %(path)s has pending changes."
 
-    def __init__(self, tree):
-        BzrError(self, path=tree.abspath('.'))
+    def __init__(self, path):
+        BzrError.__init__(self, path=path)
 
 
 # TODO(jelmer): Move to .clean_tree?
@@ -69,12 +69,12 @@
     """
     # Just check there are no changes to begin with
     if local_tree.has_changes():
-        raise WorkspaceDirty(local_tree)
+        raise WorkspaceDirty(local_tree.abspath('.'))
     if list(local_tree.unknowns()):
-        raise WorkspaceDirty(local_tree)
-
-
-def delete_items(deletables, dry_run: bool = False):
+        raise WorkspaceDirty(local_tree.abspath('.'))
+
+
+def delete_items(deletables, dry_run=False):
     """Delete files in the deletables iterable"""
     def onerror(function, path, excinfo):
         """Show warning for errors seen by rmtree.

=== modified file 'doc/en/release-notes/brz-3.1.txt'
--- doc/en/release-notes/brz-3.1.txt	2020-07-28 01:28:20 +0000
+++ doc/en/release-notes/brz-3.1.txt	2020-08-06 22:01:46 +0000
@@ -66,6 +66,10 @@
  * Prevent exceptions during commit when quilt is not installed but
    the tree contains quilt patches. (Jelmer Vernooĳ)
 
+ * Support pushing to Git repositories that do not yet exist,
+   such as is the case on Launchpad repositories.
+   (Jelmer Vernooĳ)
+
 Documentation
 *************
 
@@ -83,6 +87,9 @@
  * The ``osutils.safe_file_id`` and ``osutils.safe_revision_id`` methods
    have been removed. (Jelmer Vernooĳ)
 
+ * File ids are no longer returned in ``Tree.walkdirs``.
+   (Jelmer Vernooĳ)
+
 Internals
 *********
 

